/*******************************************************************************
***
*** Filename         : composer.c
*** Purpose          : PsiStack Email Composer
*** Author           : Matt J. Gumbley
*** Created          : 01/08/98 
*** Last updated     : 01/08/98
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#include <p_std.h>          /* basic Psion header file */
#include <p_file.h>         /* defines E_FILE_ error values, etc */
#include <p_dbf.h>          /* defines DbfXXX functions */
#include <p_date.h>         /* defines date and time functions */
#include <p_config.h>       /* defines E_CONFIG struct */
#include <p_math.h>         /* defines FP functions */
#include <epoc.h>           /* general Epoc definitions */
#include <wlib.h>           /* defines al Window Server library functions */
#include <hwif.h>           /* defines all Hwif library functions */

/* PsiStack Utility Library */
#include "..\..\library\utils\version.h"                 /* Machine detection */
#include "..\..\library\utils\machine.h"                 /* Machine detection */

/* Table of Contents structures */
#include "toc.h"

GLREF_D UWORD _UseFullScreen, _SmallFontDialog;
GLDEF_D UWORD usesmalldialogs; /* Copied to _SmallFontDialog in myuOpenDialog */
GLREF_D VOID *winHandle;

#define MENU_MSG_SEND        0

#define MENU_EDIT_COPY       1
#define MENU_EDIT_INSERT     2
#define MENU_EDIT_BRING      3

#define MENU_PREFS           4
#define MENU_ZOOM_IN         5
#define MENU_ZOOM_OUT        6
#define MENU_EXIT            7
#define MENU_VIEW_HEADERS    8
#define MENU_VIEW_MESSAGE    9

/* The commands in the main (top-level) menu bar.
   Keyboard accelerators are mostly compatible with Eudora */
LOCAL_D TEXT *main_cmds[]=
/* abcdefghijklmnopqrstuvwxyz ABCDEFGHIJKLMNOPQRSTUVWXYZ 
    #   #     ##### # #   # #      #    #   #  #       #
*/
{
  /* Message */
  "eQueue message",

  /* Edit */
  "cCopy",
  "iInsert",
  "bBring",

  /* Special */
  "qSet preferences",
  "zZoom in",
  "ZZoom out",
  "XExit, lose changes",

  /* Diamond */
  "hHeaders",
  "mMessage",
  NULL
};

#define MODE_NEXT 99
#define MODE_PREV 98

/* Mode list names */
#define MODE_HEADERS   0
#define MODE_MESSAGE   1

#define MODES 2
 
/* How many items in each menu */
#define MSG_ITEMS            1
#define EDIT_ITEMS           3
#define SPECIAL_ITEMS        4
#define DIAMOND_ITEMS        2

/* The menus in the main menu bar */
LOCAL_D H_MENU_DATA main_mdata[]=
{
  "Message", MSG_ITEMS,
  "Edit",    EDIT_ITEMS,
  "Special", SPECIAL_ITEMS,
  "\004",    DIAMOND_ITEMS,
  NULL
};

GLDEF_D TEXT ** _cmds=(&main_cmds[0]);           /* Hwif code expects this */
GLDEF_D H_MENU_DATA * _mdata=(&main_mdata[0]);   /* Hwif code expects this */
/* ^^^ To provide multiple menu palettes change these before calling 
       uPresentMenus(), see page 12 of "Programming in HWIF" */

GLREF_D UWORD *_MenuPositions;
GLREF_D UWORD *_GreyLines;
LOCAL_D UWORD lines[16];


LOCAL_D UINT MainWid;       /* ID of main window */

/* Keyboard stuff */
LOCAL_D WORD key_stat;
LOCAL_D WMSG_KEY key;
LOCAL_D WORD key_active=FALSE;

LOCAL_D P_SCR_SET_FONT Font;        /* Font used by the console */
LOCAL_D UWORD screenwidth; /* current width of main display area taking into 
                               account the status window */

LOCAL_D UWORD lastbflags=0;   /* which kind of Border flags */
LOCAL_D INT text_height=0; /* Height of the current font */
LOCAL_D INT text_width=0; /* Width of the current font */
LOCAL_D INT text_ascent=0; /* Ascent of the current font */
LOCAL_D INT text_descent=0; /* Descent of the current font */
LOCAL_D INT rows=0; /* Number of displyable rows in this font */
#define FIELDSEP 10
#define VERTICAL_LEADING 1


/* Config filename */
LOCAL_D TEXT config_fullname[P_FNAMESIZE]="LOC::M:\\APP\\PSISTACK\\COMPOSER.CFG";

LOCAL_D TEXT *szYes = "Yes";
LOCAL_D TEXT *szNo = "No";
LOCAL_D TEXT *szOk = "Ok";
LOCAL_D TEXT *szUnimplemented = "Not implemented yet";


/* Fields provided */
#define FIELD_TO         0
#define FIELD_SUBJECT    1
#define FIELD_FROM       2
#define FIELD_CC         3
#define FIELD_BCC        4
#define FIELD_ATTACH     5
#define FIELD_DATE       6
#define FIELD_PRIORITY   7
#define FIELD_RECEIPT    8
#define FIELD_ENCODING   9
#define FIELD_MESSAGE    10

#define FIELDS           11

#define FIELD_TO_LEN 80
#define FIELD_SUBJECT_LEN 80
#define FIELD_FROM_LEN 80
#define FIELD_CC_LEN 80
#define FIELD_BCC_LEN 80
#define FIELD_ATTACH_LEN 80
#define FIELD_DATE_LEN 30
#define FIELD_PRIORITY_LEN 10
#define FIELD_RECEIPT_LEN 5
#define FIELD_ENCODING_LEN 15
#define FIELD_MESSAGE_LEN 4096

LOCAL_D TEXT field_to[FIELD_TO_LEN];
LOCAL_D TEXT field_subject[FIELD_SUBJECT_LEN];
LOCAL_D TEXT field_from[FIELD_FROM_LEN];
LOCAL_D TEXT field_cc[FIELD_CC_LEN];
LOCAL_D TEXT field_bcc[FIELD_BCC_LEN];
LOCAL_D TEXT field_attach[FIELD_ATTACH_LEN];
LOCAL_D TEXT field_date[FIELD_DATE_LEN];
LOCAL_D TEXT field_priority[FIELD_PRIORITY_LEN];
LOCAL_D TEXT field_receipt[FIELD_RECEIPT_LEN];
LOCAL_D TEXT field_encoding[FIELD_ENCODING_LEN];
LOCAL_D TEXT field_message[FIELD_MESSAGE_LEN];

GLREF_C VOID field_to_handler(VOID);
GLREF_C VOID field_subject_handler(VOID);
GLREF_C VOID field_from_handler(VOID);
GLREF_C VOID field_cc_handler(VOID);
GLREF_C VOID field_bcc_handler(VOID);
GLREF_C VOID field_attach_handler(VOID);
GLREF_C VOID field_date_handler(VOID);
GLREF_C VOID field_priority_handler(VOID);
GLREF_C VOID field_receipt_handler(VOID);
GLREF_C VOID field_encoding_handler(VOID);
GLREF_C VOID field_message_handler(VOID);

#define EDITLEN 132

/* Field management structure */
struct field {
  TEXT *name;
  UWORD namelen;
  UWORD length;
  TEXT *data;
  UWORD selpos;
  UWORD curpos;
  VOID (*keyhandler)(VOID);
  VOID *hebH;
  H_EDIT_BOX heb;
} fields[FIELDS] = {
  {
    "To:", 0,
    FIELD_TO_LEN,
    field_to,
    0, 0,
    field_to_handler,
    NULL
  },
  {
    "Subject:", 0,
    FIELD_SUBJECT_LEN,
    field_subject,
    0, 0,
    field_subject_handler,
    NULL
  },
  {
    "From:", 0,
    FIELD_FROM_LEN,
    field_from,
    0, 0,
    field_from_handler,
    NULL
  },
  {
    "Cc:", 0,
    FIELD_CC_LEN,
    field_cc,
    0, 0,
    field_cc_handler,
    NULL
  },
  {
    "Bcc:", 0,
    FIELD_BCC_LEN,
    field_bcc,
    0, 0,
    field_bcc_handler,
    NULL
  },
  {
    "Attach:", 0,
    FIELD_ATTACH_LEN,
    field_attach,
    0, 0,
    field_attach_handler,
    NULL
  },
  {
    "Date:", 0,
    FIELD_DATE_LEN,
    field_date,
    0, 0,
    field_date_handler,
    NULL
  },
  {
    "Priority:", 0,
    FIELD_PRIORITY_LEN,
    field_priority,
    0, 0,
    field_priority_handler,
    NULL
  },
  {
    "Receipt:", 0,
    FIELD_RECEIPT_LEN,
    field_receipt,
    0, 0,
    field_receipt_handler,
    NULL
  },
  {
    "Encoding:", 0,
    FIELD_ENCODING_LEN,
    field_encoding,
    0, 0,
    field_encoding_handler,
    NULL
  },
  {
    NULL, 0,
    FIELD_MESSAGE_LEN,
    field_message,
    0, 0,
    field_message_handler,
    NULL
  }
};

#define RECEIPTS 2
#define RECEIPT_NO 0
#define RECEIPT_YES 1
LOCAL_D TEXT *receipts[RECEIPTS] = {
  "No",
  "Yes",
};

#define PRIORITIES 5
#define PRIORITY_LOWEST 0
#define PRIORITY_LOW 1
#define PRIORITY_NORMAL 2
#define PRIORITY_HIGH 3
#define PRIORITY_HIGHEST 4
LOCAL_D TEXT *priorities[PRIORITIES] = {
  "Lowest",
  "Low",
  "Normal",
  "High",
  "Highest",
};

#define ENCODINGS 4
#define ENCODING_MIME 0
#define ENCODING_RADIX64 1
#define ENCODING_UUENCODE 2
#define ENCODING_BINHEX 3
LOCAL_D TEXT *encodings[ENCODINGS] = {
  "MIME",
  "RADIX-64",
  "UUENCODE",
  "BINHEX",
};


LOCAL_D INT margin; /* X position of fields, determined by width of largest 
                       label */
LOCAL_D VOID *mebH;         /* The message editor */
LOCAL_D H_EDIT_BOX meb;

LOCAL_D INT currmode=0;
LOCAL_D INT currfield=0;
LOCAL_D INT headercurrfield=0; /* remembered when switching to body */
LOCAL_D INT currencoding, currpriority, currreceipt;
LOCAL_D INT changed=0;

/******************************************************************************/
/* The configuration structures */

/* Font details. Can use either Roman or Swiss fonts. The current font is
   specified as being between "0" and "4". This gets added to the font base,
   which is WS_FONT_BASE+4 for Roman and WS_FONT_BASE+8 for Swiss. The
   resulting number is then used as a Font.id.
   The font index and font base are stored in the config structure.
*/
#define ROMAN (WS_FONT_BASE+4)
#define SWISS (WS_FONT_BASE+8)

/* The master configuration structure - this gets saved and loaded */
struct config {
  UINT version;
  UINT releaselevel;
  UINT currfontidx; /* 0..4 */
  UINT currfontbase;
  BYTE displayfield[FIELDS];
  /* Size of status window; whether Status Window is showing. See WSERV
     manual, p83 */
  UWORD statwinstate;
  UWORD defencoding;
} config;

/* The displayed names of the mailboxes */ 
LOCAL_D TEXT *modelist[MODES] = {
  "Headers",
  "Message",
};

/* FORWARD */
LOCAL_C VOID SwitchFont(INT way);
LOCAL_C VOID field_leave(VOID);
LOCAL_C VOID field_enter(VOID);
LOCAL_C VOID DeleteEditors(VOID);
LOCAL_C VOID CreateEditors(VOID);

GLDEF_C INT myuOpenDialog(TEXT *title)
{
  _SmallFontDialog = usesmalldialogs;
  return uOpenDialog(title);
}

LOCAL_C VOID config_init(VOID)
{
  p_bfil(&config, sizeof(struct config), 0);
  config.version=version;
  config.releaselevel=releaselevel;
  config.currfontidx=1; /* start with 11pt */
  config.currfontbase=SWISS;
  config.defencoding=ENCODING_MIME;
}


LOCAL_C INT Confirm(TEXT *pb)
{
  if (myuOpenDialog(pb))
    return(-1);
  if (uAddButtonList(szNo,-'n',szYes,'y',NULL))
    return(-1);
  return(uRunDialog()!='y');
}


LOCAL_C VOID Ok(TEXT *pb)
{
  if (myuOpenDialog(pb))
    return;
  if (uAddButtonList(szOk,W_KEY_RETURN,NULL))
    return;
  uRunDialog();
}

VOID config_save(VOID)
{
VOID *fd = NULL;
INT errcode;
TEXT buf[50];
  errcode=p_open(&fd,config_fullname,P_FREPLACE|P_FSTREAM|P_FUPDATE);
  if (uErrorValue(errcode)) {
    p_atos(buf, "Couldn't create \"%s\"", config_fullname+21);
    wInfoMsg(buf);
    return;
  }
  p_write(fd,(VOID *)&config,sizeof(struct config));
  p_close(fd);
}

VOID config_load(VOID)
{
VOID *fd = NULL;
TEXT buf[100];
  if (p_open(&fd,config_fullname,P_FOPEN|P_FSTREAM|P_FSHARE) != 0) {
    p_atos(buf,"Creating \"%s\"", config_fullname+21);
    wInfoMsg(buf);
    config_save();
    return;
  }
  p_read(fd,(VOID *)&config,sizeof(struct config));
  if (config.version != version || config.releaselevel != releaselevel) {
    p_atos(buf, "Config file created by v%d.%02d%c\r\ncannot be used, so defaults\r\nwill be used.", config.version / 100, config.version % 100, config.releaselevel);
    Ok(buf);
    p_close(fd);
    config_init();
    config_save();
    return;
  }
  p_close(fd);
}


GLDEF_C VOID Terminate(INT num)
{
  config_save();
  p_exit(num);     /* Goodnight Vienna */
}

LOCAL_C INT ResizeWidth(UWORD width)
{
INT ret;
W_WINDATA wd;

  wd.extent.tl.x=wd.extent.tl.y=0;
  wd.extent.width=width;
  wd.extent.height=machine_fullscreenheight;
  wSetWindow(MainWid,W_WIN_EXTENT,&wd);
  ret=uErrorValue(wCheckPoint());     /* check whether successful */
  if (!ret)
    screenwidth=width;
  return(ret);
}

LOCAL_C VOID SetGmode(INT gmode)
{
G_GC gc;
  gc.gmode=gmode;
  gSetGC(0,G_GC_MASK_GMODE,&gc);
}

LOCAL_C VOID SetBold()
{
G_GC gc;
  gc.style=G_STY_BOLD;
  gSetGC(0,G_GC_MASK_STYLE,&gc);
}

LOCAL_C VOID SetNormal()
{
G_GC gc;
  gc.style=G_STY_NORMAL;
  gSetGC(0,G_GC_MASK_STYLE,&gc);
}

LOCAL_C VOID ClearWholeWindow(VOID)
{
P_RECT whole;

  whole.tl.x=whole.tl.y=0;
  whole.br.x=machine_fullscreenwidth;
  whole.br.y=machine_fullscreenheight;
  gClrRect(&whole,G_TRMODE_CLR);
}


LOCAL_C VOID CycleStatusWindow(VOID)
{
P_EXTENT pextent;
UWORD Newstatwinstate;

  switch (config.statwinstate) {
    case W_STATUS_WINDOW_BIG:
      /* big status window is currently showing -> go to off */
      if (ResizeWidth(machine_fullscreenwidth))
        return;
      config.statwinstate = W_STATUS_WINDOW_OFF;
      wStatusWindow(config.statwinstate);
      break;
    case W_STATUS_WINDOW_OFF:
      /* Status window not currently showing -> go to narrow */
      /* How wide will the status window be? shrink the main window
         by that much. */
      wInquireStatusWindow(W_STATUS_WINDOW_SMALL, &pextent);
      if (ResizeWidth(machine_fullscreenwidth - pextent.width))
        return;
      config.statwinstate = W_STATUS_WINDOW_SMALL;
      wStatusWindow(config.statwinstate);
      break;
    case W_STATUS_WINDOW_SMALL:
      /* Small status window currently showing -> go to off or big
         depending on whether we're running on a Siena or 3A/C */
      /* How wide will the status window be? shrink the main window
         by that much. */
      Newstatwinstate = ((machine_type==MACH_SIENA ||
                          machine_type==MACH_EM_SIENA) ? W_STATUS_WINDOW_OFF :
                                                         W_STATUS_WINDOW_BIG);
      wInquireStatusWindow(Newstatwinstate, &pextent);
      if (ResizeWidth(machine_fullscreenwidth - pextent.width))
        return;
      config.statwinstate = Newstatwinstate;
      wStatusWindow(config.statwinstate);
      break;
  }

  SwitchFont(config.currfontidx); /* cls, recalculate editors, border */
}



LOCAL_C VOID SwitchFont(INT way) 
{
G_FONT_INFO gfi;
G_GC gc;

  switch (way) {
    case MENU_ZOOM_OUT: /* make it smaller */
      config.currfontidx = (config.currfontidx == 0) ? 2 : config.currfontidx-1;
      break;
    case MENU_ZOOM_IN: /* make it bigger */
      config.currfontidx = (config.currfontidx == 2) ? 0 : config.currfontidx+1;
      break;
    default:
      config.currfontidx = way;
      break;
  }

  /* And configure the new font */
  Font.style = 0; /*G_STY_MONO;*/
  Font.id = config.currfontbase + config.currfontidx;

  /* Get font metrics */
  gFontInfo(Font.id, Font.style, &gfi);
  text_height=gfi.ascent+gfi.descent+VERTICAL_LEADING; 
  text_width=gfi.max_width;
  text_ascent=gfi.ascent;
  text_descent=gfi.descent;
  rows = (machine_fullscreenheight - 4) / (text_height + 1);

  /* Set this into the graphics context */
  gc.font=Font.id;
  gc.style=Font.style;
  gSetGC(0,G_GC_MASK_FONT|G_GC_MASK_STYLE,&gc);

  field_leave();
  DeleteEditors();
  ClearWholeWindow();
  gBorder(lastbflags);
  CreateEditors(); 
  gBorder(lastbflags);
  field_enter();
}

LOCAL_C INT GetWidth(TEXT *pb)
{
  return(gTextWidth(Font.id,Font.style,pb,p_slen(pb)));
}

LOCAL_C INT GetWidthBold(TEXT *pb)
{
  return(gTextWidth(Font.id,G_STY_BOLD,pb,p_slen(pb)));
}

LOCAL_C VOID SwitchMode(INT newmode) 
{
  if (newmode == currmode)
    return;

  if (currmode == MODE_HEADERS)
    headercurrfield = currfield;

  field_leave();
  DeleteEditors();

  switch (newmode) {
    case MODE_NEXT:
    case MODE_PREV:
      currmode ^= 0x01;
      break;
    default:
      currmode = newmode;
      break;
  }
  wsSelectList(currmode);

  ClearWholeWindow();
  gBorder(lastbflags);
  CreateEditors(); 
  gBorder(lastbflags);
  if (currmode == MODE_HEADERS)
    currfield = headercurrfield;
  else
    currfield = FIELD_MESSAGE;
  field_enter();
}


LOCAL_C VOID ExitApplication(VOID)
{
  if (changed) {
    if (Confirm("Message modified. Lose changes?"))
      return;
  }
  Terminate(0);
}

LOCAL_C VOID prefs_dialog(VOID)
{
UWORD encoding;
H_DI_CHOICE encch;
INT i;
  encoding = config.defencoding + 1;

  if (myuOpenDialog("Preferences") || uBeginDCL(&encch))
    return;
  for (i=0; i<ENCODINGS; i++) {
    if (uGrowDCL(&encch,encodings[i]))
      return;
  }
  if (uAddDCL("Default Encoding", &encoding, &encch) ||
      uRunDialog()<=0) 
    return;

  /* Copy data back */
  config.defencoding = encoding-1;
}


LOCAL_C VOID ManageCommand(INT index)
{
  switch (index) {
    case MENU_MSG_SEND:
      wInfoMsg(szUnimplemented);
      break;
    case MENU_EDIT_COPY:
    case MENU_EDIT_INSERT:
    case MENU_EDIT_BRING:
      wInfoMsg(szUnimplemented);
      break;
    case MENU_PREFS:
      prefs_dialog();
      break;
    case MENU_ZOOM_IN:
    case MENU_ZOOM_OUT:
      SwitchFont(index);
      lastbflags=0;
      break;
    case MENU_EXIT:
      ExitApplication();
      break;
    case MENU_VIEW_HEADERS:
      SwitchMode(MODE_HEADERS);
      break;
    case MENU_VIEW_MESSAGE:
      SwitchMode(MODE_MESSAGE);
      break;
  }
}

LOCAL_C VOID ProcessSystemCommand(VOID)
{ 
TEXT buf[P_FNAMESIZE+2];

  wGetCommand((UBYTE *)&buf[0]);
  if (buf[0]=='X')
    ExitApplication();
}


LOCAL_C VOID TryExecuteCommand(INT keycode)
{
  keycode=uLocateCommand(keycode);        /* look up accelerator table */
  if (keycode>=0)                 /* found - keycode converted to index */
    ManageCommand(keycode);
}



LOCAL_C VOID field_leave(VOID)
{
register struct field *f = &fields[currfield];
INT len;
  if (f->hebH) {
    hEBEmphasise(f->hebH, FALSE);
    p_scpy(f->data, hEBSenseText(f->hebH));
    len=hEBSenseSelect(f->hebH, &f->curpos);
    f->selpos=f->curpos+len;
  }
}


LOCAL_C VOID field_enter(VOID)
{
register struct field *f = &fields[currfield];
register INT len = p_slen(f->data);
  hEBSetText(f->hebH, f->data, len);
  if (currfield<FIELD_DATE || currfield == FIELD_MESSAGE) 
    hEBSetSelect(f->hebH, f->selpos, f->curpos);
  else {
    hEBSetSelect(f->hebH, 0, len); /* 'select-everything' fields */
  }
  hEBEmphasise(f->hebH, TRUE);
}

LOCAL_C VOID DeleteEditors(VOID)
{
INT i;
  if (currmode == MODE_HEADERS) {
    for (i=0; i<FIELDS-1; i++) 
      if (fields[i].hebH) {
        hEBClose(fields[i].hebH);
        fields[i].hebH=NULL;
      }
  }
  else
    if (fields[FIELD_MESSAGE].hebH) {
      hEBClose(fields[FIELD_MESSAGE].hebH);
      fields[FIELD_MESSAGE].hebH=NULL;
    }
}

LOCAL_C VOID CreateEditors(VOID)
{
INT i,j,w,ew,lasty;

  /* What is the width of the widest label in bold? */
  for (margin=i=0; i<FIELDS; i++) {
    fields[i].namelen=GetWidthBold(fields[i].name);
    if (margin<fields[i].namelen)
      margin=fields[i].namelen;
  }

  SetBold();

  if (currmode == MODE_HEADERS) {
    /* Create the main vertically spaced fields */
    for (i=0; i<FIELD_PRIORITY; i++) {
      fields[i].heb.maxchars=fields[i].length;
      fields[i].heb.vulen=screenwidth - 8 - margin;
      fields[i].heb.vislines = (i==FIELD_SUBJECT) ? 2 : 1;
      fields[i].heb.pos.x=2 + margin;
      fields[i].heb.pos.y=lasty=2 + (i + (i>FIELD_SUBJECT ? 1 : 0)) * 
                         (text_height+VERTICAL_LEADING);
      fields[i].heb.win=MainWid;
      fields[i].heb.font=config.currfontidx + config.currfontbase;
      fields[i].heb.style=G_STY_NORMAL;
      gPrintText(margin - fields[i].namelen,
                 fields[i].heb.pos.y+text_ascent+VERTICAL_LEADING,
                 fields[i].name,p_slen(fields[i].name));
      fields[i].hebH=hEBOpen(H_EDIT_BOX_FONT|H_EDIT_BOX_LEFT_CURSOR|
                             H_EDIT_BOX_VISLINES, &fields[i].heb);
      hEBSetText(fields[i].hebH, fields[i].data, p_slen(fields[i].data));
    }
  
    /* Create the three horizontally spaced fields individually */
    /*i=FIELD_PRIORITY;*/
    for (ew=w=j=0; j<PRIORITIES; j++) {
      w=GetWidth(priorities[j]);
      if (ew<w)
        ew=w;
    }
    ew+=8;
    fields[i].heb.maxchars=fields[i].length;
    fields[i].heb.vulen=ew;
    fields[i].heb.pos.x=2 + margin;
    fields[i].heb.pos.y=lasty=2 + (i + 1) * (text_height+VERTICAL_LEADING);
    fields[i].heb.win=MainWid;
    fields[i].heb.font=config.currfontidx + config.currfontbase;
    fields[i].heb.style=G_STY_NORMAL;
    gPrintText(fields[i].heb.pos.x - 2 - fields[i].namelen,
               fields[i].heb.pos.y+text_ascent+VERTICAL_LEADING,
               fields[i].name,p_slen(fields[i].name));
    fields[i].hebH=hEBOpen(H_EDIT_BOX_FONT|H_EDIT_BOX_LEFT_CURSOR,
                           &fields[i].heb);
    hEBSetText(fields[i].hebH, fields[i].data, p_slen(fields[i].data));
  
    i++; /* =FIELD_RECEIPT; */
    for (ew=w=j=0; j<RECEIPTS; j++) {
      w=GetWidth(receipts[j]);
      if (ew<w)
        ew=w;
    }
    ew+=8;
    fields[i].heb.maxchars=fields[i].length;
    fields[i].heb.vulen=ew;
    fields[i].heb.pos.x=2 + margin + fields[i-1].heb.vulen + margin + FIELDSEP;
    fields[i].heb.pos.y=lasty;
    fields[i].heb.win=MainWid;
    fields[i].heb.font=config.currfontidx + config.currfontbase;
    fields[i].heb.style=G_STY_NORMAL;
    gPrintText(fields[i].heb.pos.x - 2 - fields[i].namelen,
               fields[i].heb.pos.y+text_ascent+VERTICAL_LEADING,
               fields[i].name,p_slen(fields[i].name));
    fields[i].hebH=hEBOpen(H_EDIT_BOX_FONT|H_EDIT_BOX_LEFT_CURSOR,
                           &fields[i].heb);
    hEBSetText(fields[i].hebH, fields[i].data, p_slen(fields[i].data));
  
    i++; /*=FIELD_ENCODING;*/
    for (ew=w=j=0; j<ENCODINGS; j++) {
      w=GetWidth(encodings[j]);
      if (ew<w)
        ew=w;
    }
    ew+=8;
    fields[i].heb.maxchars=fields[i].length;
    fields[i].heb.vulen=ew;
    fields[i].heb.pos.x=2 + margin + 
                        fields[i-2].heb.vulen + margin + FIELDSEP + 
                        fields[i-1].heb.vulen + margin + FIELDSEP;
    fields[i].heb.pos.y=lasty;
    fields[i].heb.win=MainWid;
    fields[i].heb.font=config.currfontidx + config.currfontbase;
    fields[i].heb.style=G_STY_NORMAL;
    gPrintText(fields[i].heb.pos.x - 2 - fields[i].namelen,
               fields[i].heb.pos.y+text_ascent+VERTICAL_LEADING,
               fields[i].name,p_slen(fields[i].name));
    fields[i].hebH=hEBOpen(H_EDIT_BOX_FONT|H_EDIT_BOX_LEFT_CURSOR,
                           &fields[i].heb);
    hEBSetText(fields[i].hebH, fields[i].data, p_slen(fields[i].data));
  }
  else {
    i=FIELD_MESSAGE;
    fields[i].heb.maxchars=fields[i].length;
    fields[i].heb.vulen=screenwidth - 8;
    fields[i].heb.vislines = rows;
    fields[i].heb.pos.x=2;
    fields[i].heb.pos.y=2;
    fields[i].heb.win=MainWid;
    fields[i].heb.font=config.currfontidx + config.currfontbase;
    fields[i].heb.style=G_STY_NORMAL;
    fields[i].hebH=hEBOpen(H_EDIT_BOX_FONT|H_EDIT_BOX_LEFT_CURSOR|
                           H_EDIT_BOX_VISLINES, &fields[i].heb);
    /* field_enter will set the contents and selection */
    /* hEBSetText(fields[i].hebH, fields[i].data, p_slen(fields[i].data));*/
  }
  SetNormal();
}



GLDEF_C VOID field_base_handler(VOID)
{
  switch (key.keycode) {
    case W_KEY_UP:
      field_leave();
      if (currfield==FIELD_TO)
        currfield=FIELD_ENCODING;
      else
        currfield--;
      field_enter();
      break;
    case W_KEY_DOWN:
    case W_KEY_RETURN:
      field_leave();
      if (currfield==FIELD_ENCODING)
        currfield=FIELD_TO;
      else
        currfield++;
      field_enter();
      break;
    case W_KEY_PAGE_UP:
      field_leave();
      currfield=FIELD_TO;
      field_enter();
      break;
    case W_KEY_PAGE_DOWN:
      field_leave();
      currfield=FIELD_ENCODING;
      field_enter();
      break;
    default:
      /* Do something with key.keycode */
      hEBHandleKey(fields[currfield].hebH, key.keycode, key.modifiers);
      changed=1;
      break;
  }
}

GLDEF_C VOID field_to_handler(VOID)
{
  if (key.keycode == W_KEY_TAB) {
    wInfoMsg("address book lookup goes here");
  }
  else
    field_base_handler();
}

GLDEF_C VOID field_subject_handler(VOID)
{
  field_base_handler();
}

GLDEF_C VOID field_from_handler(VOID)
{
  field_base_handler();
}

GLDEF_C VOID field_cc_handler(VOID)
{
  if (key.keycode == W_KEY_TAB) {
    wInfoMsg("address book lookup goes here");
  }
  else
    field_base_handler();
}

GLDEF_C VOID field_bcc_handler(VOID)
{
  if (key.keycode == W_KEY_TAB) {
    wInfoMsg("address book lookup goes here");
  }
  else
    field_base_handler();
}

GLDEF_C VOID field_attach_handler(VOID)
{
  /* Only accept space/tab, which fire up a file selection window, and the 
     usual navigation keys */
  switch (key.keycode) {
    case ' ':
    case W_KEY_TAB:
      wInfoMsg("file selection dialog goes here");
      break;
    case W_KEY_UP:
    case W_KEY_DOWN:
    case W_KEY_PAGE_UP:
    case W_KEY_PAGE_DOWN:
      field_base_handler();
      break;
    default:
      p_sound(2,320);
  }
}

GLDEF_C VOID field_date_handler(VOID)
{
  /* A 'select-everything' field that allows no changes, but permits 
     navigation */
  switch (key.keycode) {
    case W_KEY_UP:
    case W_KEY_DOWN:
    case W_KEY_PAGE_UP:
    case W_KEY_PAGE_DOWN:
      field_base_handler();
      break;
    default:
      p_sound(2,320);
  }
}

GLDEF_C VOID field_priority_handler(VOID)
{
  /* A 'select-everything' field that allows space, tab, left, right to
     choose the priority level, and the navigation keys */
  switch (key.keycode) {
    case W_KEY_LEFT:
      if (currpriority == 0)
        currpriority = PRIORITIES-1;
      else
        currpriority--;
      goto update;
    case ' ':
    case W_KEY_TAB:
    case W_KEY_RIGHT:
      currpriority++;
      if (currpriority == PRIORITIES)
        currpriority = 0;
      goto update;
    case 'h':
      currpriority = PRIORITY_HIGH;
      goto update;
    case 'H':
      currpriority = PRIORITY_HIGHEST;
      goto update;
    case 'l':
      currpriority = PRIORITY_LOW;
      goto update;
    case 'L':
      currpriority = PRIORITY_LOWEST;
      goto update;
    case 'n':
      currpriority = PRIORITY_NORMAL;
      goto update;
    case W_KEY_UP:
    case W_KEY_DOWN:
    case W_KEY_PAGE_UP:
    case W_KEY_PAGE_DOWN:
      field_base_handler();
      break;
    default:
      p_sound(2,320);
  }
  return;
update:
  changed=1;
  field_leave();
  p_scpy(field_priority,priorities[currpriority]);
  field_enter();
}

GLDEF_C VOID field_receipt_handler(VOID)
{
  /* A 'select-everything' field that allows space, tab, left, right to
     choose the receipt status */
  switch (key.keycode) {
    case ' ':
    case W_KEY_TAB:
    case W_KEY_LEFT:
    case W_KEY_RIGHT:
    case 'y':
    case 'n':
      currreceipt ^= 0x01;
      changed=1;
      field_leave();
      p_scpy(field_receipt,receipts[currreceipt]);
      field_enter();
      break;
    case W_KEY_UP:
    case W_KEY_DOWN:
    case W_KEY_PAGE_UP:
    case W_KEY_PAGE_DOWN:
      field_base_handler();
      break;
    default:
      p_sound(2,320);
  }
}

GLDEF_C VOID field_encoding_handler(VOID)
{
  /* A 'select-everything' field that allows space, tab, left, right to
     choose the encoding type */
  switch (key.keycode) {
    case W_KEY_LEFT:
      if (currencoding == 0)
        currencoding = ENCODINGS-1;
      else
        currencoding--;
      goto update;
    case ' ':
    case W_KEY_TAB:
    case W_KEY_RIGHT:
      currencoding++;
      if (currencoding == ENCODINGS)
        currencoding = 0;
      goto update;
    case 'm':
      currencoding = ENCODING_MIME;
      goto update;
    case 'u':
      currencoding = ENCODING_UUENCODE;
      goto update;
    case 'b':
      currencoding = ENCODING_BINHEX;
      goto update;
    case 'r':
      currencoding = ENCODING_RADIX64;
      goto update;
    case W_KEY_UP:
    case W_KEY_DOWN:
    case W_KEY_PAGE_UP:
    case W_KEY_PAGE_DOWN:
      field_base_handler();
      break;
    default:
      p_sound(2,320);
  }
  return;
update:
  changed=1;
  field_leave();
  p_scpy(field_encoding,encodings[currencoding]);
  field_enter();
}

GLDEF_C VOID field_message_handler(VOID)
{
  changed=1;
  hEBHandleKey(fields[currfield].hebH, key.keycode, key.modifiers);
}

LOCAL_C VOID key_process(VOID)
{
INT ret, code;
  if (key.keycode & W_EVENT_KEY) {
    switch (key.keycode) {
      case CONS_EVENT_FOREGROUND:
        break;
      case CONS_EVENT_COMMAND:
        ProcessSystemCommand();
        break;
    }
  }
  else if (key.keycode & W_SPECIAL_KEY) {
    /* Psion key pressed */
    code = key.keycode & (~W_SPECIAL_KEY);
    if (key.modifiers & W_SHIFT_MODIFIER)
      code = p_toupper(code);
    TryExecuteCommand(code); /* Depends which menu is selected */
  }
  else {
    switch (key.keycode) {
      case W_KEY_DIAMOND:
        if (key.modifiers & W_SHIFT_MODIFIER)
          SwitchMode(MODE_PREV);
        else
          SwitchMode(MODE_NEXT);
        break;
      case W_KEY_MENU:
        if (key.modifiers & W_CTRL_MODIFIER) {
          SetGmode(G_TRMODE_CLR);
          gBorder(lastbflags&~(W_BORD_TOP_OFF|W_BORD_BOT_OFF));
          SetGmode(G_TRMODE_SET);
          CycleStatusWindow();
          config_save();
        }
        else {           /* display menu bar */
          /* For more than one menu palette, change _cmds and _mdata first... */

          ret=uPresentMenus();
          if (ret>0)
            TryExecuteCommand(ret); /* And call a different menu handler */
        }
        break;
      default:
        (*fields[currfield].keyhandler)();
    }
  }
}

LOCAL_C VOID MainLoop(VOID)
{
  FOREVER {
    if (key_active)
      wFlush();
    else {
      uGetKeyA(&key_stat,&key);
      key_active=TRUE;
    }

    p_iowait();

    /* Keyboard? */
    if (key_stat != E_FILE_PENDING) {
      key_process();
      key_active = FALSE;
    }
  }
}

LOCAL_C VOID CreateGC(VOID)
{
INT hgc;        /* graphics context handle (if successful) */
  hgc=gCreateGC0(MainWid);
  if (hgc<0)
    Terminate(hgc);
}

LOCAL_C VOID SpecificInit(VOID)
{
LOCAL_D INT gc;
P_EXTENT pextent;

  p_unmarka();
  _GreyLines = &lines[0];
  p_bfil(&lines[0], sizeof(lines), 0);
  uAddGreyUline('q',&lines[0]);
  uAddGreyUline('Z',&lines[0]);


  gc = gCreateGC0(uFindMainWid());
  if (_UseFullScreen) {
    /* Running on a Series 3a or Workabout */
    wFree(gc);
    uEnableGrey();
    gc = gCreateGC0(uFindMainWid());
  }

  /* Which type of machine are we running on? */
  machine_check();
  if (machine_type == MACH_UNKNOWN) {
    wInfoMsg("Unknown machine!");
    Terminate(0); /* probably don't want this here? */
  }

  /* Set initial configuration and limits based on machine type */
  switch (machine_type) {
    case MACH_SIENA:
      config.statwinstate = W_STATUS_WINDOW_SMALL;
      break;
    case MACH_EM_SIENA:
      config.statwinstate = W_STATUS_WINDOW_SMALL;
      break;
    case MACH_S3a:
      config.statwinstate = W_STATUS_WINDOW_BIG;
      break;
    case MACH_EM_S3a:
      config.statwinstate = W_STATUS_WINDOW_BIG;
      break;
    case MACH_S3c:
      config.statwinstate = W_STATUS_WINDOW_BIG;
      break;
    case MACH_S3:
      config.statwinstate = W_STATUS_WINDOW_SMALL;
      break;
    case MACH_MC400:
      config.statwinstate = W_STATUS_WINDOW_BIG;
      break;
    case MACH_MC200:
      config.statwinstate = W_STATUS_WINDOW_BIG;
      break;
    case MACH_HC:
      config.statwinstate = W_STATUS_WINDOW_SMALL;
      break;
    case MACH_WORKABOUT:
      config.statwinstate = W_STATUS_WINDOW_SMALL;
      usesmalldialogs = TRUE;
      _UseFullScreen = TRUE;
      break;
    case MACH_EM_WORKABOUT:
      config.statwinstate = W_STATUS_WINDOW_SMALL;
      usesmalldialogs = TRUE;
      _UseFullScreen = TRUE;
      break;
  }

  uEscape(FALSE);     /* disable Psion-Esc */
  MainWid=uFindMainWid();     /* find id of main display window */
  CreateGC();                 /* create default graphics context */

  /* We could analyse the command line here... */
  hCrackCommandLine();

  wsEnable();                 /* display status window */
  wsSetList(MODES,&modelist[0],currmode);

  /* Initialise everything with blank data */
  config_init();

  /* Find out what size status window we want, font type, font size,
     amongst other stuff */
  config_load();
  currencoding = config.defencoding;
  currreceipt = RECEIPT_YES;
  currpriority = PRIORITY_NORMAL;

  wInquireStatusWindow(config.statwinstate, &pextent);
  if (ResizeWidth(machine_fullscreenwidth - pextent.width))
    Terminate(0);
  wStatusWindow(config.statwinstate);


  /* fill up with some sample data */
  p_scpy(field_to, "Ronald McDonald <imbecile@mcdonalds.com>");
  p_scpy(field_subject, "McLard and fries to go... (hardcoded message)");
  p_scpy(field_from, "Matt J. Gumbley <matt@gumbley.demon.co.uk>");
  p_scpy(field_cc, "PsiStack Mailing List <psion-tcpip@inet.mailserv.alsutton.com>");
  p_scpy(field_bcc, "");
  p_scpy(field_attach, "LOC::M:\\EMAIL3\\PPP.TXT");
  p_now2str(field_date, "%*E, %D %*N %Y %I:%T%A");
  p_scpy(field_priority, priorities[currpriority]);
  p_scpy(field_receipt, receipts[currreceipt]);
  p_scpy(field_encoding, encodings[currencoding]);

  p_scpy(field_message, "\n--\nKind Regards,\n    Matt Gumbley\n    matt@gumbley.demon.co.uk\nthis\nis\na\ntest\nmessage\nand\nthis\nprogram\nis\nnot\nfinished.");
  SwitchFont(config.currfontidx); /* initialises screen metrics */

}

GLDEF_C VOID main(VOID)
{
  _UseFullScreen = TRUE;
  uCommonInit();
  SpecificInit();
  MainLoop();
}
