/*******************************************************************************
***
*** Filename         : email3.c
*** Purpose          : PsiStack Email Client
*** Author           : Matt J. Gumbley
*** Created          : 08/11/97 
*** Last updated     : 27/07/98
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#include <p_std.h>          /* basic Psion header file */
#include <p_file.h>         /* defines E_FILE_ error values, etc */
#include <p_dbf.h>          /* defines DbfXXX functions */
#include <p_date.h>         /* defines date and time functions */
#include <p_config.h>       /* defines E_CONFIG struct */
#include <p_math.h>         /* defines FP functions */

#include <p_sys.h>       /* console */

#include <epoc.h>           /* general Epoc definitions */
#include <wlib.h>           /* defines al Window Server library functions */
#include <hwif.h>           /* defines all Hwif library functions */

/* PsiStack includes */
#include "\sibosdk\include\psistack\stack.h"        /* Kernel API definitions */
#include "\sibosdk\include\psistack\sys\socket.h" /* Sockets structs/#defines */
#include "\sibosdk\include\psistack\socket.h"      /* Sockets API definitions */
#include "\sibosdk\include\psistack\events.h"       /* Kernel API definitions */
#include "\sibosdk\include\psistack\console.h"     /* Console API definitions */

/* PsiStack Utility Library */
#include "..\..\library\utils\machine.h"                 /* Machine detection */
#include "..\..\library\utils\version.h"             /* PsiStack Version Info */

/* Our includes */
#include "toc.h"                                  /* Table of contents format */

GLREF_D UWORD DatLocked;            /* non-zero when application "busy" */
GLREF_D TEXT *DatUsedPathNamePtr;   /* full path name of current file */
GLREF_D UWORD _UseFullScreen, _SmallFontDialog;
GLDEF_D UWORD usesmalldialogs; /* Copied to _SmallFontDialog in myuOpenDialog */
GLREF_D VOID *winHandle;

#define MENU_MSG_NEW         0
#define MENU_MSG_REPLY       1
#define MENU_MSG_REPLYALL    2
#define MENU_MSG_FWD         3
#define MENU_MSG_REDIRECT    4
#define MENU_MSG_REQUEUE     5
#define MENU_MSG_TRANSFER    6

#define MENU_MBX_NEW         7
#define MENU_MBX_RENAME      8
#define MENU_MBX_MOVE        9
#define MENU_MBX_DELETE      10
#define MENU_MBX_COMPRESS    11
#define MENU_MBX_TRASH       12

#define MENU_PREF_PREFS      13
#define MENU_PREF_PERSONS    14
#define MENU_PREF_USER       15
#define MENU_PREF_SERVERS    16
#define MENU_PREF_SIGNATURE  17
#define MENU_PREF_ADDRBOOK   18
#define MENU_PREF_COLUMNS    19

#define MENU_CTRL_SEND       20
#define MENU_CTRL_CHECK      21
#define MENU_CTRL_ONLINE     22
#define MENU_CTRL_OFFLINE    23
#define MENU_ZOOM_IN         24
#define MENU_ZOOM_OUT        25
#define MENU_EXIT            26

#define MENU_VIEW_IN         27
#define MENU_VIEW_OUT        28
#define MENU_VIEW_TRASH      29
#define MENU_VIEW_RXED       30
#define MENU_VIEW_TXED       31

GLREF_D UWORD *_MenuPositions;
GLREF_D UWORD *_GreyLines;
LOCAL_D UWORD lines[16];

/* The commands in the main (top-level) menu bar.
   Keyboard accelerators are mostly compatible with Eudora */
LOCAL_D TEXT *main_cmds[]=
/* abcdefghijklmnopqrstuvwxyz ABCDEFGHIJKLMNOPQRSTUVWXYZ 
    #   #     ##### # #   # #      #    #   #  #       #
*/
{
  /* Message */
  "nNew...",
  "rReply",
  "RReply to All",
  "fForward",
  "FRedirect",
  " Send Again",
  " Transfer to...",

  /* Mailbox */
  " New...",
  " Rename",
  " Move",
  " Delete",
  " Compress",
  " Empty Trash",

  /* Preferences */
  "qSet preferences",
  "ePersonalities",
  "uUser info",
  "sServers info",
  "iSignature",
  "lAddress Book",
  "cColumns",

  /* Control */
  "tSend Mail",
  "mCheck Mail",
  "oGo Online",
  "OGo Offline",
  "zZoom in",
  "ZZoom out",
  "xExit",

  /* Diamond - only shows standard (non-user) mailboxes */
  "1In",
  "0Out",
  " Trash",
  " Rcvd.",
  " Sent",
  NULL
};

#define MODE_NEXT 99
#define MODE_PREV 98

/* Mode list names */
#define MODE_IN 0
#define MODE_OUT 1
#define MODE_TRASH 2
#define MODE_RXED 3
#define MODE_TXED 4
#define MODE_USER 5

#define MODE_MAX 15

/* How many items in each menu */
#define MSG_ITEMS            7
#define MBX_ITEMS            6
#define PREF_ITEMS           7
#define CONTROL_ITEMS        7
#define DIAMOND_ITEMS        5

/* The menus in the main menu bar */
LOCAL_D H_MENU_DATA main_mdata[]=
{
  "Msg", MSG_ITEMS,
  "Folder", MBX_ITEMS,
  "Prefs", PREF_ITEMS,
  "Control", CONTROL_ITEMS,
  "\004",    DIAMOND_ITEMS,
  NULL
};

GLDEF_D TEXT ** _cmds=(&main_cmds[0]);           /* Hwif code expects this */
GLDEF_D H_MENU_DATA * _mdata=(&main_mdata[0]);   /* Hwif code expects this */
/* ^^^ To provide multiple menu palettes change these before calling 
       uPresentMenus(), see page 12 of "Programming in HWIF" */


LOCAL_D UINT MainWid;       /* ID of main window */

/* Keyboard stuff */
LOCAL_D WORD key_stat;
LOCAL_D WMSG_KEY key;
LOCAL_D WORD key_active=FALSE;

/* Stack stuff */
LOCAL_D WORD stackevent_stat;
#define MAX_STACKEVENTBUFFER 1024
LOCAL_D BYTE stackevent_buffer[MAX_STACKEVENTBUFFER];
LOCAL_D WORD stackevent_requestqueued=FALSE;
LOCAL_D UWORD stackevent_retlength;

/* Remote system stuff */
struct sockaddr_in remote; /* Connection details get resolved into here */

/* Socket stuff */
UINT mailsocket = -1;
WORD connect_stat = E_FILE_PENDING; /* socket_connecta() status word */


LOCAL_D P_SCR_SET_FONT Font;        /* Font used by the console */
LOCAL_D UWORD screenwidth; /* current width of main display area taking into 
                               account the status window */
LOCAL_D UWORD lastbflags=0;   /* which kind of Border flags */
LOCAL_D INT cent=0;             /* entry at cursor */
LOCAL_D INT tent=0;             /* entry at top of screen */
LOCAL_D INT hent=8;             /* max number of entries on screen */
LOCAL_D INT text_height=0; /* Height of the current font */
LOCAL_D INT text_width=0; /* Width of the current font */
LOCAL_D INT text_ascent=0; /* Ascent of the current font */
#define LAY_TOP_BORDER      4
#define LAY_LEFT_BORDER     4
#define LAY_RIGHT_BORDER    4
#define VERTICAL_LEADING    2
#define X_TO_END            (screenwidth-LAY_RIGHT_BORDER)
#define Y_TO_FIRST_MSG      (text_height+LAY_TOP_BORDER+3)
#define LINE_HEIGHT         (text_height)


/* Column titles */
#define TITLES 8
#define COLUMN_STATUS 0
#define COLUMN_PRIORITY 1
#define COLUMN_ATTACHMENT 2
#define COLUMN_WHO 3
#define COLUMN_DATE 4
#define COLUMN_SIZE 5
#define COLUMN_ACTION 6
#define COLUMN_SUBJECT 7

#define COLUMN_WHO_CHARS 20
#define COLUMN_DATE_CHARS 10
#define COLUMN_SIZE_CHARS 2
LOCAL_D TEXT *titles[TITLES]= {
    "S",
    "P",
    "A",
    "Who",
    "Date",
    "K",
    "V",
    "Subject"
};
LOCAL_D UWORD cwid[TITLES]; /* Column widths, initialised by SwitchFont() */
LOCAL_D BYTE cwidinchars[TITLES]= {
  1,
  1,
  1,
  COLUMN_WHO_CHARS,
  COLUMN_DATE_CHARS,
  2,
  1,
  0
};

/* Config filename */
LOCAL_D TEXT config_fullname[P_FNAMESIZE]="LOC::M:\\APP\\PSISTACK\\EMAIL3.CFG";

LOCAL_D TEXT *szYes = "Yes";
LOCAL_D TEXT *szNo = "No";
LOCAL_D TEXT *szOk = "Ok";
LOCAL_D TEXT *szUnimplemented = "Not implemented yet";
LOCAL_D TEXT *szNoBlank = "Could not create blank message";

/* The currently-selected mailbox is stored here */
LOCAL_D struct toc mailbox;

/******************************************************************************/
/* The configuration structures */

/* Font details. Can use either Roman or Swiss fonts. The current font is
   specified as being between "0" and "4". This gets added to the font base,
   which is WS_FONT_BASE+4 for Roman and WS_FONT_BASE+8 for Swiss. The
   resulting number is then used as a Font.id.
   The font index and font base are stored in the config structure.
*/
#define ROMAN (WS_FONT_BASE+4)
#define SWISS (WS_FONT_BASE+8)

#define MAX_DETAILS 40
#define MAX_MBXNAME 9
#define MAX_PERSONALITIES 4
#define MAX_SIGLINES 4
#define MAX_SIGLINEWIDTH 80

#define FLAG_SENDONCHECK       0x01
#define FLAG_REMEMBERPASSWORD  0x02
#define FLAG_LEAVEMAILONSERVER 0x04
struct personality {
  TEXT username[MAX_DETAILS]; /* The user's name */
  TEXT address[MAX_DETAILS]; /* The user's email address/POP account */
  TEXT replyaddress[MAX_DETAILS]; /* The user's Reply-To: email address */
  TEXT popserver[MAX_DETAILS]; /* Address of the POP3 server */
  TEXT poppassword[MAX_DETAILS]; /* The POP3 password */
  TEXT smtpserver[MAX_DETAILS]; /* The SMTP server */
  TEXT signature[MAX_SIGLINES][MAX_SIGLINEWIDTH+1];
  BYTE flags; /* FLAGS_??? above */
};


/* The master configuration structure - this gets saved and loaded */
struct config {
  UINT version;
  UINT releaselevel;
  UINT currmode;
  UINT currmaxmode;
  UINT currfontidx; /* 0..4 */
  UINT currfontbase;
  BYTE currpersonality;
  /* The drives where the mailboxes are stored */
  TEXT mbx_letter[MODE_MAX];
  TEXT mbx_name[MODE_MAX][MAX_MBXNAME];
  struct personality personalities[MAX_PERSONALITIES];
  /* Size of status window; whether Status Window is showing. See WSERV
     manual, p83 */
  UWORD statwinstate;
  /* Which columns get displayed */
  BYTE displaycolumn[TITLES];
  BYTE defunload;
} config;

/* The displayed names of the mailboxes */ 
LOCAL_D TEXT *modelist[MODE_MAX] = {
  config.mbx_name[0],
  config.mbx_name[1],
  config.mbx_name[2],
  config.mbx_name[3],
  config.mbx_name[4],
  config.mbx_name[5],
  config.mbx_name[6],
  config.mbx_name[7],
  config.mbx_name[8],
  config.mbx_name[9],
  config.mbx_name[10],
  config.mbx_name[11],
  config.mbx_name[12],
  config.mbx_name[13],
  config.mbx_name[14],
};

/* The file names (main file name, not .MBX, or LOC::M:\EMAIL3\ etc.) 
   of the mailboxes. These are NOT alterable, although the displayed name is.
*/ 
LOCAL_D TEXT *mboxfnames[MODE_MAX] = {
  "IN",
  "OUT",
  "TRASH",
  "RCVD",
  "SENT",
  "USER1",
  "USER2",
  "USER3",
  "USER4",
  "USER5",
  "USER6",
  "USER7",
  "USER8",
  "USER9",
  "USER10",
};

/* The types of the mailboxes */
LOCAL_D BYTE mboxtypes[MODE_MAX] = {
  TYPE_IN,
  TYPE_OUT,
  TYPE_TRASH,
  TYPE_RCVD,
  TYPE_SENT,
  TYPE_USER,
  TYPE_USER,
  TYPE_USER,
  TYPE_USER,
  TYPE_USER,
  TYPE_USER,
  TYPE_USER,
  TYPE_USER,
  TYPE_USER,
  TYPE_USER,
};
/* Forward declarations */
GLREF_C VOID user_dialog(UINT);
GLREF_C VOID server_dialog(UINT);
GLREF_C VOID RedrawEverything(VOID);
GLREF_C VOID Terminate(INT num);
LOCAL_C VOID SwitchFont(INT way);

LOCAL_C INT p_writestr(VOID *pfcb, TEXT *buf)
{
INT errcode;
  errcode = p_write(pfcb, buf, p_slen(buf));
  if (errcode)
    console_printf("couldn't write: %d\r\n", errcode);
  return errcode;
}


GLDEF_C HANDLE fire_composer(VOID)
{
HANDLE hcomp = NULL;
  hcomp = p_execc("LOC::M:\\APP\\PSISTACK\\COMPOSER.APP", NULL, 0);
  if (hcomp < 0) {
    uDisplayText("Could not launch Composer", NULL);
    p_delete(msg_filename);
  }
  else
    p_presume(hcomp);
  return hcomp;
}

GLDEF_C VOID create_msg_fire_composer(VOID)
{
VOID *msgfd;
INT errcode;
TEXT buf[80];
struct personality *p = &config.personalities[config.currpersonality];
INT i,sigexists;
TEXT *s;
  errcode=p_open(&msgfd,msg_filename,P_FREPLACE|P_FTEXT|P_FUPDATE);
  if (errcode) {
    uDisplayText(szNoBlank, NULL);
    return;
  }
  p_atos(buf, "From: %s <%s>", p->username, p->address);
  p_writestr(msgfd, buf);
  p_writestr(msgfd, "");
  for (sigexists=i=0; i<MAX_SIGLINES; i++) {
    s=&p->signature[i][0];
    if (p_slen(s)) {
      if (!sigexists) {
        p_writestr(msgfd, "--");
        sigexists=1;
      }
      p_writestr(msgfd, s);
    }
  }
  p_close(msgfd);
  fire_composer();
}

GLDEF_C VOID copy_msg_fire_composer(VOID)
{
VOID *msgfd;
INT errcode;
TEXT buf[80];
struct personality *p = &config.personalities[config.currpersonality];
INT i,sigexists;
TEXT *s;
HANDLE hcomp = NULL;
  errcode=p_open(&msgfd,msg_filename,P_FREPLACE|P_FSTREAM|P_FUPDATE);
  if (errcode) {
    uDisplayText(szNoBlank, NULL);
    return;
  }
  errcode=toc_copymsg(&mailbox, msgfd);
  if (errcode != 1) {
    p_close(msgfd);
    p_delete(msg_filename);
    uDisplayText("Could not copy message", NULL);
    return;
  }
  p_close(msgfd);
  fire_composer();
}


GLDEF_C INT myuOpenDialog(TEXT *title)
{
  _SmallFontDialog = usesmalldialogs;
  return uOpenDialog(title);
}

GLDEF_C VOID SwitchMailbox(UINT mbxno)
{
UINT errcode;
  toc_close(&mailbox);
  errcode=toc_open(&mailbox, config.mbx_letter[mbxno], mboxfnames[mbxno]);
  if (errcode != 1) {
    uDisplayText("Can't open mailbox, exiting", NULL);
    Terminate(0);
  }
}


/*******************************************************************************
***
*** Function         : config_init
*** Preconditions    : none
*** Postconditions   : The configuration structure is initialised to defaults
***
*******************************************************************************/

LOCAL_C VOID config_init(VOID)
{
static TEXT *mbx_names[MODE_MAX]={
  "In",
  "Out",
  "Trash",
  "Rcvd.",
  "Sent",
  "User 1",
  "User 2",
  "User 3",
  "User 4",
  "User 5",
  "User 6",
  "User 7",
  "User 8",
  "User 9",
  "User 10"
};
INT i;
  p_bfil(&config, sizeof(struct config), 0);

  config.version=version;
  config.releaselevel=releaselevel;
  config.currfontidx=1; /* start with 11pt */
  config.currfontbase=SWISS;
  config.currmode=0;
  config.currmaxmode=4; /* Start with the standard 5 mailboxes */
  config.currpersonality=0; /* Search for the zero inside yourself */
  for (i=0; i<MODE_MAX; i++)
    p_scpy(config.mbx_name[i],mbx_names[i]);
  for (i=0; i<MODE_MAX; i++)
    config.mbx_letter[i]='M';
  for (i=0; i<MAX_PERSONALITIES; i++)
    p_scpy(config.personalities[i].username, mbx_names[i+5]);
  for (i=0; i<TITLES; i++)
    config.displaycolumn[i]=1;
  config.defunload=1;
}

/*******************************************************************************
***
*** Function         : Confirm
*** Preconditions    : pb is a string to display in a dialog
*** Postconditions   : The message is displayed, and this returns TRUE iff user
***                    answered Yes
***
*******************************************************************************/

LOCAL_C INT Confirm(TEXT *pb)
{
  if (myuOpenDialog(pb))
    return(-1);
  if (uAddButtonList(szNo,-'n',szYes,'y',NULL))
    return(-1);
  return(uRunDialog()!='y');
}


/*******************************************************************************
***
*** Function         : Ok
*** Preconditions    : pb is a string to display in a dialog.
*** Postconditions   : The message is shown with an 'Ok' button.
***
*******************************************************************************/

LOCAL_C VOID Ok(TEXT *pb)
{
  if (myuOpenDialog(pb))
    return;
  if (uAddButtonList(szOk,W_KEY_RETURN,NULL))
    return;
  uRunDialog();
}



LOCAL_C INT drive_exists(TEXT letter)
{
VOID *dcb;
INT ret;
TEXT device[P_FNAMESIZE];
  if ((ret=p_open(&dcb,"LOC::",P_FDEVICE))<0){ 
    return 0;
  }
  while (!p_iow(dcb,P_FREAD,&device[0],NULL)) {
    if (device[0]==p_toupper(letter)) {
      ret = 1;
      break;
    }
  }
  p_close(dcb);
  return ret;
}



VOID personality_dialog(VOID)
{
UWORD currpersonality=config.currpersonality+1;
H_DI_CHOICE ch;

INT i;
  if (myuOpenDialog("Choose active personality") || uBeginDCL(&ch))
    return;
  for (i=0; i<MAX_PERSONALITIES; i++) {
    if (uGrowDCL(&ch,config.personalities[i].username))
      return;
  }
  if (uAddDCL("Name", &currpersonality, &ch) ||
      uRunDialog()<=0) 
    return;

  /* Copy data back */
  config.currpersonality = currpersonality-1;
}


GLREF_C VOID signature_dialog(UINT personality)
{
TEXT buf[50];
struct personality *p=&config.personalities[personality];
H_DI_SEDIT di_line[MAX_SIGLINES];
TEXT line[MAX_SIGLINES][MAX_SIGLINEWIDTH+1];
int i;
  /* Copy data in */
  for (i=0; i<MAX_SIGLINES; i++) {
    di_line[i].len=MAX_SIGLINEWIDTH;
    di_line[i].width=40;
    di_line[i].str=&line[i][0];
    uZTStoBCS(di_line[i].str, &p->signature[i][0]);
  }
  p_atos(buf, "Signature for Personality #%d",personality+1);
  if (myuOpenDialog(buf))
    return;

  for (i=0; i<MAX_SIGLINES; i++) 
    if (uAddDialogItem(H_DIALOG_SEDIT, " ", &di_line[i]))
      return;
  if (uRunDialog()<=0)
    return;

  /* Copy data back */
  for (i=0; i<MAX_SIGLINES; i++) {
    line[i][1+line[i][0]]=0;
    p_scpy(&p->signature[i][0],&line[i][1]);
  }
}


GLREF_C VOID user_dialog(UINT personality)
{
TEXT buf[50];
struct personality *p=&config.personalities[personality];
H_DI_SEDIT di_name;
TEXT name[MAX_DETAILS+1];
H_DI_SEDIT di_eaddress;
TEXT eaddress[MAX_DETAILS+1];
H_DI_SEDIT di_raddress;
TEXT raddress[MAX_DETAILS+1];

  /* Copy data in */
  di_name.len=MAX_DETAILS;
  di_name.width=20;
  di_name.str=&name[0];
  uZTStoBCS(di_name.str, p->username);

  di_eaddress.len=MAX_DETAILS;
  di_eaddress.width=20;
  di_eaddress.str=&eaddress[0];
  uZTStoBCS(di_eaddress.str, p->address);

  di_raddress.len=MAX_DETAILS;
  di_raddress.width=20;
  di_raddress.str=&raddress[0];
  uZTStoBCS(di_raddress.str, p->replyaddress);

  p_atos(buf, "User details for Personality #%d",personality+1);
  if (myuOpenDialog(buf) ||
      uAddDialogItem(H_DIALOG_SEDIT, "Real name", &di_name) ||
      uAddDialogItem(H_DIALOG_SEDIT, "Email addr.", &di_eaddress) ||
      uAddDialogItem(H_DIALOG_SEDIT, "Reply-To addr.", &di_raddress) ||
      uRunDialog()<=0) 
    return;

  /* Copy data back */
  name[1+name[0]]=0;
  p_scpy(p->username,&name[1]);

  eaddress[1+eaddress[0]]=0;
  p_scpy(p->address,&eaddress[1]);

  raddress[1+raddress[0]]=0;
  p_scpy(p->replyaddress,&raddress[1]);
}

GLREF_C VOID prefs_dialog(VOID)
{
UWORD unload;

  unload = config.defunload + 1;

  if (myuOpenDialog("Preferences") ||
      uAddChoiceList("Unload preference",&unload,szNo,szYes,NULL))
    return;
  if (uRunDialog()<=0)
    return;

  config.defunload = unload-1;
}
 

GLREF_C VOID server_dialog(UINT personality)
{
TEXT buf[50];
struct personality *p=&config.personalities[personality];
H_DI_SEDIT di_smtpserver;
TEXT smtpserver[MAX_DETAILS+1];
H_DI_SEDIT di_popserver;
TEXT popserver[MAX_DETAILS+1];
H_DI_XINPUT di_poppassword;
TEXT poppassword[MAX_DETAILS+2];
UWORD rempass;
UWORD leavemail;
UWORD sendoncheck;

  /* Copy data in */
  di_smtpserver.len=MAX_DETAILS;
  di_smtpserver.width=20;
  di_smtpserver.str=&smtpserver[0];
  uZTStoBCS(di_smtpserver.str, p->smtpserver);

  di_popserver.len=MAX_DETAILS;
  di_popserver.width=20;
  di_popserver.str=&popserver[0];
  uZTStoBCS(di_popserver.str, p->popserver);

  di_poppassword.str=&poppassword[0];
  uZTStoBCS(di_poppassword.str, p->poppassword);

  rempass = p->flags & FLAG_REMEMBERPASSWORD ? 2 : 1;
  leavemail = p->flags & FLAG_LEAVEMAILONSERVER ? 2 : 1;
  sendoncheck = p->flags & FLAG_SENDONCHECK ? 2 : 1;

  p_atos(buf, "Server details for Personality #%d",personality+1);
  if (myuOpenDialog(buf) ||
      uAddDialogItem(H_DIALOG_SEDIT, "SMTP server", &di_smtpserver) ||
      uAddDialogItem(H_DIALOG_SEDIT, "POP3 server", &di_popserver) ||
      uAddDialogItem(H_DIALOG_XINPUT, "POP password", &di_poppassword) ||
      uAddChoiceList("Remember it?",&rempass,szNo,szYes,NULL) ||
      uAddChoiceList("Send on check?",&sendoncheck,szNo,szYes,NULL) ||
      uAddChoiceList("Leave on server?",&leavemail,szNo,szYes,NULL))
    return;
  if (uRunDialog()<=0)
    return;

  smtpserver[1+smtpserver[0]]=0;
  p_scpy(p->smtpserver,&smtpserver[1]);

  popserver[1+popserver[0]]=0;
  p_scpy(p->popserver,&popserver[1]);

  poppassword[1+poppassword[0]]=0;
  p_scpy(p->poppassword,&poppassword[1]);

  p->flags &= ~(FLAG_REMEMBERPASSWORD | 
                FLAG_LEAVEMAILONSERVER | 
                FLAG_SENDONCHECK);
  p->flags |= (rempass == 2) ? FLAG_REMEMBERPASSWORD : 0;
  p->flags |= (leavemail == 2) ? FLAG_LEAVEMAILONSERVER : 0;
  p->flags |= (sendoncheck == 2) ? FLAG_SENDONCHECK : 0;
}

VOID columns_dialog(VOID)
{
UWORD status, priority, attachment, date, size, action;
  size = config.displaycolumn[COLUMN_SIZE] ? 2 : 1;
  status = config.displaycolumn[COLUMN_STATUS] ? 2 : 1;
  priority = config.displaycolumn[COLUMN_PRIORITY] ? 2 : 1;
  attachment = config.displaycolumn[COLUMN_ATTACHMENT] ? 2 : 1;
  date = config.displaycolumn[COLUMN_DATE] ? 2 : 1;
  size = config.displaycolumn[COLUMN_SIZE] ? 2 : 1;
  action = config.displaycolumn[COLUMN_ACTION] ? 2 : 1;
  if (myuOpenDialog(NULL /*"Display which columns?"*/) ||
      uAddChoiceList("Status",&status,szNo,szYes,NULL) ||
      uAddChoiceList("Priority",&priority,szNo,szYes,NULL) ||
      uAddChoiceList("Attachments",&attachment,szNo,szYes,NULL) ||
      uAddChoiceList("Date",&date,szNo,szYes,NULL) ||
      uAddChoiceList("Size",&size,szNo,szYes,NULL) ||
      uAddChoiceList("Action",&action,szNo,szYes,NULL))
    return;
  if (uRunDialog()<=0)
    return;

  config.displaycolumn[COLUMN_STATUS] = (status == 2);
  config.displaycolumn[COLUMN_PRIORITY] = (priority == 2);
  config.displaycolumn[COLUMN_ATTACHMENT] = (attachment == 2);
  config.displaycolumn[COLUMN_DATE] = (date == 2);
  config.displaycolumn[COLUMN_SIZE] = (size == 2);
  config.displaycolumn[COLUMN_ACTION] = (action == 2);
}

/*******************************************************************************
***
*** Function         : ask_questions
*** Preconditions    : The config file has not been created yet
*** Postconditions   : Some config file stuff is prompted for
***
*******************************************************************************/

VOID ask_questions(VOID)
{
INT a_ex, b_ex;
H_DI_CHOICE ch[MODE_USER];
UWORD choice[MODE_USER];
TEXT buf[20];
INT i;
  uDisplayText("      -=-=> Welcome to Email3 <=-=-",
               "Email3 needs to be configured...",
               "You will now be asked:",
               "* Where to store mailboxes",
               "    (If you have SSD(s) inserted)",
               "* Your name, your email address",
               "* Your Mail servers - SMTP & POP3",
               "    (All changeable later)", NULL);
  /* The user can stick their mailboxes on any drive, M, A or B. Obviously,
     if they haven't got an A or B, then they can't use them, and they have to 
     use M. If they buy an SSD later, they can move their mailboxes. 

     If A or B exist, ask the user where they would like to store the mail.
     Ask on a per-mailbox basis...
  */

  a_ex=drive_exists('A');
  b_ex=drive_exists('B');
  if (a_ex || b_ex) {
    if (myuOpenDialog("Choose folder locations"))
      return;
    for (i=0; i<MODE_USER; i++) {
      if (uBeginDCL(&ch[i]))
        return;
      if (uGrowDCL(&ch[i],"Internal"))
        return;
      if (a_ex)
        if (uGrowDCL(&ch[i],"A"))
          return;
      if (b_ex)
        if (uGrowDCL(&ch[i],"B"))
          return;
      choice[i] = 1;
      p_atos(buf,"%s drive",modelist[i]);
      if (uAddDCL(buf, &choice[i], &ch[i]))
        return;
    }
    if (uRunDialog()>0) {
      for (i=0; i<MODE_USER; i++) {
        if (choice[i] == 1)
           config.mbx_letter[i]='M';
        else 
          if (choice[i] == 2 && a_ex)
            config.mbx_letter[i]='A';
          else 
            config.mbx_letter[i]='B';
      }
    }
  }

  user_dialog(0);
  server_dialog(0);
  signature_dialog(0);
}


/*******************************************************************************
***
*** Function         : config_save
*** Preconditions    : There is a valid - or default - configuration
*** Postconditions   : The configuration is saved to disk
***
*******************************************************************************/

VOID config_save(VOID)
{
VOID *fd = NULL;
INT errcode;
TEXT buf[50];
  errcode=p_open(&fd,config_fullname,P_FREPLACE|P_FSTREAM|P_FUPDATE);
  if (uErrorValue(errcode)) {
    p_atos(buf, "Couldn't create \"%s\"", config_fullname+21);
    wInfoMsg(buf);
    return;
  }
  p_write(fd,(VOID *)&config,sizeof(struct config));
  p_close(fd);
}

/*******************************************************************************
***
*** Function         : config_load
*** Preconditions    : none in particular
*** Postconditions   : The configuration is loaded
***
*******************************************************************************/

VOID config_load(VOID)
{
VOID *fd = NULL;
TEXT buf[100];
  if (p_open(&fd,config_fullname,P_FOPEN|P_FSTREAM|P_FSHARE) != 0) {
    p_atos(buf,"Creating \"%s\"", config_fullname+21);
    wInfoMsg(buf);
    ask_questions();
    config_save();
    return;
  }
  p_read(fd,(VOID *)&config,sizeof(struct config));
  if (config.version != version || config.releaselevel != releaselevel) {
    p_atos(buf, "Config file created by v%d.%02d%c\r\ncannot be used, so defaults\r\nwill be used.", config.version / 100, config.version % 100, config.releaselevel);
    Ok(buf);
    p_close(fd);
    config_init();
    config_save();
    return;
  }
  p_close(fd);
}

GLDEF_C VOID BarGauge(ULONG size, ULONG pos)
{
TEXT buf[30];
UINT num;
  num = (UINT) ((pos * 100L) / size);
  p_atos(buf,"%d%% complete", num);
  wInfoMsg(buf);
}

/*******************************************************************************
***
*** Function         : MailboxesOK
*** Preconditions    : The config is loaded (or init'd)
*** Postconditions   : TOCs exist for all created mailboxes, and are created if
***                    they either don't exist, or are out of date.
***                    Returns TRUE if OK or FALSE on a fatal error.
***
*******************************************************************************/

GLDEF_C UINT MailboxesOK(VOID)
{
INT i;
INT errcode;
struct toc m;
TEXT buf[50];
static TEXT *szEnd="Cannot continue - terminating";
static TEXT *szCreateFail=" creation failed";
static TEXT *szHDRFail=" HDR write failed";

  for (i=0; i<config.currmaxmode; i++) {
    p_bfil((VOID *)&m, sizeof(struct toc), 0);
    p_atos(buf,"Checking %s mailbox...", modelist[i]);
    /* Delay busy message for half a second */
    wSetBusyMsg(buf, W_CORNER_BOTTOM_LEFT|1);
    /* Does the mail folder directory exist on this drive? */
    if (!toc_direxists(config.mbx_letter[i])) {

      if (toc_mkdir(config.mbx_letter[i]) != 0) {
        p_atos(buf,"%s folder directory%s", modelist[i], szCreateFail);
        uDisplayText(buf, szEnd, NULL);
        wCancelBusyMsg();
        return FALSE;
      }
    }
    errcode=toc_open(&m, config.mbx_letter[i], mboxfnames[i]);
    switch (errcode) {
      case 1: 
        /* Everything OK. Just close the mailbox (will update TOC header) */
        toc_close(&m);
        wCancelBusyMsg();
        break;
      case E_TOC_MBXOPENFAIL:
        /* Create empty mailbox file, and then create header */
        wInfoMsg("Creating...");
        errcode=toc_format(&m,mboxtypes[i],config.mbx_letter[i],mboxfnames[i]);
        wCancelBusyMsg();
        /* If that failed, we're dead */
        switch (errcode) {
          case 1:
            /* Everything OK. Continue... */
            break;
          case E_TOC_MBXCREATEFAIL:
            p_atos(buf,"MBX %s%s", modelist[i], szCreateFail);
            uDisplayText(buf, szEnd, NULL);
            return FALSE;
          case E_TOC_TOCCREATEFAIL:
            p_atos(buf,"TOC %s%s", modelist[i], szCreateFail);
            uDisplayText(buf, szEnd, NULL);
            return FALSE;
          case E_TOC_TOCHDRWRITEFAIL:
            p_atos(buf,"TOC %s%s", modelist[i], szHDRFail);
            uDisplayText(buf, szEnd, NULL);
            return FALSE;
          default:
            uDisplayText("Format - what happened?!", NULL);
            return FALSE;
        }
        break;
      case E_TOC_TOCOPENFAIL:
      case E_TOC_TOCSTALE:
        /* MBX exists - need to (re)create the TOC */
        wInfoMsg("Recreating TOC...");
        errcode=toc_create(&m,mboxtypes[i],config.mbx_letter[i],mboxfnames[i],
                           BarGauge);
        wCancelBusyMsg();
        /* If that failed, we're dead */
        switch (errcode) {
          case 1:
            /* Everything OK. Continue... */
            break;
          case E_TOC_MBXOPENFAIL:
            p_atos(buf,"MBX %s open failed", modelist[i]);
            uDisplayText(buf, szEnd, NULL);
            return FALSE;
          case E_TOC_TOCCREATEFAIL:
            p_atos(buf,"TOC %s%s", modelist[i], szCreateFail);
            uDisplayText(buf, szEnd, NULL);
            return FALSE;
          case E_TOC_TOCHDRWRITEFAIL:
            p_atos(buf,"TOC %s%s", modelist[i], szHDRFail);
            uDisplayText(buf, szEnd, NULL);
            return FALSE;
          case E_TOC_MSGWRITEFAIL:
            p_atos(buf,"TOC %s MSG write failef", modelist[i]);
            uDisplayText(buf, szEnd, NULL);
            return FALSE;
          default:
            uDisplayText("Format - what happened?!", NULL);
            return FALSE;
        }
        break;
      default:
        uDisplayText("Open - what happened?!", NULL);
        wCancelBusyMsg();
    }
  }
  return TRUE;
}


/*******************************************************************************
***
*** Function         : Terminate(exitcode)
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID Terminate(INT num)
{
  config_save();
  p_exit(num);     /* Goodnight Vienna */
}


/*******************************************************************************
***
*** Function         : ResizeWidth
*** Preconditions    : 
*** Postconditions   : Try to change the width of the main display region 
***
*******************************************************************************/

LOCAL_C INT ResizeWidth(UWORD width)
{
INT ret;
W_WINDATA wd;

  wd.extent.tl.x=wd.extent.tl.y=0;
  wd.extent.width=width;
  wd.extent.height=machine_fullscreenheight;
  wSetWindow(MainWid,W_WIN_EXTENT,&wd);
  ret=uErrorValue(wCheckPoint());     /* check whether successful */
  if (!ret)
    screenwidth=width;
  return(ret);
}


/*******************************************************************************
***
*** Function         : SetGmode
*** Preconditions    : 
*** Postconditions   : Sets Gmode of current graphics context
***
*******************************************************************************/

LOCAL_C VOID SetGmode(INT gmode)
{
G_GC gc;
  gc.gmode=gmode;
  gSetGC(0,G_GC_MASK_GMODE,&gc);
}



/*******************************************************************************
***
*** Function         : ClearWholeWindow
*** Preconditions    : 
*** Postconditions   : Clear the entire screen 
***
*******************************************************************************/

LOCAL_C VOID ClearWholeWindow(VOID)
{
P_RECT whole;

  whole.tl.x=whole.tl.y=0;
  whole.br.x=machine_fullscreenwidth;
  whole.br.y=machine_fullscreenheight;
  gClrRect(&whole,G_TRMODE_CLR);
}



/*******************************************************************************
***
*** Function         : DrawTitles
*** Preconditions    : 
*** Postconditions   : The column titles are drawn at the top of the screen.
***
*******************************************************************************/

LOCAL_C VOID DrawTitles(VOID)
{
INT i;
INT totwid;
TEXT *buf;
P_RECT box;

  totwid=0;
  for (i=0; i<TITLES; i++)
    if (config.displaycolumn[i])
      totwid+=cwid[i];
  box.tl.x=LAY_LEFT_BORDER;
  box.tl.y=LAY_TOP_BORDER;
  box.br.y=Y_TO_FIRST_MSG - 1; /*box.tl.y+LINE_HEIGHT+1;*/
  for (i=0; i<TITLES; i++) {
    if (!config.displaycolumn[i] || cwid[i]==0)
      continue;
    if (i==(TITLES-1))
      box.br.x=X_TO_END;
    else {
      box.br.x=box.tl.x+cwid[i];
      gDrawLine(box.br.x,0,box.br.x,machine_fullscreenheight); /*box.br.y+1);*/
    }
    buf=titles[i];
    gPrintBoxText(&box,text_ascent,G_TEXT_ALIGN_LEFT,0,buf,p_slen(buf));
    box.br.x+=2;
    box.tl.x=box.br.x;
  }
  gDrawLine(0,box.br.y-2,screenwidth,box.br.y-2);
}


/*******************************************************************************
***
*** Function         : InvertLine
*** Preconditions    : 
*** Postconditions   : Inverts a given line for the cursor
***
*******************************************************************************/

LOCAL_C VOID InvertLine(INT j)
{
P_EXTENT ext;

  ext.tl.x=LAY_LEFT_BORDER-1;
  ext.tl.y=Y_TO_FIRST_MSG+(j*LINE_HEIGHT)-1;
  ext.width=X_TO_END-LAY_LEFT_BORDER+2;
  ext.height=LINE_HEIGHT;
  gInvObloid(&ext);
}

/*******************************************************************************
***
*** Function         : DrawLine
*** Preconditions    : 
*** Postconditions   : Draws a message header
***
*******************************************************************************/

LOCAL_C VOID DrawLine(INT j)
{
P_RECT box;
TEXT *buf;
TEXT datestr[40];
INT i;
INT tocind;
UWORD stat;
INT errcode;

  /* Which table of contents entry is this? */
  tocind = tent+j;

  box.tl.x=LAY_LEFT_BORDER;
  box.tl.y=Y_TO_FIRST_MSG+(j*LINE_HEIGHT);
  box.br.y=box.tl.y+LINE_HEIGHT;
  /* Clear past end of list */
  if (tocind >= mailbox.th.th_nummsgs) {
    box.br.x=X_TO_END;
    gClrRect(&box,G_TRMODE_CLR);
    for (i=0; i<TITLES; i++) {
      if (!config.displaycolumn[i])
        continue;
      if (i==(TITLES-1))
        box.br.x=X_TO_END;
      else {
        if (cwid[i]==0)
          return;
        box.br.x=box.tl.x+cwid[i];
        gDrawLine(box.br.x,0,box.br.x,box.br.y+1);
      }
      box.br.x+=2;
      box.tl.x=box.br.x;
    }
  }
  else {
    /* Load this table of contents entry */
    errcode = toc_readrandom(&mailbox, tocind);
    if (errcode != 1) {
      uDisplayText("Couldn't load MBX entry", NULL);
      return;
    }

    box.br.x=LAY_LEFT_BORDER;
    /* Display actual data before end of list */
    /* Status column */
    if (config.displaycolumn[COLUMN_STATUS]) {
      box.tl.x=box.br.x;
      box.br.x=box.tl.x+cwid[COLUMN_STATUS];
      stat=mailbox.tm.tm_status & STATUS_STATMASK;
      switch (stat) {
        case STATUS_STATUNREAD: buf = "\376"; break;
        case STATUS_STATREAD: buf = " "; break;
        case STATUS_STATREPLY: buf = "R"; break;
        case STATUS_STATFWD: buf = "F"; break;
        case STATUS_STATREDIR: buf = "D"; break;
        case STATUS_STATSENT: buf = "S"; break;
        case STATUS_STATQUEUED: buf = "Q"; break;
        case STATUS_STATTQUEUED: buf = "T"; break;
        case STATUS_STATDRAFTED: buf = "-"; break;
        default: buf = "?"; break;
      }
      gPrintBoxText(&box,text_ascent,G_TEXT_ALIGN_CENTRE,0,buf,1);
      if (cwid[COLUMN_STATUS]==0)
        return;
      gDrawLine(box.br.x,box.tl.y,box.br.x,box.br.y);
      box.br.x+=2;
    }

    /* Priority column */
    if (config.displaycolumn[COLUMN_PRIORITY]) {
      box.tl.x=box.br.x;
      box.br.x=box.tl.x+cwid[COLUMN_PRIORITY];
      stat=mailbox.tm.tm_status & STATUS_PRIMASK;
      switch (stat) {
        case STATUS_PRILOWEST: buf = "L"; break;
        case STATUS_PRILOW: buf = "l"; break;
        case STATUS_PRINORMAL: buf = "-"; break;
        case STATUS_PRIHIGH: buf = "h"; break;
        case STATUS_PRIHIGHEST: buf = "H"; break;
        default: buf = "?"; break;
      }
      gPrintBoxText(&box,text_ascent,G_TEXT_ALIGN_LEFT,0,buf,1);
      if (cwid[COLUMN_PRIORITY]==0)
        return;
      gDrawLine(box.br.x,box.tl.y,box.br.x,box.br.y);
      box.br.x+=2;
    }

    /* Attachment column */
    if (config.displaycolumn[COLUMN_ATTACHMENT]) {
      box.tl.x=box.br.x;
      box.br.x=box.tl.x+cwid[COLUMN_ATTACHMENT];
      buf = (mailbox.tm.tm_status & STATUS_ATTACH) ? "\260" : " ";
      gPrintBoxText(&box,text_ascent,G_TEXT_ALIGN_LEFT,0,buf,1);
      if (cwid[COLUMN_ATTACHMENT]==0)
        return;
      gDrawLine(box.br.x,box.tl.y,box.br.x,box.br.y);
      box.br.x+=2;
    }

    /* Who column */
    if (config.displaycolumn[COLUMN_WHO]) {
      box.tl.x=box.br.x;
      box.br.x=box.tl.x+cwid[COLUMN_WHO]-1;
      buf=mailbox.tm.tm_who;
      gPrintBoxText(&box,text_ascent,G_TEXT_ALIGN_LEFT,0,buf,p_slen(buf));
      box.br.x++;
      if (cwid[COLUMN_WHO]==0)
        return;
      gDrawLine(box.br.x,box.tl.y,box.br.x,box.br.y);
      box.br.x+=2;
    }

    /* Date column */
    if (config.displaycolumn[COLUMN_DATE]) {
      box.tl.x=box.br.x;
      box.br.x=box.tl.x+cwid[COLUMN_DATE]-1;
      toc_displaydate(datestr, mailbox.tm.tm_date);
      gPrintBoxText(&box,text_ascent,G_TEXT_ALIGN_LEFT,0,datestr,
                    p_slen(datestr));
      box.br.x++;
      if (cwid[COLUMN_DATE]==0)
        return;
      gDrawLine(box.br.x,box.tl.y,box.br.x,box.br.y);
      box.br.x+=2;
    }

    /* Size column */
    if (config.displaycolumn[COLUMN_SIZE]) {
      box.tl.x=box.br.x;
      box.br.x=box.tl.x+cwid[COLUMN_SIZE];
      stat=(UWORD) ((mailbox.tm.tm_size/1024L) & 0xFFFF);
      if (stat==0)
        stat=1;
      p_atos(datestr,"%d",stat);
      gPrintBoxText(&box,text_ascent,G_TEXT_ALIGN_LEFT,0,datestr,
                    p_slen(datestr));
      if (cwid[COLUMN_SIZE]==0)
        return;
      gDrawLine(box.br.x,box.tl.y,box.br.x,box.br.y);
      box.br.x+=2;
    }

    /* Action column */
    if (config.displaycolumn[COLUMN_ACTION]) {
      box.tl.x=box.br.x;
      box.br.x=box.tl.x+cwid[COLUMN_ACTION];
      stat=mailbox.tm.tm_status & STATUS_SERVMASK;
      if (stat & STATUS_SERVFETCH && stat & STATUS_SERVDELETE)
        buf="g";
      else if (stat & STATUS_SERVFETCH)
        buf="f";
      else if (stat & STATUS_SERVDELETE)
        buf="d";
      else
        buf="?";
      gPrintBoxText(&box,text_ascent,G_TEXT_ALIGN_LEFT,0,buf,1);
      if (cwid[COLUMN_ACTION]==0)
        return;
      gDrawLine(box.br.x,box.tl.y,box.br.x,box.br.y);
      box.br.x+=2;
    }

    /* Subject column */
    if (config.displaycolumn[COLUMN_SUBJECT]) {
      box.tl.x=box.br.x;
      box.br.x=box.tl.x+cwid[COLUMN_SUBJECT];
      buf=mailbox.tm.tm_subject;
      gPrintBoxText(&box,text_ascent,G_TEXT_ALIGN_LEFT,0,buf,p_slen(buf));
    }
  }
}

/*******************************************************************************
***
*** Function         : DrawList
*** Preconditions    : 
*** Postconditions   : Draws all currently shown messages
***
*******************************************************************************/

LOCAL_C VOID DrawList(VOID)
{
INT j;

  for (j=0; j<hent; j++)
    DrawLine(j);
  InvertLine(cent-tent);
}

/*******************************************************************************
***
*** Function         : CycleStatusWindow
*** Preconditions    : 
*** Postconditions   : Attempt to flip through the different states of the 
***                    status window 
***
*******************************************************************************/

LOCAL_C VOID CycleStatusWindow(VOID)
{
P_EXTENT pextent;
UWORD Newstatwinstate;

  switch (config.statwinstate) {
    case W_STATUS_WINDOW_BIG:
      /* big status window is currently showing -> go to off */
      if (ResizeWidth(machine_fullscreenwidth))
        return;
      config.statwinstate = W_STATUS_WINDOW_OFF;
      wStatusWindow(config.statwinstate);
      break;
    case W_STATUS_WINDOW_OFF:
      /* Status window not currently showing -> go to narrow */
      /* How wide will the status window be? shrink the main window
         by that much. */
      wInquireStatusWindow(W_STATUS_WINDOW_SMALL, &pextent);
      if (ResizeWidth(machine_fullscreenwidth - pextent.width))
        return;
      config.statwinstate = W_STATUS_WINDOW_SMALL;
      wStatusWindow(config.statwinstate);
      break;
    case W_STATUS_WINDOW_SMALL:
      /* Small status window currently showing -> go to off or big
         depending on whether we're running on a Siena or 3A/C */
      /* How wide will the status window be? shrink the main window
         by that much. */
      Newstatwinstate = ((machine_type==MACH_SIENA ||
                          machine_type==MACH_EM_SIENA) ? W_STATUS_WINDOW_OFF :
                                                         W_STATUS_WINDOW_BIG);
      wInquireStatusWindow(Newstatwinstate, &pextent);
      if (ResizeWidth(machine_fullscreenwidth - pextent.width))
        return;
      config.statwinstate = Newstatwinstate;
      wStatusWindow(config.statwinstate);
      break;
  }

  /* redraw the border */
  ClearWholeWindow();
  SwitchFont(config.currfontidx); /* recalculate column widths */
  gBorder(lastbflags);
  DrawTitles();
  DrawList();
}


/*******************************************************************************
***
*** Function         : GetWidth
*** Preconditions    : pb is a string
*** Postconditions   : Length in pixels of the string
***
*******************************************************************************/

LOCAL_C INT GetWidth(TEXT *pb)
{
  return(gTextWidth(Font.id,Font.style,pb,p_slen(pb)));
}


/*******************************************************************************
***
*** Function         : SwitchFont
*** Preconditions    : newfont is MENU_ZOOM_IN/OUT
*** Postconditions   : The font is made larger/smaller, column widths are done
***
*******************************************************************************/

LOCAL_C VOID SwitchFont(INT way) 
{
G_FONT_INFO gfi;
G_GC gc;
INT totwid, colsdisplayed;
INT i, currwid;

  switch (way) {
    case MENU_ZOOM_OUT: /* make it smaller */
      config.currfontidx = (config.currfontidx == 0) ? 3 : config.currfontidx-1;
      break;
    case MENU_ZOOM_IN: /* make it bigger */
      config.currfontidx = (config.currfontidx == 3) ? 0 : config.currfontidx+1;
      break;
    default:
      config.currfontidx = way;
      break;
  }

  /* And configure the new font */
  Font.style = 0; /*G_STY_MONO;*/
  Font.id = config.currfontbase + config.currfontidx;

  /* Get font metrics */
  gFontInfo(Font.id, Font.style, &gfi);
  text_height=gfi.ascent+gfi.descent+VERTICAL_LEADING; 
  text_width=gfi.max_width;
  text_ascent=gfi.ascent;

  colsdisplayed=1;
  /* Configure the column widths based on the new font */
  totwid=LAY_LEFT_BORDER;

  for (i=0; i<(TITLES-1); i++) {
    cwid[i]=text_width * cwidinchars[i];
    if (config.displaycolumn[i]) {
      totwid+=cwid[i];
      colsdisplayed++;
    }
  }
  cwid[COLUMN_SUBJECT]=X_TO_END-totwid-(colsdisplayed << 1); 
  /* the << 1 means 2 pixels between each column to account for */

goto fred;
  /* Clip the column that hits the right hand side, and zero those that
     follow. Can't do this in the above loop, since we don't know how 
     many columns are displayed. */
  currwid=LAY_LEFT_BORDER;
  for (i=0; i<TITLES; currwid+=cwid[i++]+2) {
    if (currwid+cwid[i]+2 > X_TO_END) {
      /* Clip it */
      cwid[i]-= (currwid+cwid[i]+(colsdisplayed << 2)-X_TO_END);
      for (i++;i<TITLES; i++)
        cwid[i]=0;
      break;
    }
  }
fred:

  /* Set this into the graphics context */
  gc.font=Font.id;
  gc.style=Font.style;
  gSetGC(0,G_GC_MASK_FONT|G_GC_MASK_STYLE,&gc);

  /* How many messages can we display on screen? */
  hent=(machine_fullscreenheight-Y_TO_FIRST_MSG)/LINE_HEIGHT;
  /* If the cursor was on the last line of the screen, and we made the font
   * larger, hent would have decreased, and the last line might not now be
   * visible. It would be techy to move the screen contents up to show the 
   * cursor on the last line again. That's too coplex. Just home it. */
  cent = tent = 0;
}



LOCAL_C VOID DoScroll(INT dely)
{                   /* scroll the display vertically by given amount */
P_POINT offset;         /* offset to scroll */
P_RECT src;             /* source region for scroll */

  offset.x=0;
  offset.y=dely*LINE_HEIGHT;
  src.tl.x=LAY_LEFT_BORDER-1;
  src.br.x=X_TO_END+1;
  if (dely>0) {             /* scrolling down - so exclude bottom from source */
    src.tl.y=Y_TO_FIRST_MSG;
    src.br.y=Y_TO_FIRST_MSG+(hent-dely)*LINE_HEIGHT;
  }
  else {               /* scrolling up - so exclude top from source */
    src.tl.y=Y_TO_FIRST_MSG-(dely*LINE_HEIGHT);
    src.br.y=Y_TO_FIRST_MSG+hent*LINE_HEIGHT;
  }
  wScrollRect(MainWid,&src,&offset);
}

/*******************************************************************************
***
*** Function         : ScrollPageBy(delta)
*** Preconditions    : delta is how much to scroll by
*** Postconditions   : 
***
*******************************************************************************/

LOCAL_C VOID ScrollPageBy(INT delta)
{                   /* implementation of PgUp / PgDn */
  if (!mailbox.th.th_nummsgs)
    return;         /* no entries to display */
  InvertLine(cent-tent); /* remove existing cursor */
  if (delta<(-(hent-1)))
    delta=(-(hent-1));     /* scroll at most three records */
  else if (delta>(hent-1))
    delta=(hent-1);
  tent+=delta;
  cent+=delta;
  DrawList();
}

/*******************************************************************************
***
*** Function         : CursorMove(newcent)
*** Preconditions    : newcent is the new cursor position
*** Postconditions   : Change the cursor position with smooth scrolling
***
*******************************************************************************/

LOCAL_C VOID CursorMove(INT newcent)
{
INT toscroll;       /* amount of scroll needed (if any) */
  if (!mailbox.th.th_nummsgs)
    return;         /* no entries - hence nothing to do */
  if (newcent==cent)
    return;         /* cursor already at specified position */
  InvertLine(cent-tent); /* remove existing cursor */
  toscroll=tent-newcent;
  if (toscroll>0) {               /* new cursor position off top of screen */
    tent-=toscroll;     /* adjust record of entry at top of screen */
    if (toscroll<hent)
      DoScroll(toscroll);     /* only actually scroll if less than 4 */
    else
      toscroll=hent; /* toscroll now holds number of lines to redraw */
    while (toscroll)
      DrawLine(--toscroll);       /* redraw lines not scrolled */
  }
  else {
    toscroll=-(hent-1)-toscroll;
    if (toscroll>0) {            /* new cursor position off bottom of screen */
      tent+=toscroll; /* adjust record of entry at top of screen */
      if (toscroll<hent)
        DoScroll(-toscroll);
      else
        toscroll=hent;
      while (toscroll)
        DrawLine(hent-toscroll--);     /* redraw lines not scrolled */
    }
  }
  cent=newcent;
  InvertLine(cent-tent);      /* draw the cursor back in */
}


/*******************************************************************************
***
*** Function         : SwitchMode
*** Preconditions    : newmode is 0..MAX or a MODE_NEXT/PREV code
*** Postconditions   : The view is changed to that view.
***
*******************************************************************************/

LOCAL_C VOID SwitchMode(INT newmode) 
{
  switch (newmode) {
    case MODE_NEXT:
      config.currmode = (config.currmode == config.currmaxmode) ? 0 : config.currmode+1;
      break;
    case MODE_PREV:
      config.currmode = (config.currmode == 0) ? config.currmaxmode : config.currmode-1;
      break;
    default:
      config.currmode = newmode;
      break;
  }
  wsSelectList(config.currmode);
  /* Load new folder */
  SwitchMailbox(config.currmode);
  /* redisplay folder view */
  RedrawEverything();
}



/*******************************************************************************
***
*** Function         : ExitApplication
*** Preconditions    : User wants to exit
*** Postconditions   : 
***
*******************************************************************************/

LOCAL_C VOID ExitApplication(VOID)
{
UWORD unload=config.defunload+1;
  if (myuOpenDialog("Exit Email3?") ||
      uAddChoiceList("Unload Stack?",&unload,szNo,szYes,NULL) ||
      uAddButtonList(szNo,-'n',szYes,'y',NULL))
    return;
  if (uRunDialog()=='y') {
    if (unload==2)
      stack_unload();
    Terminate(0);
  }
}


/*******************************************************************************
***
*** Function         : ManageCommand
*** Preconditions    : The user selected a menu item
*** Postconditions   : Switch on command index to appropriate function
***
*******************************************************************************/

LOCAL_C VOID ManageCommand(INT index)
{
  switch (index) {
    case MENU_MSG_NEW:
      create_msg_fire_composer();
      break;
    case MENU_MSG_REPLY:
    case MENU_MSG_REPLYALL:
    case MENU_MSG_FWD:
    case MENU_MSG_REDIRECT:
    case MENU_MSG_REQUEUE:
    case MENU_MSG_TRANSFER:

    case MENU_MBX_NEW:
    case MENU_MBX_RENAME:
    case MENU_MBX_MOVE:
    case MENU_MBX_DELETE:
    case MENU_MBX_COMPRESS:
    case MENU_MBX_TRASH:

    case MENU_PREF_PREFS:
      prefs_dialog();
      break;
    case MENU_PREF_PERSONS:
      personality_dialog();
      break;
    case MENU_PREF_USER:
      user_dialog(config.currpersonality);
      break;
    case MENU_PREF_SERVERS:
      server_dialog(config.currpersonality);
      break;
    case MENU_PREF_SIGNATURE:
      signature_dialog(config.currpersonality);
      break;
    case MENU_PREF_ADDRBOOK:
      wInfoMsg(szUnimplemented);
      break;
    case MENU_PREF_COLUMNS:
      columns_dialog();
      SwitchFont(config.currfontidx);
      lastbflags=0;
      RedrawEverything();
      break;
    case MENU_CTRL_SEND:
    case MENU_CTRL_CHECK:
    case MENU_CTRL_ONLINE:
    case MENU_CTRL_OFFLINE:
      wInfoMsg(szUnimplemented);
      break;
    case MENU_ZOOM_IN:
    case MENU_ZOOM_OUT:
      SwitchFont(index);
      lastbflags=0;
      RedrawEverything();
      break;
    case MENU_EXIT:
      ExitApplication();
      break;
    case MENU_VIEW_IN:
      SwitchMode(MODE_IN);
      break;
    case MENU_VIEW_OUT:
      SwitchMode(MODE_OUT);
      break;
    case MENU_VIEW_TRASH:
      SwitchMode(MODE_TRASH);
      break;
    case MENU_VIEW_RXED:
      SwitchMode(MODE_RXED);
      break;
    case MENU_VIEW_TXED:
      SwitchMode(MODE_TXED);
      break;
  }
}


/*******************************************************************************
***
*** Function         : ProcessSystemCommand
*** Preconditions    : 
*** Postconditions   : Respond to message from System Screen
***
*******************************************************************************/

LOCAL_C VOID ProcessSystemCommand(VOID)
{ 
TEXT buf[P_FNAMESIZE+2];

  wGetCommand((UBYTE *)&buf[0]);
  if (buf[0]=='X')
    ExitApplication();
}


/*******************************************************************************
***
*** Function         : TryExecuteCommand
*** Preconditions    : 
*** Postconditions   : Try to find command to execute matching accelerator 
***                    passed
***
*******************************************************************************/

LOCAL_C VOID TryExecuteCommand(INT keycode)
{
  keycode=uLocateCommand(keycode);        /* look up accelerator table */
  if (keycode>=0)                 /* found - keycode converted to index */
    ManageCommand(keycode);
}


/*******************************************************************************
***
*** Function         : key_process
*** Preconditions    : A keypress is passed
*** Postconditions   : It's handled.
***
*******************************************************************************/

LOCAL_C VOID key_process(VOID)
{
INT ret, code;
  if (key.keycode & W_EVENT_KEY) {
    switch (key.keycode) {
      case CONS_EVENT_FOREGROUND:
        break;
      case CONS_EVENT_COMMAND:
        ProcessSystemCommand();
        break;
    }
  }
  else if (key.keycode & W_SPECIAL_KEY) {
    /* Psion key pressed */
    code = key.keycode & (~W_SPECIAL_KEY);
    if (key.modifiers & W_SHIFT_MODIFIER)
      code = p_toupper(code);
    TryExecuteCommand(code); /* Depends which menu is selected */
  }
  else {
    switch (key.keycode) {
      case W_KEY_DIAMOND:
        if (key.modifiers & W_SHIFT_MODIFIER)
          SwitchMode(MODE_PREV);
        else
          SwitchMode(MODE_NEXT);
        break;
      case W_KEY_MENU:
        if (key.modifiers & W_CTRL_MODIFIER) {
          SetGmode(G_TRMODE_CLR);
          gBorder(lastbflags&~(W_BORD_TOP_OFF|W_BORD_BOT_OFF));
          SetGmode(G_TRMODE_SET);
          CycleStatusWindow();
          config_save();
        }
        else {           /* display menu bar */
          /* For more than one menu palette, change _cmds and _mdata first... */

          ret=uPresentMenus();
          if (ret>0)
            TryExecuteCommand(ret); /* And call a different menu handler */
        }
        break;
      case W_KEY_UP:
        if (cent)
          CursorMove(cent-1);
        break;
      case W_KEY_DOWN:
        if (cent<mailbox.th.th_nummsgs-1)
          CursorMove(cent+1);
        break;
      case W_KEY_PAGE_UP:
        if (key.modifiers&W_CTRL_MODIFIER)
          CursorMove(0);
        else if (!tent)
          CursorMove(0);
        else
          ScrollPageBy(-tent);
        break;
      case W_KEY_PAGE_DOWN:
        if (key.modifiers&W_CTRL_MODIFIER)
          CursorMove(mailbox.th.th_nummsgs-1);
        else if (tent+hent>=mailbox.th.th_nummsgs)
          CursorMove(mailbox.th.th_nummsgs-1);
        else
          ScrollPageBy(mailbox.th.th_nummsgs-tent-hent);
        break;
      case W_KEY_RETURN:
        copy_msg_fire_composer();
        break;
      default:
        /* Do something with key.keycode */
        break;
    }
  }
}

/*******************************************************************************
***
*** Function         : stackevent_process()
*** Preconditions    : Some stack event has occurred. The event code is stored 
***                    in stackevent_stat; data is in stackevent_buffer.
*** Postconditions   : 
***
*******************************************************************************/

LOCAL_C VOID stackevent_process(VOID)
{
  switch (stackevent_stat) {
    case EVT_LINEDROPPED:
      wInfoMsg("Line dropped");
      break;
    case EVT_SCRIPTING:
      wInfoMsg("Starting script");
      break;
    case EVT_LINK_UP:
      wInfoMsg("Negotiating");
      break;
    case EVT_NET_UP:
      wInfoMsg("Online");
      break;
    case EVT_LINK_DOWN:
      wInfoMsg("Closing connection"); 
      break;
    case EVT_NET_DOWN:
      wInfoMsg("Offline");
      break;
    case EVT_NEW_METHOD:
      wInfoMsg("Selected method...");
      break;
    case EVT_UI_MESSAGE:
      wInfoMsg(stackevent_buffer);
      break;
    case EVT_TERM:
      wInfoMsg("Terminal mode");
      break;
    case EVT_UI_TERMDATA:
      /*p_write(winHandle, stackevent_buffer, stackevent_retlength);*/
      break;
    default:
      wInfoMsg("stackevent_process bug");
      console_printf("stackevent_process: unknown event 0x%04x\r\n", stackevent_stat);
      break;
  }
}


/*******************************************************************************
***
*** Function         : CheckBorderArrows
*** Preconditions    : 
*** Postconditions   : Check if the border needs redisplaying on account of
***                    scroll arrows
***
*******************************************************************************/

LOCAL_C VOID CheckBorderArrows(VOID)
{
INT bflags=0; /* the flags that ought to be displayed */

  if (tent)       /* check if more entries off screen top */
    bflags|=W_BORD_TOP_ON;
  else
    bflags|=W_BORD_TOP_OFF;
  if (tent+hent<mailbox.th.th_nummsgs) /* entries off screen bottom */
    bflags|=W_BORD_BOT_ON;
  else
    bflags|=W_BORD_BOT_OFF;
  if (bflags!=lastbflags)     /* check if border needs to be redrawn */
    gBorder(lastbflags=bflags);
}


/*******************************************************************************
***
*** Function         : RedrawEverythng
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

LOCAL_C VOID RedrawEverything(VOID)
{
  ClearWholeWindow();
  CheckBorderArrows();
  DrawTitles();
  DrawList();
}

/*******************************************************************************
***
*** Function         : MainLoop
*** Preconditions    : The system has been initialised.
*** Postconditions   : This is the main loop - control never passes out.
***
*******************************************************************************/

LOCAL_C VOID MainLoop(VOID)
{
  FOREVER {
    CheckBorderArrows();    /* update border arrow display if changed */
    if (key_active)
      wFlush();
    else {
      uGetKeyA(&key_stat,&key);
      key_active=TRUE;
    }

    /* Re-queue stack event request */
    if (!stackevent_requestqueued) {
      if (stack_queue_event_request(0xffff, &stackevent_stat, 
                                            &stackevent_buffer, 
                                            MAX_STACKEVENTBUFFER,
                                            &stackevent_retlength) < 0) {
        wInfoMsg("Can't queue stack event request!");
        Terminate(0); 
      }
      stackevent_requestqueued=TRUE;
    }
      
    p_iowait();

    /* Keyboard? */
    if (key_stat != E_FILE_PENDING) {
      key_process();
      key_active = FALSE;
    }

    /* Stack? */
    if (stackevent_stat != E_FILE_PENDING) {
      stackevent_process();
      stackevent_requestqueued = FALSE;
    }
  }
}



/*******************************************************************************
***
*** Function         : CreateGC
*** Preconditions    : 
*** Postconditions   : Attempt to create default graphics context on main 
***                    window
***
*******************************************************************************/

LOCAL_C VOID CreateGC(VOID)
{
INT hgc;        /* graphics context handle (if successful) */
  hgc=gCreateGC0(MainWid);
  if (hgc<0)
    Terminate(hgc);
}


/*******************************************************************************
***
*** Function         : SetupConsoleFont()
*** Preconditions    : Font is set to a font, machine_fullscreencols,
***                    machine_fullscreenrows is the full of the screen.
*** Postconditions   : The console font is changed, and the console resized.
***
*******************************************************************************/

VOID SetupConsoleFont(VOID)
{
UWORD func;
P_RECT rect;
  /* Change the console font. Can't do this on MCx00's */
  if (machine_type != MACH_MC200 && machine_type != MACH_MC400) {
    /* Set the font */
    func = P_SCR_FONT;
    p_iow(winHandle, P_FSET, &func, &Font);

    /* Set the console size and position */
    func = P_SCR_WSET;
    rect.tl.x=0;
    rect.tl.y=0;
    rect.br.x=machine_fullscreencols;
    rect.br.y=machine_fullscreenrows;
    if (p_iow(winHandle, P_FSET, &func, &rect)!=0)
      wInfoMsg("Couldn't resize console");
  }
}


/*******************************************************************************
***
*** Function         : CursorOff
*** Preconditions    : 
*** Postconditions   : The Cursor is Off
***
*******************************************************************************/

LOCAL_C VOID CursorOff(VOID)
{
UWORD pflag,func;
  pflag = FALSE;
  func = P_SCR_CURSOR;
  p_iow(winHandle, P_FSET, &func, &pflag);
}



/*******************************************************************************
***
*** Function         : SpecificInit
*** Preconditions    : HWIF has been initialised
*** Postconditions   : The rest of the system is initialised.
***
*******************************************************************************/

LOCAL_C VOID SpecificInit(VOID)
{
LOCAL_D INT gc;
P_EXTENT pextent;

  p_unmarka();
  _GreyLines = &lines[0];
  p_bfil(&lines[0], sizeof(lines), 0);
  uAddGreyUline('Z',&lines[0]);
  uAddGreyUline('O',&lines[0]);
  uAddGreyUline('R',&lines[0]);
  uAddGreyUline('F',&lines[0]);
  uAddGreyUline('m',&lines[0]);
  uAddGreyUline('n',&lines[0]);
  uAddGreyUline('i',&lines[0]);
  uAddGreyUline('e',&lines[0]);

  gc = gCreateGC0(uFindMainWid());
  if (_UseFullScreen) {
    /* Running on a Series 3a or Workabout */
    wFree(gc);
    uEnableGrey();
    gc = gCreateGC0(uFindMainWid());
  }

  /* Which type of machine are we running on? */
  machine_check();
  if (machine_type == MACH_UNKNOWN) {
    wInfoMsg("Unknown machine!");
    Terminate(0); /* probably don't want this here? */
  }

  /* Set initial configuration and limits based on machine type */
  switch (machine_type) {
    case MACH_SIENA:
      config.statwinstate = W_STATUS_WINDOW_SMALL;
      break;
    case MACH_EM_SIENA:
      config.statwinstate = W_STATUS_WINDOW_SMALL;
      break;
    case MACH_S3a:
      config.statwinstate = W_STATUS_WINDOW_BIG;
      break;
    case MACH_EM_S3a:
      config.statwinstate = W_STATUS_WINDOW_BIG;
      break;
    case MACH_S3c:
      config.statwinstate = W_STATUS_WINDOW_BIG;
      break;
    case MACH_S3:
      config.statwinstate = W_STATUS_WINDOW_SMALL;
      break;
    case MACH_MC400:
      config.statwinstate = W_STATUS_WINDOW_BIG;
      break;
    case MACH_MC200:
      config.statwinstate = W_STATUS_WINDOW_BIG;
      break;
    case MACH_HC:
      config.statwinstate = W_STATUS_WINDOW_SMALL;
      break;
    case MACH_WORKABOUT:
      config.statwinstate = W_STATUS_WINDOW_SMALL;
      usesmalldialogs = TRUE;
      _UseFullScreen = TRUE;
      break;
    case MACH_EM_WORKABOUT:
      config.statwinstate = W_STATUS_WINDOW_SMALL;
      usesmalldialogs = TRUE;
      _UseFullScreen = TRUE;
      break;
  }

  uEscape(FALSE);     /* disable Psion-Esc */
  MainWid=uFindMainWid();     /* find id of main display window */
  CreateGC();                 /* create default graphics context */

  /* We could analyse the command line here... */
  hCrackCommandLine();

  wsEnable();                 /* display status window */
  /*wsSetList(MODE_MAX,&modelist[0],currmode);*/

  if (stack_loaded() < 0) {
    if (stack_load() < 0) {
      wInfoMsg("Couldn't load TCP/IP stack");
      Terminate(0);
    }
    else 
      wInfoMsg("TCP/IP stack loaded");
  }

  /* Initialise everything with blank data */
  config_init();

  /* Find out what size status window we want, font type, font size,
     amongst other stuff */
  config_load();
  wsSetList(config.currmaxmode+1,&modelist[0],config.currmode);

  /* Check all mailboxes to see if they exist, create TOCs if only MBXs exist
     or if the TOC is out of date */
  if (!MailboxesOK())
    Terminate(0);


  wInquireStatusWindow(config.statwinstate, &pextent);
  if (ResizeWidth(machine_fullscreenwidth - pextent.width))
    Terminate(0);
  wStatusWindow(config.statwinstate);

  SwitchFont(config.currfontidx); /* initialises screen metrics */
  CursorOff();

  /* Load from the selected mailbox */
  SwitchMailbox(config.currmode);

  RedrawEverything();
}


/*******************************************************************************
***
*** Function         : main
*** Preconditions    : Zilch-o
*** Postconditions   : Happy user.
***
*******************************************************************************/

GLDEF_C VOID main(VOID)
{
  _UseFullScreen = TRUE;
  uCommonInit();
  SpecificInit();
  MainLoop();
}
