/*******************************************************************************
***
*** Filename         : toc.c
*** Purpose          : Table of COntents routines
*** Author           : Matt J. Gumbley
*** Created          : 13/07/98
*** Last updated     : 08/02/98
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#include <p_std.h>          /* basic Psion header file */
#include <p_file.h>         /* defines E_FILE_ error values, etc */
#include <p_date.h>         /* defines date and time functions */
#include <p_config.h>       /* defines E_CONFIG struct */
#include <epoc.h>           /* general Epoc definitions */
/*#include <wlib.h> */          /* defines al Window Server library functions */
/*#include <hwif.h>    */       /* defines all Hwif library functions */

#include "toc.h"

#include "\sibosdk\include\psistack\console.h"     /* Console API definitions */
#define min(a,b) ((a)<(b)?(a):(b))

GLDEF_D TEXT *msg_filename="LOC::M:\\EMAIL3\\MESSAGE.TXT";

LOCAL_D TEXT *toc_folder="\\EMAIL3\\";

LOCAL_C VOID toc_buildnames(struct toc *t, TEXT driveletter, TEXT *basename)
{
  p_atos(t->mbxname, "LOC::%c:%s%s.MBX", driveletter, toc_folder, basename);
  p_atos(t->tocname, "LOC::%c:%s%s.TOC", driveletter, toc_folder, basename);
}

LOCAL_C ULONG toc_convertdate(TEXT *datestr)
{
P_DATE pdate;
P_DAYSEC pdaysec;
ULONG systime=0L;
WORD day,month,year,hour,minute,second,corr;
INT i;
static TEXT *monabbrevs[12]= {
  "Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"
};

  /* datestr will be an RFC822 standard (?) date string, like this:
     Wed, 29 Jul 1998 09:32:34 +0000
     Convert it to the number of seconds since 00:00:00 Jan 1, 1970 
     The syntax is something like this: (in a Perl regexp stylee)
     \s*
     ((Mon|Tue|Wed|Thu|Fri|Sat|Sun),?\s+)?
     \d{1,2}\s+
     (Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\s+
     \d\d(\d\d)?\s+
     \d{1,2}:\d{1,2}:\d{1,2}\s+
     ([-+]\d{4}\s+)?
     \S*
     Of course, we don't parse it fully, that would be a total 'mare...
  */
  corr=0;

  /* Skip WS */
  while (*datestr && p_isspace(*datestr)) datestr++;
  if (!*datestr)
    return systime;

  /* There's an optional day name and optional comma, which we don't need,
     so skip to the next digit, which will be the day number. */
  while (*datestr && !p_isdigit(*datestr)) datestr++;
  if (!*datestr)
    return systime;
  if (p_stoi(&datestr,&day)!=0)
    return systime;
  /* Make day start at 0 */
  day--;

  /* datestr should now be on the WS between day and month. Skip to Month */
  while (*datestr && p_isspace(*datestr)) datestr++;
  if (!*datestr)
    return systime;
  /* I could use p_nmmona (p.110) here. But RFC822 probably demands English
     month names... */
  for (i=0; i<12; i++)
    if (p_bcmpi(monabbrevs[i], 3, datestr, 3)==0)
      break;
  if (i==12)
    return systime;
  month=i;

  /* Advance to the next digit (the year). Year can be two or four digits */
  while (*datestr && !p_isdigit(*datestr)) datestr++;
  if (!*datestr)
    return systime;
  if (p_stoi(&datestr,&year)!=0)
    return systime;
  /* Convert years to years since 1900. 2-digit years are pivoted at 1970
     That is, any year less than 70 is 20xx, otherwise it's 19xx.
     e.g. 00 is 2000, 35 is 2035, 69 is 2069, 70 is 1970, 99 is 1999

     I could do if (year > 99) here, but if you get sent a mail from a
     mailer with a Y2K problem, in 2000, it might be dated 100... so move
     the four-digit check into this century

     Of course, if someone sends you a mail from 1856, then you have a 
     serious problem. With the arrow of time.

     Also, since system time goes from 00:00 1/1/1970, you can't use dates
     before then.
  */
  if (year>=1900) {
    year -= 1900; /* year is [0..maxint-1900] (which gets truncated to 255) */
  }
  else {
    /* Assume a two-digit year */
    if (year>=70) {
      /* this is fine, what we expect for years [1970..xxxx] ( [70..xxxx] ) */
    }
    else {
      /* pivot */
      year += 100; /* So 0=100, 69=169, 70=70 */
    }
  }
  /* Year is now [0..255]. Phew! */


  /* Now on the WS after the year. Advance to the next digit (hour) */
  while (*datestr && !p_isdigit(*datestr)) datestr++;
  if (!*datestr)
    return systime;
  if (p_stoi(&datestr,&hour)!=0)
    return systime;
  /* Now on the hh:mm colon */
  if (*datestr!=':')
    return systime;
  datestr++;
  /* Now on the minute */
  if (p_stoi(&datestr,&minute)!=0)
    return systime;
  /* Now on the mm:ss colon */
  if (*datestr!=':')
    return systime;
  datestr++;
  /* Now on the second */
  if (p_stoi(&datestr,&second)!=0)
    return systime;

  /* Now on the WS following the time. We may have a +|-corr. 
     All the rest of the date is probably optional, so don't have a cow if it 
     isn't there.
  */
  while (*datestr && p_isspace(*datestr)) datestr++;
  if (*datestr=='-' || *datestr=='+') {
    p_stoi(&datestr,&corr);
  }

  /* OK, let's do the conversions. First, put all this stuff into a P_DATE */
  pdate.year=year & 0x00ff;
  pdate.month=month & 0x00ff;
  pdate.day=day & 0x00ff;
  pdate.hour=hour & 0x00ff;
  pdate.minute=minute & 0x00ff;
  pdate.second=second & 0x00ff;
  /* Convert to a P_DAYSEC */
  p_dttods(&pdate,&pdaysec);
  /* And convert to system time */
  p_dstost(&pdaysec,&systime);
  return systime;
}


LOCAL_C INT toc_readmsg(struct toc *t)
{
INT inHeader = 1;
INT inBody = 0;
INT gotFrom = 0;
#define LINELENGTH 256
static TEXT line[LINELENGTH];
static LONG pos=0L;
LONG size;
static INT unget=0;
static INT len;
static INT firstEOF=1;
TEXT *s, *d;
INT i;
  p_bfil(&t->tm, sizeof(struct tocmsg), 0);
  /* Read every line. We might have read the previous messages' From header 
     last time this routine was called. */
  for (;;) {
    if (unget) {
      t->tm.tm_offset=pos;
      unget=0;
    }
    else {
      if ((len=p_read(t->mbxfd, line, LINELENGTH)) == E_FILE_EOF) {
        p_seek(t->mbxfd, P_FRSENSE, &size);
        t->tm.tm_size=size - t->tm.tm_offset;
        if (firstEOF) {
          firstEOF = 0;
          return 1;
        }
        return 0;
      }
      p_seek(t->mbxfd, P_FRSENSE, &pos); /* pos of start of just-read line? */
    }
    /* Line might have been cached as ungot, or read */
    line[len]='\0';
    if (len == 0) {
      inBody=1;
      inHeader=0;
      continue;
    }
    if (p_bcmp(line,min(len,5),"From ",5)==0) {
      inHeader=1;
      if (gotFrom) {
        /* cached */
        unget=1;
        break;
      }
      else {
        t->tm.tm_offset=pos;
        gotFrom=1;
      }
      continue;
    }
    if (inHeader && p_bcmp(line,min(len,9),"Subject: ",9)==0) {
      line[TOCMSG_SUBJECT_SIZE-1]='\0';
      p_scpy(t->tm.tm_subject,line+9);
      continue;
    }
    if (inHeader && p_bcmp(line,min(len,6),"From: ",6)==0) {
      /* Skip " characters, stop at < */
      line[TOCMSG_WHO_SIZE-1]='\0';
      for (i=0,s=line+6,d=t->tm.tm_who; i<TOCMSG_WHO_SIZE && *s; s++,d++) {
        if (*s == '"') {
          d--;
          continue;
        }
        if (*s == '<') {
          if (i>0 && *(d-1)==' ')
            *(d-1)='\0';
          break;
        }
        *d=*s;
      }
      *d='\0';
      continue;
    }
    if (inHeader && p_bcmp(line,min(len,6),"Date: ",6)==0) {
      /*line[TOCMSG_SUBJECT_SIZE-1]='\0';
      p_scpy(t->tm.tm_subject,line+6);*/
      t->tm.tm_date=toc_convertdate(line+6);
      continue;
    }
    if (inBody) {
    }
  }
  p_seek(t->mbxfd, P_FRSENSE, &size);
  t->tm.tm_size=size - t->tm.tm_offset;
  return 1;
}

GLDEF_C INT toc_copymsg(struct toc *t, VOID *outfd)
{
INT errcode;
#define CMBUFSIZE 128
UBYTE buf[CMBUFSIZE];
UINT read,toread;
ULONG pos, left;

  pos=t->tm.tm_offset;
  left=t->tm.tm_size;
  p_seek(t->mbxfd,P_FABS,&pos);
  while (left != 0) {
    toread = (left > CMBUFSIZE) ? CMBUFSIZE : (UINT)left;
    read=p_read(t->mbxfd, &buf, toread);
    if (read<0) {
      console_puts("toc_copymsg read failed");
      return E_TOC_MSGCOPYFAIL;
    }
    left -= read;
    errcode=p_write(outfd, buf, read);
    if (errcode<0) {
      console_puts("toc_copymsg write failed");
      return E_TOC_MSGCOPYFAIL;
    }
  }
  return 1;
}

GLDEF_C VOID toc_displaydate(TEXT *buf, ULONG mtime)
{
ULONG ttime;
P_DATE tdate; /* today's date */
P_DAYSEC tdaysec;

P_DAYSEC mdaysec;
P_DATE mdate; /* mail date, as converted from mtime parameter */

  p_sttods(&mtime, &mdaysec);
  p_dstodt(&mdaysec, &mdate);

  ttime = p_date();
  p_sttods(&ttime, &tdaysec);
  p_dstodt(&tdaysec, &tdate);

  /* If the mail came today, give the time as 01:34pm
     In the last 7 days, give time as Thu 01:34pm
     Before the last seven days, give time as 15/07/1998 */
  if (tdaysec.day == mdaysec.day) {
    /* Max size of string: 00:00am == 7 */
    p_st2str(buf, "%6%:%T%7", &mtime); 
    return;
  }
  if (mdaysec.day >= (tdaysec.day - 7)) {
    /* The max size of the string isn't known, since the length of the day
       abbreviation is country-dependent. We just hope it's small enough to 
       display. If this causes problems, mail me, and I'll increase the 
       column width. Width >= 7, probably Thu 01:00a == 10 
    */
    p_st2str(buf, "%*E %6%:%T%*7", &mtime); 
    return;
  }
  /* Max width is 01/01/1998 == 10 */
  p_st2str(buf, "%1%/%2%/%3", &mtime);
}

GLDEF_C INT toc_direxists(TEXT driveletter)
{
TEXT buf[40];
  p_atos(buf, "LOC::%c:%s", driveletter, toc_folder);
  return p_testpth(buf) == 0;
}


GLDEF_C INT toc_mkdir(TEXT driveletter)
{
TEXT buf[40];
  p_atos(buf, "LOC::%c:%s", driveletter, toc_folder);
  return p_mkdir(buf);
}


GLDEF_C INT toc_create(struct toc *t, BYTE type, TEXT driveletter, TEXT *basename, VOID (*progress)(ULONG, ULONG))
{
INT errcode;
P_INFO info;
LONG pos;

  toc_buildnames(t, driveletter, basename);

  /* Open the mailbox file */
  errcode=p_open(&t->mbxfd,t->mbxname,P_FOPEN|P_FTEXT|P_FRANDOM);
  if (errcode) {
    return E_TOC_MBXOPENFAIL;
  }

  /* Create the toc file */
  errcode=p_open(&t->tocfd,t->tocname,P_FREPLACE|P_FSTREAM|P_FUPDATE|P_FRANDOM);
  if (errcode) {
    p_close(t->mbxfd);
    return E_TOC_TOCCREATEFAIL;
  }

  /* Create blank TOC header */
  t->th.th_type=type;
  t->th.th_nummsgs=0;
  t->th.th_mbxdatestamp=0; /* if the toc_create crashes, it'll be forced next time */
  errcode=p_write(t->tocfd, (VOID *)&t->th, sizeof(struct tochdr));
  if (errcode) {
    errcode = E_TOC_TOCHDRWRITEFAIL;
    goto wipe;
  }

  /* How large is the mailbox file? (For the displayed bar gauge percentage) */
  p_finfo(t->mbxname, &info);

  
  /* Now scan through the mailbox, adding TOC entries. */
  while (toc_readmsg(t)) {
    (*progress)(info.size,t->tm.tm_offset);
    t->th.th_nummsgs++;
    errcode=p_write(t->tocfd, (VOID *)&t->tm, sizeof(struct tocmsg));
    if (errcode) {
      errcode = E_TOC_MSGWRITEFAIL;
      goto wipe;
    }
  }

  /* Rewind and write the finished header */
  pos=0L;
  p_seek(t->tocfd,P_FABS,&pos);
  p_finfo(t->mbxname, &info);
  t->th.th_mbxdatestamp=info.modst;
  errcode=p_write(t->tocfd, (VOID *)&t->th, sizeof(struct tochdr));
  if (errcode) {
    errcode = E_TOC_TOCHDRWRITEFAIL;
    goto wipe;
  }

  p_close(t->mbxfd);
  p_close(t->tocfd);
  return 1;

wipe:
  /* Everything went pear-shaped. Close files, remove TOC */
  p_close(t->tocfd);
  p_close(t->mbxfd);
  p_delete(t->tocname);
  return errcode;
}


GLDEF_C INT toc_format(struct toc *t, BYTE type, TEXT driveletter, TEXT *basename)
{
INT errcode;
P_INFO info;

  toc_buildnames(t, driveletter, basename);

  /* Create the mailbox file */
  errcode=p_open(&t->mbxfd,t->mbxname,P_FREPLACE|P_FTEXT|P_FRANDOM);
  if (errcode) 
    return E_TOC_MBXCREATEFAIL;

  /* Create the toc file */
  errcode=p_open(&t->tocfd,t->tocname,P_FREPLACE|P_FSTREAM|P_FUPDATE|P_FRANDOM);
  if (errcode) {
    p_close(t->mbxfd);
    p_delete(t->mbxname);
    return E_TOC_TOCCREATEFAIL;
  }

  /* Create blank TOC header */
  t->th.th_type=type;
  t->th.th_nummsgs=0;
  p_finfo(t->mbxname, &info);
  t->th.th_mbxdatestamp=info.modst;
  errcode=p_write(t->tocfd, (VOID *)&t->th, sizeof(struct tochdr));
  if (errcode) {
    errcode = E_TOC_TOCHDRWRITEFAIL;
    goto wipe;
  }
  
  p_close(t->mbxfd);
  p_close(t->tocfd);
  return 1;

wipe:
  /* Everything went pear-shaped. Close files, remove TOC */
  p_close(t->tocfd);
  p_close(t->mbxfd);
  p_delete(t->tocname);
  p_delete(t->mbxname);
  return errcode;
}
 
GLDEF_C INT toc_open(struct toc *t, TEXT driveletter, TEXT *basename)
{
INT errcode;
P_INFO info;
  toc_buildnames(t, driveletter, basename);
  /* Open the mailbox file */
  errcode=p_open(&t->mbxfd,t->mbxname,P_FOPEN|P_FTEXT|P_FRANDOM);
  if (errcode) {
    return E_TOC_MBXOPENFAIL;
  }

  /* When was the MBX opened? Date stamp in TOC should match... */
  p_finfo(t->mbxname, &info);

  /* Open the toc file */
  errcode=p_open(&t->tocfd,t->tocname,P_FOPEN|P_FSTREAM|P_FUPDATE|P_FRANDOM);
  if (errcode) {
    p_close(t->mbxfd);
    return E_TOC_TOCOPENFAIL;
  }

  errcode=p_read(t->tocfd, &t->th, sizeof(struct tochdr));
  if (t->th.th_mbxdatestamp != info.modst) {
    p_close(t->mbxfd);
    p_close(t->tocfd);
    return E_TOC_TOCSTALE;
  }
  t->toc_recno = -1;
  t->toc_open = 1;
  return 1;
}

GLDEF_C INT toc_readrandom(struct toc *t, INT recno)
{
LONG pos;
INT errcode;
  if (t->toc_recno == recno)
    return 1; /* Already read */
  if (t->toc_recno == recno -1)
    return toc_read(t); /* Read next */
  /* Position then read */
  pos = sizeof(struct tochdr)+(recno*sizeof(struct tocmsg));
  p_seek(t->tocfd,P_FABS,&pos);
  t->toc_recno = recno;
  errcode = p_read(t->tocfd, &t->tm, sizeof(struct tocmsg));
  if (errcode < 0)
    return E_TOC_MSGREADFAIL;
  return 1;
}

GLDEF_C INT toc_read(struct toc *t)
{
INT errcode;
  t->toc_recno++;
  errcode = p_read(t->tocfd, &t->tm, sizeof(struct tocmsg));
  if (errcode < 0)
    return E_TOC_MSGREADFAIL;
  return 1;
}

GLDEF_C INT toc_close(struct toc *t)
{
INT errcode = 1;
LONG pos = 0L;
P_INFO info;
  if (!t->toc_open)
    return 0;

  /* When was the MBX opened? Date stamp in TOC should match... */
  p_finfo(t->mbxname, &info);
  t->th.th_mbxdatestamp = info.modst;

  p_seek(t->tocfd,P_FABS,&pos);
  errcode=p_write(t->tocfd, (VOID *)&t->th, sizeof(struct tochdr));
  if (errcode) 
    errcode = E_TOC_TOCHDRWRITEFAIL;
  p_close(t->mbxfd);
  p_close(t->tocfd);
  t->toc_open = 0;
  return errcode;
}

