/*******************************************************************************
***
*** Filename         : toc.h
*** Purpose          : Table Of Contents file format and definitions
*** Author           : Matt J. Gumbley
*** Created          : 13/07/98
*** Last updated     : 02/08/98
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#ifndef TOC_H
#define TOC_H


/* 8421842184218421
   SSSSPPPAFD-
*/
#define STATUS_STATMASK    0xF000 /* unique values */
#define STATUS_STATUNREAD  0x0000
#define STATUS_STATREAD    0x1000
#define STATUS_STATREPLY   0x2000
#define STATUS_STATFWD     0x3000
#define STATUS_STATREDIR   0x4000
#define STATUS_STATSENT    0x5000
#define STATUS_STATQUEUED  0x6000
#define STATUS_STATTQUEUED 0x7000
#define STATUS_STATDRAFTED 0x8000
#define STATUS_STATDELETED 0x9000


#define STATUS_PRIMASK     0x0E00 /* unique values */
#define STATUS_PRIHIGHEST  0x0000
#define STATUS_PRIHIGH     0x0200
#define STATUS_PRINORMAL   0x0400
#define STATUS_PRILOW      0x0600
#define STATUS_PRILOWEST   0x0800

#define STATUS_ATTACH      0x0100

#define STATUS_SERVMASK    0x00C0 /* bit values */
#define STATUS_SERVFETCH   0x0080
#define STATUS_SERVDELETE  0x0040

#define TOCMSG_WHO_SIZE 40
#define TOCMSG_SUBJECT_SIZE 64

struct tocmsg {
  UWORD tm_status;
  ULONG tm_offset; /* or an offset? */
  ULONG tm_size; /* or an offset? */
  ULONG tm_date;
  TEXT  tm_who[TOCMSG_WHO_SIZE];
  TEXT  tm_subject[TOCMSG_SUBJECT_SIZE];
};

#define TYPE_IN 0
#define TYPE_OUT 1
#define TYPE_TRASH 2
#define TYPE_RCVD 3
#define TYPE_SENT 4
#define TYPE_USER 5

struct tochdr {
  BYTE  th_type; /* TYPE_???? */
  UWORD th_nummsgs; 
  ULONG th_mbxdatestamp; /* mod. date of MBX file, for detecting stale TOC */
};
  
struct toc {
  TEXT mbxname[40];
  TEXT tocname[40];
  VOID *mbxfd;
  VOID *tocfd;
  INT toc_recno;
  INT toc_open;
  struct tochdr th;
  struct tocmsg tm;
};

#define E_TOC_MBXCREATEFAIL   -1
#define E_TOC_MBXOPENFAIL     -2
#define E_TOC_TOCCREATEFAIL   -3
#define E_TOC_TOCOPENFAIL     -4
#define E_TOC_TOCHDRWRITEFAIL -5
#define E_TOC_MSGWRITEFAIL    -7
#define E_TOC_MSGREADFAIL     -8
#define E_TOC_TOCSTALE        -9
#define E_TOC_MSGCOPYFAIL     -10

GLREF_D TEXT *msg_filename;

GLREF_C VOID toc_displaydate(TEXT *buf, ULONG time);
GLREF_C INT toc_copymsg(struct toc *t, VOID *outfd);
GLREF_C INT toc_direxists(TEXT driveletter);
GLREF_C INT toc_mkdir(TEXT driveletter);
GLREF_C INT toc_create(struct toc *t, BYTE type, TEXT driveletter, TEXT *basename, VOID (*progress)(ULONG, ULONG));
GLREF_C INT toc_format(struct toc *t, BYTE type, TEXT driveletter, TEXT *basename);
GLREF_C INT toc_open(struct toc *t, TEXT driveletter, TEXT *basename);
GLREF_C INT toc_close(struct toc *t);
GLREF_C INT toc_readrandom(struct toc *t, INT recno);
GLREF_C INT toc_read(struct toc *t);
GLREF_C INT toc_close(struct toc *t);

#endif /* TOC_H */
