/*******************************************************************************
***
*** Filename         : telnet.c
*** Purpose          : PsiStack Telnet Client Application
*** Author           : Matt J. Gumbley
*** Created          : 09/07/98
*** Last updated     : 09/07/98
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#include "..\..\library\utils\sysincl.h"
#include "..\..\library\utils\machine.h"
#include "..\..\library\utils\version.h"
#include "..\..\include\psistack\sys\stack.h" /* Kernel structure definitions */
#include "..\..\include\psistack\stack.h"           /* Kernel API definitions */
#include "..\..\include\psistack\socket.h"         /* Sockets API definitions */
#include "..\..\include\psistack\events.h"          /* Kernel API definitions */
#include "..\..\include\psistack\console.h"        /* Console API definitions */
#include "..\..\library\utils\utils.h"  /* IP address conversion & validation */

GLREF_D UWORD DatLocked;            /* non-zero when application "busy" */
GLREF_D TEXT *DatUsedPathNamePtr;   /* full path name of current file */
GLREF_D UWORD _UseFullScreen, _SmallFontDialog;
GLREF_D VOID *winHandle;

#define MENU_CON_CONNECT 0
#define MENU_CON_DISCONN (MENU_CON_CONNECT + 1)
#define MENU_CON_POP1    (MENU_CON_DISCONN + 1)
#define MENU_CON_POP2    (MENU_CON_POP1 + 1)
#define MENU_CON_POP3    (MENU_CON_POP2 + 1)
#define MENU_CON_POP4    (MENU_CON_POP3 + 1)

#define MENU_PREFS       (MENU_CON_POP4 + 1)
#define MENU_LOGSTART    (MENU_PREFS + 1)
#define MENU_LOGSTOP     (MENU_LOGSTART + 1)
#define MENU_NETUP       (MENU_LOGSTOP + 1)
#define MENU_NETDOWN     (MENU_NETUP + 1)

#define MENU_CLEAR       (MENU_NETDOWN + 1)
#define MENU_ZOOM_IN     (MENU_CLEAR + 1)
#define MENU_ZOOM_OUT    (MENU_ZOOM_IN + 1)
#define MENU_UNLOAD      (MENU_ZOOM_OUT + 1)
#define MENU_EXIT        (MENU_UNLOAD + 1)

GLREF_D UWORD *_MenuPositions;
GLREF_D UWORD *_GreyLines;
LOCAL_D UWORD lines[16];

/* the commands in the menu bar */
LOCAL_D TEXT *cmds[]=
/* abcdefghijklmnopqrstuvwxyz ABCDEFGHIJKLMNOPQRSTUVWXYZ 
   
*/
{
  /* Connection */
  "rRemote system",
  "dDisconnect",
  "1(none)\0                       ",
  "2(none)\0                       ",
  "3(none)\0                       ",
  "4(none)\0                       ",

  /* Control */
  "pPreferences",
  "lStart Log",
  "sStop Log",
  "oGo online",
  "OGo offline",

  /* Special */
  "yClear",
  "zZoom in",
  "ZZoom out",
  "KExit + Unload",
  "xExit",
  NULL
};

#define CON_ITEMS          6       /* how many items in each menu */
#define CTRL_ITEMS         5       /* how many items in each menu */
#define SPECIAL_ITEMS      5

#define ITEMS_BEFORE_CON        0
#define ITEMS_BEFORE_CTRL       (ITEMS_BEFORE_COM+CON_ITEMS)
#define ITEMS_BEFORE_SPECIAL    (ITEMS_BEFORE_CTRL+CTRL_ITEMS)

/* the menus in the menu bar */
LOCAL_D H_MENU_DATA mdata[]=
{
  "Connection", CON_ITEMS,
  "Control", CTRL_ITEMS,
  "Special", SPECIAL_ITEMS,
  NULL
};

GLDEF_D TEXT ** _cmds=(&cmds[0]);           /* Hwif code expects this */
GLDEF_D H_MENU_DATA * _mdata=(&mdata[0]);   /* Hwif code expects this */

/* ^^^ To provide multiple menu palettes (which we'll need with the full 
       data model), change these before calling uPresentMenus(), see page 12 of 
       "Programming in HWIF" */

LOCAL_D TEXT *szYes = "Yes";
LOCAL_D TEXT *szNo = "No";
LOCAL_D TEXT *szIgnore = "Ignore";
LOCAL_D TEXT *szUnimplemented = "Not implemented yet";
LOCAL_D TEXT *szConfirm = "Confirm";
LOCAL_D TEXT *szStackCommFailure = "Stack communication failed";
LOCAL_D TEXT *szConnectionNotUp = "Warning: no Network Connection";


LOCAL_D UINT MainWid;       /* ID of main window */

/* Keyboard stuff */
LOCAL_D WORD keystat;
LOCAL_D WMSG_KEY key;
LOCAL_D WORD keyactive=FALSE;

/* Stack stuff */
LOCAL_D WORD stackevent_stat;
#define MAX_STACKEVENTBUFFER 1024
/* Assume term mode, expand all */
LOCAL_D BYTE stackevent_buffer[MAX_STACKEVENTBUFFER];
LOCAL_D WORD stackevent_requestqueued=FALSE;
LOCAL_D UWORD stackevent_retlength;

/* Remote system stuff */
struct sockaddr_in remote; /* Connection details get resolved into here */
#define REMOTEHOSTSIZE 40
#define REMOTEHOSTLISTSIZE 5
TEXT remotehost[REMOTEHOSTLISTSIZE][REMOTEHOSTSIZE];
UWORD remoteport[REMOTEHOSTLISTSIZE];
UWORD remotetermtype[REMOTEHOSTLISTSIZE];
IPaddr ip_loopback  = {127,0,0,1};

/* Socket stuff */
UINT mysocket = -1;
WORD connect_stat = E_FILE_PENDING; /* socket_connecta() status word */


/* Size of status window; whether Status Window is showing. See WSERV
   manual, p83 */
GLDEF_D UWORD StatWinState;

LOCAL_D P_SCR_SET_FONT Font;        /* Font used by the console */
LOCAL_D UWORD ScreenWidth;          /* width of main display area */

/* Config filename */
LOCAL_D TEXT config_fullname[P_FNAMESIZE] =
  "LOC::M:\\APP\\PSISTACK\\TELNET.CFG";

/* Forward declarations */
LOCAL_C VOID SwitchMode(INT);
LOCAL_C VOID SetupConsoleFont(VOID);
LOCAL_C VOID HomeCursor(VOID);



VOID config_save(VOID)
{
VOID *fd = NULL;
INT errcode;

  errcode=p_open(&fd,config_fullname,P_FREPLACE|P_FSTREAM|P_FUPDATE);
  if (uErrorValue(errcode)) {
    p_print("Couldn't create \"%s\"\r\n", config_fullname+21);
    return;
  }
  p_write(fd,(VOID *)&version,sizeof(int));
  p_write(fd,(VOID *)&releaselevel,sizeof(char));
  p_write(fd,(VOID *)&StatWinState,sizeof(UWORD));
  p_write(fd,(VOID *)&remotehost,REMOTEHOSTSIZE*REMOTEHOSTLISTSIZE);
  p_write(fd,(VOID *)&remoteport,sizeof(UWORD)*REMOTEHOSTLISTSIZE);
  p_write(fd,(VOID *)&remotetermtype,sizeof(UWORD)*REMOTEHOSTLISTSIZE);
  p_close(fd);
}


VOID config_load(VOID)
{
VOID *fd = NULL;
int fileversion;
char filereleaselevel;

  if (p_open(&fd,config_fullname,P_FOPEN|P_FSTREAM|P_FSHARE) != 0) {
    p_print("Creating \"%s\"\r\n", config_fullname+21);
    config_save();
    return;
  }
  p_read(fd,(VOID *)&fileversion,sizeof(int));
  p_read(fd,(VOID *)&filereleaselevel,sizeof(char));
  if (fileversion != version || filereleaselevel != releaselevel) {
    p_print("Config file created by v%d.%02d%c\r\ncannot be used, so defaults\r\nwill be used.\r\n", fileversion / 100, fileversion % 100, filereleaselevel);
    p_close(fd);
    config_save();
    return;
  }

  p_read(fd,(VOID *)&StatWinState,sizeof(UWORD));
  p_read(fd,(VOID *)&remotehost,REMOTEHOSTSIZE*REMOTEHOSTLISTSIZE);
  p_read(fd,(VOID *)&remoteport,sizeof(UWORD)*REMOTEHOSTLISTSIZE);
  p_read(fd,(VOID *)&remotetermtype,sizeof(UWORD)*REMOTEHOSTLISTSIZE);
  p_close(fd);
}


void hexdump(byte *buf, word16 len)
{ 
char LINE[80];
long offset=0L;
long left;
int i,upto16,x,bufindex;
unsigned char b;
  bufindex = 0;
  left= len;
  while (left>0) {
    for (i=0; i<78; i++)
      LINE[i]=' ';
    LINE[9]=LINE[59]='|';
    LINE[77]='\r';
    LINE[78]='\n';
    LINE[79]='\0';
    p_atos(LINE,"%08lx",offset);
    LINE[8]=' ';
    upto16 = (left>16) ? 16 : (int)left;
    for (x=0; x<upto16; x++) {
      b = buf[bufindex++];
      LINE[11+(3*x)]=hexdig((b&0xf0)>>4);
      LINE[12+(3*x)]=hexdig(b&0x0f);
      LINE[61+x]=p_isprint((char)b) ? ((char)b) : '.';
    }
    console_print(LINE);
    offset+=16L;
    left-=16L;
  }
}

/*******************************************************************************
***
*** Function         : RemoteSystemDialog
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C INT RemoteSystemDialog()
{
H_DI_SEDIT di_host;
TEXT host[REMOTEHOSTSIZE];
IPaddr hostip;
H_DI_NUMBER di_port;
LONG port;
UWORD termtype;

UINT ni_state;
UINT conretval;

static IPaddr remotesystem   = {192,168,0,1};

  ni_state = stack_get_netifstate();
  if (ni_state < 0) {
    wInfoMsg(szStackCommFailure);
    return;
  }

  /* Warn user we're not connected, but allow dialog, since we could be
     connecting to a local server */
  if (ni_state != NIS_UP) {
    wInfoMsg(szConnectionNotUp);
  }
 
  /* Copy data in from remote*[0] */
  di_host.len=REMOTEHOSTSIZE-1;
  di_host.width=16;
  di_host.str=&host[0];
  uZTStoBCS(di_host.str, (TEXT *)remotehost);

  di_port.value=&port;
  di_port.low=1;
  di_port.high=65535L;
  port = remoteport[0];

  termtype = remotetermtype[0];

  /* Run dialog until valid */
  for (;;) {
    if (uOpenDialog("Connect to Remote System") ||
        uAddDialogItem(H_DIALOG_SEDIT, "Host IP Addr", &di_host) ||
        uAddDialogItem(H_DIALOG_NUMBER, "Port", &di_port) ||
        uAddChoiceList("TERM type", &termtype, "VT100", "ANSI", "VT52", NULL) ||
        uRunDialog()<=0)
      return FALSE;

    /* Validate */
    host[1+host[0]]=0;
    if (!IPaddrConvertsz(host+1, hostip)) 
      wInfoMsg("Bad IP address");
    else
      break;
  }

  /* All OK. Copy to main remotehost structure */
  p_scpy((TEXT *)remotehost, host+1);
  remoteport[0] = port;
  remotetermtype[0] = termtype;

  /* Adjust LRU list, if different */

  config_save();

  /* Now start a connection, asynchronously. */
  /* Convert the remote*[0] into a sockaddr_in structure */
  remote.sin_port = remoteport[0];
  p_bcpy(&remote.sin_addr, hostip, IP_ALEN);
  remote.sin_len = sizeof(struct sockaddr_in);
  remote.sin_family = AF_INET;
  p_bfil(&remote.sin_zero,8, 0);

/*hexdump((byte *)&remote, sizeof(struct sockaddr_in));*/

  conretval = socket_connecta(mysocket, 
                              (struct sockaddr *)&remote, 
                              sizeof(struct sockaddr_in), 
                              &connect_stat);
  if (conretval < 0) {
    p_print("socket_connect failed: %d\r\n", conretval);
  }
  else {
    p_print("Connecting to %s:%d...\r\n", host+1, port);
  }
  return TRUE;
}

/*******************************************************************************
***
*** Function         : Confirm
*** Preconditions    : 
*** Postconditions   : Present a "No/Yes" dialog
***
*******************************************************************************/

GLDEF_C INT Confirm(TEXT *pb)
{
  if (uOpenDialog(pb))    /* title text supplied by caller */
    return(-1);
  if (uAddButtonList(szNo,-'n',szYes,'y',NULL))
    return(-1);
  return(uRunDialog()!='y');   /* return TRUE iff user answered Yes */
}


/*******************************************************************************
***
*** Function         : HomeCursor()
*** Preconditions    : 
*** Postconditions   : As it says...
***
*******************************************************************************/

VOID HomeCursor(VOID)
{
static P_POINT home = {0,0};
static UWORD func = P_SCR_POSA;
  p_iow(winHandle, P_FSET, &func, &home);
}


/*******************************************************************************
***
*** Function         : ClearWholeWindow
*** Preconditions    : 
*** Postconditions   : Clear the entire screen and home cursor
***
*******************************************************************************/

LOCAL_C VOID ClearWholeWindow(VOID)
{
P_RECT whole;

  whole.tl.x=whole.tl.y=0;
  whole.br.x=machine_fullscreenwidth;
  whole.br.y=machine_fullscreenheight;
  gClrRect(&whole,G_TRMODE_CLR);
  HomeCursor();
}


/*******************************************************************************
***
*** Function         : SetStyle
*** Preconditions    : 
*** Postconditions   : Set the style of the current graphics context
***
*******************************************************************************/

LOCAL_C VOID SetStyle(INT style)
{
G_GC gc;

  gc.style=style;
  gSetGC(0,G_GC_MASK_STYLE,&gc);
}


/*******************************************************************************
***
*** Function         : telnet_close()
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID telnet_close(INT num)
{
  console_puts("telnet: closing socket...?");
  if (mysocket >= 0) {
    console_puts("here we go...");
    socket_close(mysocket);
  }
  config_save();
  p_exit(num);     /* Goodnight Vienna */
}

/*******************************************************************************
***
*** Function         : ExitApplication
*** Preconditions    : User wants to exit; unload is TRUE for unload; 
***                    FALSE for keep stack loaded.
*** Postconditions   : 
***
*******************************************************************************/

LOCAL_C VOID ExitApplication(int unload)
{
TEXT buf[20];
  p_atos(buf, "Exit Telnet%s?", unload ? " & Unload" : "");
  if (!Confirm(buf)) {
    if (unload)
      stack_unload();
    telnet_close(0);
  }
}


/*******************************************************************************
***
*** Function         : ManageCommand
*** Preconditions    : The user selected a menu item
*** Postconditions   : Switch on command index to appropriate function
***
*******************************************************************************/

LOCAL_C VOID ManageCommand(INT index)
{
UINT closeretval;

  switch (index) {
    case MENU_CON_CONNECT:
      if (connect_stat == 0) {
        if (Confirm("Disconnect?")) {
          closeretval = socket_close(mysocket);
          if (closeretval < 0)
            p_print("socket_close() failed: %d\r\n", closeretval);
          connect_stat = E_FILE_PENDING;
        }
        else
          break;
      }
      RemoteSystemDialog();
      break;
    case MENU_CON_DISCONN:
      if (connect_stat != 0) {
        wInfoMsg("Not connected!");
        break;
      }
      closeretval = socket_close(mysocket);
      if (closeretval < 0)
        p_print("socket_close() failed: %d\r\n", closeretval);
      connect_stat = E_FILE_PENDING;
      break;
    case MENU_CON_POP1:
      break;
    case MENU_CON_POP2:
      break;
    case MENU_CON_POP3:
      break;
    case MENU_CON_POP4:
      break;
    case MENU_PREFS:
      break;
    case MENU_LOGSTART:
      break;
    case MENU_LOGSTOP:
      break;
    case MENU_CLEAR:
      ClearWholeWindow();
      break;
    case MENU_ZOOM_IN:
    case MENU_ZOOM_OUT:
      wInfoMsg(szUnimplemented);
      break;
    case MENU_NETUP:
      /* This needs to be less coupled to the network interface states - 
         you should just be able to do stack_net_up and stack_net_down... 
         When the full data model is in, we'll have to have a choice dialog
         here, to choose a method (if the user has any in their prompt list)
      */
      if (stack_open_netif(NIS_UP) <= 0)
        wInfoMsg(szStackCommFailure);
      break;
    case MENU_NETDOWN:
      if (stack_close_netif() <= 0) 
        wInfoMsg(szStackCommFailure);
      break;
    case MENU_UNLOAD:
      ExitApplication(TRUE);
      break;
    case MENU_EXIT:
      ExitApplication(FALSE);
    break;
  }
}



/*******************************************************************************
***
*** Function         : ProcessSystemCommand
*** Preconditions    : 
*** Postconditions   : Respond to message from System Screen
***
*******************************************************************************/

LOCAL_C VOID ProcessSystemCommand(VOID)
{ 
TEXT buf[P_FNAMESIZE+2];

  wGetCommand((UBYTE *)&buf[0]);
  if (buf[0]=='X')
    ExitApplication(FALSE);
}


/*******************************************************************************
***
*** Function         : TryExecuteCommand
*** Preconditions    : 
*** Postconditions   : Try to find command to execute matching accelerator 
***                    passed
***
*******************************************************************************/

LOCAL_C VOID TryExecuteCommand(INT keycode)
{
  keycode=uLocateCommand(keycode);        /* look up accelerator table */
  if (keycode>=0)                 /* found - keycode converted to index */
    ManageCommand(keycode);
}


/*******************************************************************************
***
*** Function         : ResizeWidth
*** Preconditions    : 
*** Postconditions   : Try to change the width of the main display region 
***
*******************************************************************************/

LOCAL_C INT ResizeWidth(UWORD width)
{
INT ret;
W_WINDATA wd;

  wd.extent.tl.x=wd.extent.tl.y=0;
  wd.extent.width=width;
  wd.extent.height=machine_fullscreenheight;
  wSetWindow(MainWid,W_WIN_EXTENT,&wd);
  ret=uErrorValue(wCheckPoint());     /* check whether successful */
  if (!ret)
    ScreenWidth=width;
  return(ret);
}


/*******************************************************************************
***
*** Function         : CycleStatusWindow
*** Preconditions    : 
*** Postconditions   : Attempt to flip through the different states of the 
***                    status window 
***
*******************************************************************************/

LOCAL_C VOID CycleStatusWindow(VOID)
{
P_EXTENT pextent;
UWORD NewStatWinState;

  switch (StatWinState) {
    case W_STATUS_WINDOW_BIG:
      /* big status window is currently showing -> go to off */
      if (ResizeWidth(machine_fullscreenwidth))
        return;
      StatWinState = W_STATUS_WINDOW_OFF;
      wStatusWindow(StatWinState);
      break;
    case W_STATUS_WINDOW_OFF:
      /* Status window not currently showing -> go to narrow */
      /* How wide will the status window be? shrink the main window
         by that much. */
      wInquireStatusWindow(W_STATUS_WINDOW_SMALL, &pextent);
      if (ResizeWidth(machine_fullscreenwidth - pextent.width))
        return;
      StatWinState = W_STATUS_WINDOW_SMALL;
      wStatusWindow(StatWinState);
      break;
    case W_STATUS_WINDOW_SMALL:
      /* Small status window currently showing -> go to off or big
         depending on whether we're running on a Siena or 3A/C */
      /* How wide will the status window be? shrink the main window
         by that much. */
      NewStatWinState = ((machine_type==MACH_SIENA ||
                          machine_type==MACH_EM_SIENA) ? W_STATUS_WINDOW_OFF :
                                                         W_STATUS_WINDOW_BIG);
      wInquireStatusWindow(NewStatWinState, &pextent);
      if (ResizeWidth(machine_fullscreenwidth - pextent.width))
        return;
      StatWinState = NewStatWinState;
      wStatusWindow(StatWinState);
      break;
  }

  /*ClearWholeWindow(); */
  /* Redraw here? */
}



/*******************************************************************************
***
*** Function         : CursorOn
*** Preconditions    : 
*** Postconditions   : The Cursor is On
***
*******************************************************************************/

LOCAL_C VOID CursorOn(VOID)
{
UWORD pflag,func;
  pflag = TRUE;
  func = P_SCR_CURSOR;
  p_iow(winHandle, P_FSET, &func, &pflag);
}

/*******************************************************************************
***
*** Function         : key_process
*** Preconditions    : A keypress is passed
*** Postconditions   : It's handled.
***
*******************************************************************************/

LOCAL_C VOID key_process(VOID)
{
INT ret, code;
  if (key.keycode & W_EVENT_KEY) {
    switch (key.keycode) {
      case CONS_EVENT_FOREGROUND:
        break;
      case CONS_EVENT_COMMAND:
        ProcessSystemCommand();
        break;
    }
  }
  else if (key.keycode & W_SPECIAL_KEY) {
    /* Psion key pressed */
    code = key.keycode & (~W_SPECIAL_KEY);
    if (key.modifiers & W_SHIFT_MODIFIER)
      code = p_toupper(code);
    TryExecuteCommand(code); /* Depends which menu is selected */
  }
  else {
    switch (key.keycode) {
      case W_KEY_MENU:
        if (key.modifiers & W_CTRL_MODIFIER) {
          CycleStatusWindow();
          config_save();
        }
        else {           /* display menu bar */
          /* For more than one menu palette, change _cmds and _mdata first... */

          ret=uPresentMenus();
          if (ret>0)
            TryExecuteCommand(ret); /* And call a different menu handler */
        }
        break;
      default:
        if (connect_stat == 0) {
          /* Connected, send keypress */
        }
        else {
          wInfoMsg("Not connected to remote system");
        }
        break;
    }
  }
}


/*******************************************************************************
***
*** Function         : stackevent_process()
*** Preconditions    : Some stack event has occurred. The event code is stored 
***                    in stackevent_stat; data is in stackevent_buffer.
*** Postconditions   : 
***
*******************************************************************************/

LOCAL_C VOID stackevent_process(VOID)
{
  switch (stackevent_stat) {
    case EVT_LINEDROPPED:
      wInfoMsg("Line dropped");
      break;
    case EVT_SCRIPTING:
      wInfoMsg("Starting script");
      break;
    case EVT_LINK_UP:
      wInfoMsg("Negotiating");
      break;
    case EVT_NET_UP:
      wInfoMsg("Online");
      break;
    case EVT_LINK_DOWN:
      wInfoMsg("Closing connection"); 
      break;
    case EVT_NET_DOWN:
      wInfoMsg("Offline");
      break;
    case EVT_NEW_METHOD:
      wInfoMsg("Selected method...");
      break;
    case EVT_UI_MESSAGE:
      wInfoMsg(stackevent_buffer);
      break;
    case EVT_TERM:
      wInfoMsg("Terminal mode");
      break;
    case EVT_UI_TERMDATA:
      p_write(winHandle, stackevent_buffer, stackevent_retlength);
      break;
    default:
      wInfoMsg("stackevent_process bug");
      console_printf("stackevent_process: unknown event 0x%04x\r\n", stackevent_stat);
      break;
  }
}


/*******************************************************************************
***
*** Function         : MainLoop
*** Preconditions    : The system has been initialised.
*** Postconditions   : This is the main loop - control never passes out.
***
*******************************************************************************/

LOCAL_C VOID MainLoop(VOID)
{
  FOREVER {
    if (keyactive)
      wFlush();
    else {
      uGetKeyA(&keystat,&key);
      keyactive=TRUE;
    }

    if (!stackevent_requestqueued) { /* probably don't want to receive all */
      if (stack_queue_event_request(0xffff, &stackevent_stat, 
                                            &stackevent_buffer, 
                                            MAX_STACKEVENTBUFFER,
                                            &stackevent_retlength) < 0) {
        p_puts("Can't queue stack event request!");
        telnet_close(0); 
      }
      stackevent_requestqueued=TRUE;
    }
      
    p_iowait();

    /* Keyboard? */
    if (keystat != E_FILE_PENDING) {
      key_process();
      keyactive = FALSE;
    }

    /* Stack? */
    if (stackevent_stat != E_FILE_PENDING) {
      stackevent_process();
      stackevent_requestqueued = FALSE;
    }

    /* Connected? */
    if (connect_stat != E_FILE_PENDING) {
      switch (connect_stat) {
        case 0: /* 3-Way handshake succeeded */
          wInfoMsg("Connected to remote system");
          break;
        default:
          p_print("socket_connecta() failed: %d\r\n", connect_stat);
          break;
      }
    }
  }
}

/*******************************************************************************
***
*** Function         : CreateGC
*** Preconditions    : 
*** Postconditions   : Attempt to create default graphics context on main 
***                    window
***
*******************************************************************************/

LOCAL_C VOID CreateGC(VOID)
{
INT hgc;        /* graphics context handle (if successful) */
  hgc=gCreateGC0(MainWid);
  if (hgc<0)
    telnet_close(hgc);
}



/*******************************************************************************
***
*** Function         : SetupConsoleFont()
*** Preconditions    : Font is set to a font, machine_fullscreencols,
***                    machine_fullscreenrows is the full of the screen.
*** Postconditions   : The console font is changed, and the console resized.
***
*******************************************************************************/

VOID SetupConsoleFont(VOID)
{
UWORD func;
P_RECT rect;
  /* Change the console font. Can't do this on MCx00's */
  if (machine_type != MACH_MC200 && machine_type != MACH_MC400) {
    /* Set the font */
    func = P_SCR_FONT;
    p_iow(winHandle, P_FSET, &func, &Font);

    /* Set the console size and position */
    func = P_SCR_WSET;
    rect.tl.x=0;
    rect.tl.y=0;
    rect.br.x=machine_fullscreencols;
    rect.br.y=machine_fullscreenrows;
    if (p_iow(winHandle, P_FSET, &func, &rect)!=0)
      wInfoMsg("Couldn't resize console");
  }
}


/*******************************************************************************
***
*** Function         : SpecificInit
*** Preconditions    : HWIF has been initialised
*** Postconditions   : The rest of the system is initialised.
***
*******************************************************************************/


LOCAL_C VOID SpecificInit(VOID)
{
LOCAL_D INT gc, i;
P_EXTENT pextent;

  p_unmarka();
  _GreyLines = &lines[0];
  p_bfil(&lines[0], sizeof(lines), 0);
  uAddGreyUline('Z',&lines[0]);
  uAddGreyUline('d',&lines[0]);
  uAddGreyUline('s',&lines[0]);

  gc = gCreateGC0(uFindMainWid());
  if (_UseFullScreen) {
    /* Running on a Series 3a or Workabout */
    wFree(gc);
    uEnableGrey();
    gc = gCreateGC0(uFindMainWid());
  }

  /* Which type of machine are we running on? */
  machine_check();
  if (machine_type == MACH_UNKNOWN) {
    p_puts("Unknown machine!");
    telnet_close(0); /* probably don't want this here? */
  }

  /* Set initial configuration and limits based on machine type */
  Font.style = G_STY_MONO;
  switch (machine_type) {
    case MACH_SIENA:
      StatWinState = W_STATUS_WINDOW_SMALL;
      Font.id = WS_FONT_BASE+8;
      break;
    case MACH_EM_SIENA:
      StatWinState = W_STATUS_WINDOW_SMALL;
      Font.id = WS_FONT_BASE+8;
      break;
    case MACH_S3a:
      StatWinState = W_STATUS_WINDOW_BIG;
      Font.id = WS_FONT_BASE+8;
      break;
    case MACH_EM_S3a:
      StatWinState = W_STATUS_WINDOW_BIG;
      Font.id = WS_FONT_BASE+8;
      break;
    case MACH_S3c:
      StatWinState = W_STATUS_WINDOW_BIG;
      Font.id = WS_FONT_BASE+8;
      break;
    case MACH_S3:
      StatWinState = W_STATUS_WINDOW_SMALL;
      Font.id = WS_FONT_BASE;
      break;
    case MACH_MC400:
      StatWinState = W_STATUS_WINDOW_BIG;
      Font.id = WS_FONT_BASE+3;
      break;
    case MACH_MC200:
      StatWinState = W_STATUS_WINDOW_BIG;
      Font.id = WS_FONT_BASE+3;
      break;
    case MACH_HC:
      StatWinState = W_STATUS_WINDOW_SMALL;
      Font.id = WS_FONT_BASE+2;
      break;
    case MACH_WORKABOUT:
      StatWinState = W_STATUS_WINDOW_SMALL;
      Font.id = WS_FONT_BASE+8;
      _SmallFontDialog = TRUE;
      _UseFullScreen = TRUE;
      break;
    case MACH_EM_WORKABOUT:
      StatWinState = W_STATUS_WINDOW_SMALL;
      Font.id = WS_FONT_BASE+8;
      _SmallFontDialog = TRUE;
      break;
  }
  SetupConsoleFont();
  uEscape(FALSE);             /* disable Psion-Esc */
  MainWid=uFindMainWid();     /* find id of main display window */
  CreateGC();                 /* create default graphics context */
  /* analyse command line */
  hCrackCommandLine();
  /*wsSetList(2,&modelist[0],CurrMode);*/
  /*DisplayStart();*/

  p_print("PsiStack Telnet v%d.%02d%c\r\n", version / 100, version % 100, 
          releaselevel);
  p_print("Built on " __DATE__ "\r\n");
  p_puts(szCopyright);
  p_puts(szWebsite);
  p_puts(szNotice);
  p_print("Machine: %s\r\n",machine_name);

  if (stack_loaded() < 0) {
    if (stack_load() < 0) {
      p_puts("Couldn't load TCP/IP stack");
      telnet_close(0);
    }
    else 
      p_puts("TCP/IP stack loaded");
  }
  else 
    p_puts("TCP/IP Stack already loaded");

  CursorOn();

  /* Initialise the remote connection info defaults */
  for (i=0; i<REMOTEHOSTLISTSIZE; i++) {
    p_scpy(&remotehost[i][0],"127.0.0.1");
    remoteport[i] = 7;
    remotetermtype[i] = 0;
  }

  /* Find out what size status window we want */
  config_load();

  wInquireStatusWindow(StatWinState, &pextent);
  if (ResizeWidth(machine_fullscreenwidth - pextent.width))
    telnet_close(0);
  wStatusWindow(StatWinState);

  /* Initialise socket */
  mysocket = socket_socket(PF_INET, SOCK_STREAM, 0);
  if (mysocket < 0) {
    p_print("socket_socket() failed: %d\r\n", mysocket);
    telnet_close(0);
  }
}


/*******************************************************************************
***
*** Function         : main
*** Preconditions    : Zilch-o
*** Postconditions   : Happy user.
***
*******************************************************************************/

GLDEF_C VOID main(VOID)
{
  _UseFullScreen=TRUE;
  uCommonInit();                /* initialisation common to all Hwif programs */
  SpecificInit();                      /* initialisation specific to PsiStack */
  MainLoop();                            /* the central event processing loop */
}

