/*******************************************************************************
***
*** Filename         : console.c
*** Purpose          : PsiStack's diagnostic stand-alone console
*** Author           : Matt J. Gumbley
*** Created          : 13/02/98
*** Last updated     : 13/02/98
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#include "..\library\utils\sysincl.h" 
#include "..\library\utils\machine.h" 
#include "..\library\utils\version.h" 
#include "..\include\psistack\sys\global.h" 
#include "..\include\psistack\sys\ip.h" 
#include "..\library\utils\utils.h" 
#include "conmsg.h"

/* The format of messages received by the console */
typedef struct conrxmsg {
  E_MESSAGE cm_e_message;
  CONMSG    cm_msg;
} CONRXMSG;

#define cm_type   cm_e_message.type                /* Of type CMT_??? below */
#define cm_pid    cm_e_message.pid
#define cm_len    cm_msg.cm_len
#define cm_data   cm_msg.cm_data

GLREF_D UWORD DatLocked;            /* non-zero when application "busy" */
GLREF_D TEXT *DatUsedPathNamePtr;   /* full path name of current file */
GLREF_D UWORD _UseFullScreen, _SmallFontDialog;
GLREF_D VOID *winHandle;

/*#define MODE_NEXT 99
#define MODE_PREV 98
LOCAL_D UINT CurrMode = 0;*/

/* Mode list names */
/*#define MODE_IFACE1 0
#define MODE_IFACE2 1
 
LOCAL_D TEXT *modelist[] = {
  "I1",
  "I2",
};*/

#define MENU_LOG         0
#define MENU_CLS         1
#define MENU_EXIT        (MENU_CLS + 1)

/*#define MENU_VIEW_IFACE1 (MENU_EXIT + 1)
#define MENU_VIEW_IFACE2 (MENU_VIEW_IFACE1 + 1)*/

GLREF_D UWORD *_MenuPositions;
GLREF_D UWORD *_GreyLines;
LOCAL_D UWORD lines[16];

/* the commands in the menu bar */
LOCAL_D TEXT *cmds[]=
/* abcdefghijklmnopqrstuvwxyz ABCDEFGHIJKLMNOPQRSTUVWXYZ 
              #           ##
*/
{
  /* Special */
  "lLogging",
  "yClear",
  "xExit",
  NULL
};

#define SPECIAL_ITEMS      3

#define ITEMS_BEFORE_SPECIAL    0
/*#define ITEMS_BEFORE_DIAMOND    (ITEMS_BEFORE_SPECIAL+SCREEN_SPECIAL)*/

/* the menus in the menu bar */
LOCAL_D H_MENU_DATA mdata[]=
{
  "Special", SPECIAL_ITEMS,
  /*"\004", DIAMOND_ITEMS, */
  NULL
};


GLDEF_D TEXT ** _cmds=(&cmds[0]);           /* Hwif code expects this */
GLDEF_D H_MENU_DATA * _mdata=(&mdata[0]);   /* Hwif code expects this */

LOCAL_D UINT MainWid;       /* ID of main window */

/* Keyboard stuff */
LOCAL_D WORD keystat;
LOCAL_D WMSG_KEY key;
LOCAL_D WORD keyactive=FALSE;

/* Console message stuff */
GLDEF_D WORD    msg_status; /* When 0 msg_this points to a message to process */
LOCAL_D CONRXMSG *msg_this;

/* Size of status window; whether Status Window is showing. See WSERV
   manual, p83 */
GLDEF_D UWORD StatWinState;

LOCAL_D P_SCR_SET_FONT Font;        /* Font used by the console */
LOCAL_D UWORD ScreenWidth;          /* width of main display area */

/* GUI Config filename */
LOCAL_D TEXT console_config_fullname[P_FNAMESIZE] =
  "LOC::M:\\APP\\PSISTACK\\CONSOLE.CFG";

/* Log file stuff */

#define LOG_DEVICE_CONSOLE 1
#define LOG_DEVICE_LOGFILE 2
#define LOG_DEVICE_BOTH 3
#define LOG_OPENMODE_APPEND 1
#define LOG_OPENMODE_OVERWRITE 2
GLREF_D UWORD log_openmode;
GLDEF_D UWORD log_device = LOG_DEVICE_CONSOLE;
GLDEF_D UWORD log_openmode = LOG_OPENMODE_OVERWRITE;
LOCAL_D VOID *log_handle = NULL;
LOCAL_D TEXT log_fullname[P_FNAMESIZE] = "LOC::M:\\PSISTACK\\CONSLOG.TXT";

/* Forward declarations */
LOCAL_C VOID SwitchMode(INT);
LOCAL_C VOID SetupConsoleFont(VOID);
LOCAL_C VOID HomeCursor(VOID);



VOID log_close(VOID)
{
  switch (log_device) {
    case LOG_DEVICE_LOGFILE:
    case LOG_DEVICE_BOTH:
      if (log_handle != NULL) {
        p_close(log_handle);
        log_handle = NULL;
      }
      break;
  }
}

VOID log_open(VOID)
{
INT errcode;

  log_close();

  switch (log_device) {
    case LOG_DEVICE_CONSOLE:
      break;
    case LOG_DEVICE_LOGFILE:
    case LOG_DEVICE_BOTH:
      switch (log_openmode) {
        case LOG_OPENMODE_APPEND:
          errcode=p_open(&log_handle,log_fullname,P_FAPPEND|P_FSTREAM|P_FUPDATE);
          if (errcode == E_FILE_NXIST) 
            errcode=p_open(&log_handle,log_fullname,P_FCREATE|P_FSTREAM|P_FUPDATE);
          break;
        default:
        case LOG_OPENMODE_OVERWRITE:
          errcode=p_open(&log_handle,log_fullname, P_FREPLACE|P_FSTREAM|P_FUPDATE);
          break;
      }
      if (errcode != 0) {
        p_printf("Couldn't open log file \"%s\" error %d", log_fullname, errcode);
        log_device = LOG_DEVICE_CONSOLE;
        log_handle = NULL;
      }
      break;
  }
}



VOID log_write(UBYTE *buf, UWORD len)
{
  if (log_device == LOG_DEVICE_BOTH || log_device == LOG_DEVICE_CONSOLE) {
    p_write(winHandle, buf, len);
    p_iow(winHandle, P_FFLUSH);
  }
  if (log_device == LOG_DEVICE_BOTH || log_device == LOG_DEVICE_LOGFILE) {
    p_write(log_handle, buf, len);
    p_iow(log_handle, P_FFLUSH);
  }
}

VOID CDECL log_printf(TEXT *fmt,UINT arg,...)
{
UINT len;
UBYTE buf[256]; 
  len = p_atob((TEXT *)&buf[0], fmt, &arg);
  log_write(buf, (UWORD)len);
}

VOID log_print(TEXT *str)
{
  log_write((UBYTE *)str, p_slen(str));
}

/*******************************************************************************
***
*** Function         : log_hexdump
*** Preconditions    : buf points to an area of memory and len is the length of
***                    this area
*** Postconditions   : An offset | hexadecimal | printable ASCII dump of the
***                    designated memory area is output
***
*******************************************************************************/

void log_hexdump(UBYTE *buf, UWORD buflen)
{ 
char LINE[80];
UWORD offset=0L;
UWORD left=buflen;
int i,upto16,x;
unsigned char b;
  while (left>0) {
    for (i=0; i<78; i++)
      LINE[i]=' ';
    LINE[9]=LINE[59]='|';
    LINE[77]='\r';
    LINE[78]='\n';
    LINE[79]='\0';
    p_atos(LINE,"    %04X",offset); 
    LINE[8]=' ';
    upto16 = (left>16) ? 16 : (int)left;
    for (x=0; x<upto16; x++) {
      b = buf[offset+x];
      LINE[11+(3*x)]=hexdig((b&0xf0)>>4);
      LINE[12+(3*x)]=hexdig(b&0x0f);
      LINE[61+x]=p_isprint((char)b) ? ((char)b) : '.';
    }
    log_write((UBYTE *)LINE,80);
    offset+=16;
    left-=16;
  }
}

/*******************************************************************************
***
*** Function         : log_ip_dump outputs an interpreted datagram trace to the 
***                    console
*** Preconditions    : i points to an IP datagram 
*** Postconditions   : ip_verlen indicates that this is an IPv4 frame and
***                    a trace is output. Otherwise, nothing.
***
*******************************************************************************/

void log_ip_dump(struct ip *i)
{
char sc,dc;
word16 fragoff;
TEXT buf[256];
TEXT buf2[30];

  if ((i->ip_verlen & 0xf0) != 0x40) {
    log_print("ip_dump: not an IPv4 frame!\r\n");
  }
  else {
    p_atos(buf,"IHL: 0x%02x longs (0x%04x bytes) TOS: 0x%02x ID: 0x%04x\r\n",
      i->ip_verlen & 0x0f, IP_HLEN(i), i->ip_tos, i->ip_id);
    log_print(buf);

    p_atos(buf,"Total length: 0x%04x Payload: 0x%04x bytes\r\n",
      i->ip_len, i->ip_len - IP_HLEN(i));
    log_print(buf);

    fragoff = i->ip_fragoff;
    p_atos(buf,"Frag offset: 0x%04x [%c] [%c] TTL: 0x%02x\r\n",
      fragoff & IP_FRAGOFF, 
      fragoff & IP_DF ? 'D' : ' ',
      fragoff & IP_MF ? 'M' : ' ', i->ip_ttl);
    log_print(buf);

    p_atos(buf,"Checksum: 0x%04x Protocol: ",i->ip_cksum);
    switch (i->ip_proto) {
      case IPT_ICMP: 
        p_scat(buf,"ICMP"); 
        break;
      case IPT_IGMP:
        p_scat(buf,"IGMP");
        break;
      case IPT_GGP:
        p_scat(buf,"GGP");
        break;
      case IPT_TCP:  
        p_scat(buf,"TCP"); 
        break;
      case IPT_EGP:
        p_scat(buf,"EGP");
        break;
      case IPT_IGP:
        p_scat(buf,"IGP");
        break;
      case IPT_UDP:  
        p_scat(buf,"UDP"); 
        break;
      case IPT_ISOTP4:
        p_scat(buf,"ISO TP4");
        break;
      case IPT_OSPFIGP:
        p_scat(buf,"OSPF");
        break;
      default:
        p_atos(buf2, "Unknown 0x%02x", i->ip_proto); 
        p_scat(buf,buf2);
        break;
    }
    log_print(buf);
    if (IP_CLASSA(i->ip_src))
      sc='A';
    if (IP_CLASSB(i->ip_src))
      sc='B';
    if (IP_CLASSC(i->ip_src))
      sc='C';
    if (IP_CLASSD(i->ip_src))
      sc='D';
    if (IP_CLASSE(i->ip_src))
      sc='E';

    if (IP_CLASSA(i->ip_dst))
      dc='A';
    if (IP_CLASSB(i->ip_dst))
      dc='B';
    if (IP_CLASSC(i->ip_dst))
      dc='C';
    if (IP_CLASSD(i->ip_dst))
      dc='D';
    if (IP_CLASSE(i->ip_dst))
      dc='E';
    p_atos(buf,"\r\nSRC [%c]: %d.%d.%d.%d DST [%c]: %d.%d.%d.%d\r\n***\r\n",
           sc, i->ip_src[0], i->ip_src[1], i->ip_src[2], i->ip_src[3], 
           dc, i->ip_dst[0], i->ip_dst[1], i->ip_dst[2], i->ip_dst[3]);
    log_print(buf);

    /* Detailed protocol dump must be done when the datagram reaches higher 
       layers, and is converted to host byte order by those layers.*/
  }
}

VOID console_config_save(VOID)
{
VOID *fd = NULL;
INT errcode;

  errcode=p_open(&fd,console_config_fullname,P_FREPLACE|P_FSTREAM|P_FUPDATE);
  if (uErrorValue(errcode)) {
    log_print("Couldn't create config file:\r\n");
    log_print((TEXT *) ((UINT)console_config_fullname+21));
    return;
  }
  p_write(fd,(VOID *)&version,sizeof(int));
  p_write(fd,(VOID *)&releaselevel,sizeof(char));
  p_write(fd,(VOID *)&StatWinState,sizeof(UWORD));
  p_write(fd,(VOID *)&log_device,sizeof(UWORD));
  p_write(fd,(VOID *)&log_openmode,sizeof(UWORD));
  p_close(fd);
}


VOID console_config_load(VOID)
{
VOID *fd = NULL;
int fileversion;
char filereleaselevel;

  if (p_open(&fd,console_config_fullname,P_FOPEN|P_FSTREAM|P_FSHARE) != 0) {
    log_printf("Creating \"%s\"\r\n", 
               (UINT)console_config_fullname+21);
    console_config_save();
    return;
  }
  p_read(fd,(VOID *)&fileversion,sizeof(int));
  p_read(fd,(VOID *)&filereleaselevel,sizeof(char));
  if (fileversion != version || filereleaselevel != releaselevel) {
    log_printf("Config file created by v%d.%02d%c\r\ncannot be used, so defaults\r\nwill be used.\r\n", fileversion / 100, fileversion % 100, filereleaselevel);
    p_close(fd);
    console_config_save();
    return;
  }

  p_read(fd,(VOID *)&StatWinState,sizeof(UWORD));
  p_read(fd,(VOID *)&log_device,sizeof(UWORD));
  p_read(fd,(VOID *)&log_openmode,sizeof(UWORD));
  p_close(fd);
}



/*******************************************************************************
***
*** Function         : msg_cancel()
*** Preconditions    : An asynchronous read request may be outstanding.
*** Postconditions   : It is cancelled.
***
*******************************************************************************/

GLDEF_C VOID msg_cancel()
{
  p_mcancel();
}
/*******************************************************************************
***
*** Function         : console_close()
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID console_close(INT num)
{
  console_config_save();
  msg_cancel();
  p_exit(num);     /* Goodnight Vienna */
}

/*******************************************************************************
***
*** Function         : msg_queue
*** Preconditions    : An asynchronous message receive request is queued
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID msg_queue()
{
  p_mreceive(&msg_status, &msg_this);
}


/*******************************************************************************
***
*** Function         : msg_init()
*** Preconditions    : 
*** Postconditions   : The message queue is initialised, and msg_init() is TRUE.
***                    Or, some error occurred, and it's FALSE.
***
*******************************************************************************/

GLDEF_C UINT msg_init()
{
  /* Initialise the message queue */
  switch (p_minit(E_MAX_PROCESSES - 4, sizeof(CONMSG))) {
    case E_GEN_NOMEMORY: 
      p_puts("No memory for messages"); 
      return FALSE;
    case E_GEN_NOSEM:
      p_puts("No semaphores for messages");
      return FALSE;
  }

  /* Queue our first asynchronous read */
  msg_queue();
  return TRUE;
}





/*******************************************************************************
***
*** Function         : msg_process()
*** Preconditions    : A message has been received, and msg_this is pointing to
***                    it.
*** Postconditions   : It is dealt with.
***
*******************************************************************************/

GLDEF_C VOID msg_process()
{
static UBYTE msgbuf[CM_MSGMAX];
UWORD msglen;
int unload = FALSE;

  /* Read the message into the msgbuf from the client data segment */
  msglen = msg_this->cm_len > CM_MSGMAX ? 
           CM_MSGMAX : msg_this->cm_len;
  p_pcpyfr(msg_this->cm_pid, msg_this->cm_data, msgbuf, msglen);

  switch (msg_this->cm_type) {

    case CMT_TEXT :
      log_write(msgbuf, msglen);
      break;
      
    case CMT_BINARY :
      log_hexdump(msgbuf, msglen);
      break;

    case CMT_IP :
      log_ip_dump((struct ip *)msgbuf);
      break;

    case CMT_UNLOAD :
      log_print("Unloading the console\r\n");
      unload = TRUE;
      break;

    default:
      /* The stack kernel can't handle database message (for instance) */
      log_printf("msg_process: bad msg type %d\r\n", msg_this->cm_type);
      break;
  }

  /* Finished with this message */
  p_mfree(msg_this, 0);
  if (unload)
    console_close(0);
}

/*******************************************************************************
***
*** Function         : HomeCursor()
*** Preconditions    : 
*** Postconditions   : As it says...
***
*******************************************************************************/

VOID HomeCursor(VOID)
{
static P_POINT home = {0,0};
static UWORD func = P_SCR_POSA;
  p_iow(winHandle, P_FSET, &func, &home);
}


/*******************************************************************************
***
*** Function         : ClearWholeWindow
*** Preconditions    : 
*** Postconditions   : Clear the entire screen and home cursor
***
*******************************************************************************/

LOCAL_C VOID ClearWholeWindow(VOID)
{
P_RECT whole;

  whole.tl.x=whole.tl.y=0;
  whole.br.x=machine_fullscreenwidth;
  whole.br.y=machine_fullscreenheight;
  gClrRect(&whole,G_TRMODE_CLR);
  HomeCursor();
}


/*******************************************************************************
***
*** Function         : SetStyle
*** Preconditions    : 
*** Postconditions   : Set the style of the current graphics context
***
*******************************************************************************/

LOCAL_C VOID SetStyle(INT style)
{
G_GC gc;

  gc.style=style;
  gSetGC(0,G_GC_MASK_STYLE,&gc);
}



/*******************************************************************************
***
*** Function         : Confirm
*** Preconditions    : 
*** Postconditions   : Present a "No/Yes" dialog
***
*******************************************************************************/

GLDEF_C INT Confirm(TEXT *pb)
{
  if (uOpenDialog(pb))    /* title text supplied by caller */
    return(-1);
  if (uAddButtonList("No",-'n',"Yes",'y',NULL))
    return(-1);
  return(uRunDialog()!='y');   /* return TRUE iff user answered Yes */
}

/*******************************************************************************
***
*** Function         : ExitApplication
*** Preconditions    : User wants to exit
*** Postconditions   : 
***
*******************************************************************************/

LOCAL_C VOID ExitApplication(VOID)
{
TEXT buf[20];
  p_scpy(buf, "Exit Console?");
  if (!Confirm(buf))
    console_close(0);
}


/*******************************************************************************
***
*** Function         : LogDialog
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID LogDialog(VOID)
{
UWORD device, ao;

  /* Copy data in */
  device = log_device;
  ao = log_openmode;

  if (uOpenDialog("Logging facilities") ||
      uAddChoiceList("Output device", &device, 
                     "Console", "Logfile", "Both", NULL) ||
      uAddChoiceList("Log file", &ao, 
                     "Append", "Overwrite", NULL))
    return;
  if (uRunDialog()<=0)
    return;

  log_device = device;
  log_openmode = ao;
  log_open();
  console_config_save();
}

/*******************************************************************************
***
*** Function         : ManageCommand
*** Preconditions    : The user selected a menu item
*** Postconditions   : Switch on command index to appropriate function
***
*******************************************************************************/

LOCAL_C VOID ManageCommand(INT index)
{
  switch (index) {
    case MENU_LOG:
      LogDialog();
      break;
    case MENU_CLS:
      ClearWholeWindow();
      break;
    case MENU_EXIT:
      ExitApplication();
    break;
  }
}



/*******************************************************************************
***
*** Function         : ProcessSystemCommand
*** Preconditions    : 
*** Postconditions   : Respond to message from System Screen
***
*******************************************************************************/

LOCAL_C VOID ProcessSystemCommand(VOID)
{ 
TEXT buf[P_FNAMESIZE+2];

  wGetCommand((UBYTE *)&buf[0]);
  if (buf[0]=='X')
    ExitApplication();
}


/*******************************************************************************
***
*** Function         : TryExecuteCommand
*** Preconditions    : 
*** Postconditions   : Try to find command to execute matching accelerator 
***                    passed
***
*******************************************************************************/

LOCAL_C VOID TryExecuteCommand(INT keycode)
{
  keycode=uLocateCommand(keycode);        /* look up accelerator table */
  if (keycode>=0)                 /* found - keycode converted to index */
    ManageCommand(keycode);
}


/*******************************************************************************
***
*** Function         : ResizeWidth
*** Preconditions    : 
*** Postconditions   : Try to change the width of the main display region 
***
*******************************************************************************/

LOCAL_C INT ResizeWidth(UWORD width)
{
INT ret;
W_WINDATA wd;

  wd.extent.tl.x=wd.extent.tl.y=0;
  wd.extent.width=width;
  wd.extent.height=machine_fullscreenheight;
  wSetWindow(MainWid,W_WIN_EXTENT,&wd);
  ret=uErrorValue(wCheckPoint());     /* check whether successful */
  if (!ret)
    ScreenWidth=width;
  return(ret);
}


/*******************************************************************************
***
*** Function         : CycleStatusWindow
*** Preconditions    : 
*** Postconditions   : Attempt to flip through the different states of the 
***                    status window 
***
*******************************************************************************/

LOCAL_C VOID CycleStatusWindow(VOID)
{
P_EXTENT pextent;
UWORD NewStatWinState;

  switch (StatWinState) {
    case W_STATUS_WINDOW_BIG:
      /* big status window is currently showing -> go to off */
      if (ResizeWidth(machine_fullscreenwidth))
        return;
      StatWinState = W_STATUS_WINDOW_OFF;
      wStatusWindow(StatWinState);
      break;
    case W_STATUS_WINDOW_OFF:
      /* Status window not currently showing -> go to narrow */
      /* How wide will the status window be? shrink the main window
         by that much. */
      wInquireStatusWindow(W_STATUS_WINDOW_SMALL, &pextent);
      if (ResizeWidth(machine_fullscreenwidth - pextent.width))
        return;
      StatWinState = W_STATUS_WINDOW_SMALL;
      wStatusWindow(StatWinState);
      break;
    case W_STATUS_WINDOW_SMALL:
      /* Small status window currently showing -> go to off or big
         depending on whether we're running on a Siena or 3A/C */
      /* How wide will the status window be? shrink the main window
         by that much. */
      NewStatWinState = ((machine_type==MACH_SIENA ||
                          machine_type==MACH_EM_SIENA) ? W_STATUS_WINDOW_OFF :
                                                         W_STATUS_WINDOW_BIG);
      wInquireStatusWindow(NewStatWinState, &pextent);
      if (ResizeWidth(machine_fullscreenwidth - pextent.width))
        return;
      StatWinState = NewStatWinState;
      wStatusWindow(StatWinState);
      break;
  }

  /*ClearWholeWindow(); */
  /* Redraw here? */
}


/*******************************************************************************
***
*** Function         : SwitchMode
*** Preconditions    : newmode is 0..1 or a MODE_NEXT/PREV code
*** Postconditions   : The view is changed to that view.
***
*******************************************************************************/

/*LOCAL_C VOID SwitchMode(INT newmode) 
{
  switch (newmode) {
    case MODE_NEXT:
    case MODE_PREV:
      CurrMode ^= 0x01; */ /* Toggle bottom bit */ /*
      break;
    default:
      CurrMode = newmode;
      break;
  }
  wsSelectList(CurrMode);
  ClearWholeWindow(); */
  /* Redisplay virtual console contents for this interface */ 
/* }*/


/*******************************************************************************
***
*** Function         : key_process
*** Preconditions    : A keypress is passed
*** Postconditions   : It's handled.
***
*******************************************************************************/

LOCAL_C VOID key_process(VOID)
{
INT ret, code;
  if (key.keycode & W_EVENT_KEY) {
    switch (key.keycode) {
      case CONS_EVENT_FOREGROUND:
        break;
      case CONS_EVENT_COMMAND:
        ProcessSystemCommand();
        break;
    }
  }
  else if (key.keycode & W_SPECIAL_KEY) {
    /* Psion key pressed */
    code = key.keycode & (~W_SPECIAL_KEY);
    if (key.modifiers & W_SHIFT_MODIFIER)
      code = p_toupper(code);
    TryExecuteCommand(code);
  }
  else {
    switch (key.keycode) {
/*      case W_KEY_DIAMOND:
        if (key.modifiers & W_SHIFT_MODIFIER)
          SwitchMode(MODE_PREV);
        else
          SwitchMode(MODE_NEXT);
        break;*/
      case W_KEY_MENU:
        if (key.modifiers & W_CTRL_MODIFIER) {
          CycleStatusWindow();
          console_config_save();
        }
        else {           /* display menu bar */
          ret=uPresentMenus();
          if (ret>0)
            TryExecuteCommand(ret);
        }
        break;
      default:
        break;
    }
  }
}



/*******************************************************************************
***
*** Function         : MainLoop
*** Preconditions    : The system has been initialised.
*** Postconditions   : This is the main loop - control never passes out.
***
*******************************************************************************/

LOCAL_C VOID MainLoop(VOID)
{
  FOREVER {
    if (keyactive)
      wFlush();
    else {
      uGetKeyA(&keystat,&key);
      keyactive=TRUE;
    }

    p_iowait();

    /* Messages from clients? */
    if (msg_status != E_FILE_PENDING) {
      msg_process();
      msg_queue();
    }

    /* This should be called userinterface_process, I suppose... */
    /* Keyboard? */
    if (keystat != E_FILE_PENDING) {
      key_process();
      keyactive = FALSE;
    }
  }
}

/*******************************************************************************
***
*** Function         : CreateGC
*** Preconditions    : 
*** Postconditions   : Attempt to create default graphics context on main 
***                    window
***
*******************************************************************************/

LOCAL_C VOID CreateGC(VOID)
{
INT hgc;        /* graphics context handle (if successful) */
  hgc=gCreateGC0(MainWid);
  if (hgc<0)
    console_close(hgc);
}



/*******************************************************************************
***
*** Function         : SetupConsoleFont()
*** Preconditions    : Font is set to a font, machine_fullscreencols,
***                    machine_fullscreenrows is the full of the screen.
*** Postconditions   : The console font is changed, and the console resized.
***
*******************************************************************************/

VOID SetupConsoleFont(VOID)
{
UWORD func;
P_RECT rect;
  /* Change the console font. Can't do this on MCx00's */
  if (machine_type != MACH_MC200 && machine_type != MACH_MC400) {
    /* Set the font */
    func = P_SCR_FONT;
    p_iow(winHandle, P_FSET, &func, &Font);

    /* Set the console size and position */
    func = P_SCR_WSET;
    rect.tl.x=0;
    rect.tl.y=0;
    rect.br.x=machine_fullscreencols;
    rect.br.y=machine_fullscreenrows;
    if (p_iow(winHandle, P_FSET, &func, &rect)!=0)
      wInfoMsg("Couldn't resize console");
  }
}


/*******************************************************************************
***
*** Function         : SpecificInit
*** Preconditions    : HWIF has been initialised
*** Postconditions   : The rest of the system is initialised.
***
*******************************************************************************/


LOCAL_C VOID SpecificInit(VOID)
{
UWORD pflag,func;
LOCAL_D INT gc;
P_EXTENT pextent;

  p_unmarka();
  _GreyLines = &lines[0];
  p_bfil(&lines[0], sizeof(lines), 0);
  uAddGreyUline('Z',&lines[0]);

  gc = gCreateGC0(uFindMainWid());
  if (_UseFullScreen) {
    /* Running on a Series 3a or Workabout */
    wFree(gc);
    uEnableGrey();
    gc = gCreateGC0(uFindMainWid());
  }

  /* Which type of machine are we running on? */
  machine_check();
  if (machine_type == MACH_UNKNOWN) {
    p_puts("Unknown machine!");
    console_close(0);
  }

  /* Set initial configuration and limits based on machine type */
  Font.style = G_STY_MONO;
  switch (machine_type) {
    case MACH_SIENA:
      StatWinState = W_STATUS_WINDOW_SMALL;
      Font.id = WS_FONT_BASE+8;
      break;
    case MACH_EM_SIENA:
      StatWinState = W_STATUS_WINDOW_SMALL;
      Font.id = WS_FONT_BASE+8;
      break;
    case MACH_S3a:
      StatWinState = W_STATUS_WINDOW_BIG;
      Font.id = WS_FONT_BASE+8;
      break;
    case MACH_EM_S3a:
      StatWinState = W_STATUS_WINDOW_BIG;
      Font.id = WS_FONT_BASE+8;
      break;
    case MACH_S3c:
      StatWinState = W_STATUS_WINDOW_BIG;
      Font.id = WS_FONT_BASE+8;
      break;
    case MACH_S3:
      StatWinState = W_STATUS_WINDOW_SMALL;
      Font.id = WS_FONT_BASE;
      break;
    case MACH_MC400:
      StatWinState = W_STATUS_WINDOW_BIG;
      Font.id = WS_FONT_BASE+3;
      break;
    case MACH_MC200:
      StatWinState = W_STATUS_WINDOW_BIG;
      Font.id = WS_FONT_BASE+3;
      break;
    case MACH_HC:
      StatWinState = W_STATUS_WINDOW_SMALL;
      Font.id = WS_FONT_BASE+2;
      break;
    case MACH_WORKABOUT:
      StatWinState = W_STATUS_WINDOW_SMALL;
      Font.id = WS_FONT_BASE+8;
      _SmallFontDialog = TRUE;
      _UseFullScreen = TRUE;
      break;
    case MACH_EM_WORKABOUT:
      StatWinState = W_STATUS_WINDOW_SMALL;
      Font.id = WS_FONT_BASE+8;
      _SmallFontDialog = TRUE;
      break;
  }
  SetupConsoleFont();
  uEscape(FALSE);             /* disable Psion-Esc */
  MainWid=uFindMainWid();     /* find id of main display window */
  CreateGC();                 /* create default graphics context */
  /* analyse command line */
  hCrackCommandLine();
  /*wsSetList(2,&modelist[0],CurrMode);*/
  /*DisplayStart();*/

  p_print("PsiStack Console v%d.%02d%c %s\r\n", version / 100, version % 100, 
          releaselevel, szNickname);
  p_print("Built on " __DATE__ "\r\n");
  p_puts(szCopyright);
  p_puts(szWebsite);
  p_puts(szNotice);
  p_print("Machine: %s\r\n",machine_name);

  /* Turn on the cursor */
  pflag = TRUE;
  func = P_SCR_CURSOR;
  p_iow(winHandle, P_FSET, &func, &pflag);

  /* Find out what size status window we want */
  console_config_load();
  log_open();

  wInquireStatusWindow(StatWinState, &pextent);
  if (ResizeWidth(machine_fullscreenwidth - pextent.width))
    console_close(0);
  wStatusWindow(StatWinState);

  if (!msg_init()) {
    p_puts("Couldn't init messaging");
    console_close(0);
  }
}


/*******************************************************************************
***
*** Function         : main
*** Preconditions    : Zilch-o
*** Postconditions   : Happy user.
***
*******************************************************************************/

GLDEF_C VOID main(VOID)
{
  _UseFullScreen=TRUE;
  uCommonInit();                /* initialisation common to all Hwif programs */
  SpecificInit();                      /* initialisation specific to PsiStack */
  MainLoop();                            /* the central event processing loop */
}

