/*******************************************************************************
***
*** Filename         : dbms.c
*** Purpose          : PsiStack's database management system
*** Author           : Matt J. Gumbley
*** Created          : 17/02/98
*** Last updated     : 27/02/98
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#include "..\library\utils\sysincl.h" 
#include "..\library\utils\machine.h" 
#include "..\library\utils\version.h" 
#include "..\include\psistack\dbms.h" 
#include "..\include\psistack\console.h" 
#include "dbmsg.h"
#include "dbmsgi.h"

/* Prototypes for tables */
#include "dbisp.h"
#include "dbphn.h"
#include "dbsrv.h"
#include "dbprt.h"
#include "dbisv.h"
#include "dbmph.h"
#include "dbhct.h"
#include "dbmet.h"
#include "dbdev.h"
#include "dbsys.h"
#include "dbhst.h"

/* DBMS message stuff */
GLDEF_D WORD    msg_status; /* When 0 msg_this points to a message to process */
GLDEF_D DBRXMSG *msg_this;


/* Set this to FALSE if you want to close */
int active = TRUE;

GLREF_C UWORD dbms_stat(VOID);
GLREF_C UWORD dbms_unload(VOID);

/* Bletcherous function pointer dispatch table. Read this as:
   "dispatch is a DTN_MAXTABLES x DOP_MAXOPERATIONS array of pointers to
    functions, each of which have a void parameter list, and return a UWORD" 

   Each row is a table; each column an operation. */

UWORD (*dispatch [DTN_MAXTABLES][DOP_MAXOPERATIONS] )(VOID) = {
  dbms_stat, dbms_unload, dbisp_seek, dbisp_read, dbisp_write, dbisp_delete,
                          dbisp_kread, dbisp_kwrite, dbisp_kdelete,
                          dbisp_lock, dbisp_unlock,

  dbms_stat, dbms_unload, dbphn_seek, dbphn_read, dbphn_write, dbphn_delete,
                          dbphn_kread, dbphn_kwrite, dbphn_kdelete,
                          dbphn_lock, dbphn_unlock,
  
  dbms_stat, dbms_unload, dbsrv_seek, dbsrv_read, dbsrv_write, dbsrv_delete,
                          dbsrv_kread, dbsrv_kwrite, dbsrv_kdelete,
                          dbsrv_lock, dbsrv_unlock,

  dbms_stat, dbms_unload, dbprt_seek, dbprt_read, dbprt_write, dbprt_delete,
                          dbprt_kread, dbprt_kwrite, dbprt_kdelete,
                          dbprt_lock, dbprt_unlock,

  dbms_stat, dbms_unload, dbisv_seek, dbisv_read, dbisv_write, dbisv_delete,
                          dbisv_kread, dbisv_kwrite, dbisv_kdelete,
                          dbisv_lock, dbisv_unlock,

  dbms_stat, dbms_unload, dbmph_seek, dbmph_read, dbmph_write, dbmph_delete,
                          dbmph_kread, dbmph_kwrite, dbmph_kdelete,
                          dbmph_lock, dbmph_unlock,

  dbms_stat, dbms_unload, dbhct_seek, dbhct_read, dbhct_write, dbhct_delete,
                          dbhct_kread, dbhct_kwrite, dbhct_kdelete,
                          dbhct_lock, dbhct_unlock,

  dbms_stat, dbms_unload, dbmet_seek, dbmet_read, dbmet_write, dbmet_delete,
                          dbmet_kread, dbmet_kwrite, dbmet_kdelete,
                          dbmet_lock, dbmet_unlock,

  dbms_stat, dbms_unload, dbdev_seek, dbdev_read, dbdev_write, dbdev_delete,
                          dbdev_kread, dbdev_kwrite, dbdev_kdelete,
                          dbdev_lock, dbdev_unlock,

  dbms_stat, dbms_unload, dbsys_seek, dbsys_read, dbsys_write, dbsys_delete,
                          dbsys_kread, dbsys_kwrite, dbsys_kdelete,
                          dbsys_lock, dbsys_unlock,

  dbms_stat, dbms_unload, dbhst_seek, dbhst_read, dbhst_write, dbhst_delete,
                          dbhst_kread, dbhst_kwrite, dbhst_kdelete,
                          dbhst_lock, dbhst_unlock,
};


#define OC_OPEN 0
#define OC_CLOSE 1
UWORD (*openclose [DTN_MAXTABLES][2] )(VOID) = {
  dbisp_open, dbisp_close,
  dbphn_open, dbphn_close,
  dbsrv_open, dbsrv_close,
  dbprt_open, dbprt_close,
  dbisv_open, dbisv_close,
  dbmph_open, dbmph_close,
  dbhct_open, dbhct_close,
  dbmet_open, dbmet_close,
  dbdev_open, dbdev_close,
  dbsys_open, dbsys_close,
  dbhst_open, dbhst_close,
};


TEXT *dbnames[DTN_MAXTABLES] = {
  "ISP",
  "PHN",
  "SRV",
  "PRT",
  "ISV",
  "MPH",
  "HCT",
  "MET",
  "DEV",
  "SYS",
  "HST",
};

TEXT *opnames[DOP_MAXOPERATIONS] = {
  "STAT   ",
  "UNLOAD ",
  "SEEK   ",
  "READ   ",
  "WRITE  ",
  "DELETE ",
  "KREAD  ",
  "KWRITE ",
  "KDELETE",
  "LOCK   ",
  "UNLOCK ",
};

/*******************************************************************************
***
*** Function         : msg_cancel()
*** Preconditions    : An asynchronous read request may be outstanding.
*** Postconditions   : It is cancelled.
***
*******************************************************************************/

GLDEF_C VOID msg_cancel()
{
  p_mcancel();
}

GLDEF_C UWORD dbms_stat(VOID)
{
  console_puts("dbms_stat");
  return 0;
}

GLDEF_C UWORD dbms_unload(VOID)
{
  active=FALSE;
  console_puts("dbms_unload");
  return 1;
}

GLDEF_C UWORD dbms_close(VOID)
{
int i;
int allok=TRUE;
UWORD retval;
  console_puts("dbms_flush");
  for (i=0; i<DTN_MAXTABLES; i++) {
    console_printf("closing %s...\r\n", (UINT)dbnames[i]);
    retval = (*(openclose[i][OC_CLOSE]))();
    if (!retval) {
      console_puts("  failed");
      allok=FALSE;
    }
  }
  return allok;
}

GLDEF_C UWORD dbms_open(VOID)
{
int i;
int allok=TRUE;
UWORD retval;
  console_puts("dbms_open");
  for (i=0; i<DTN_MAXTABLES; i++) {
    console_printf("opening %s...\r\n", (UINT)dbnames[i]);
    retval = (*(openclose[i][OC_OPEN]))();
    if (!retval) {
      console_puts("  failed");
      allok=FALSE;
    }
  }
  return allok;
}

GLDEF_C VOID dbms_exit(INT num)
{
  console_puts("dbms_exit");
  dbms_close();
  msg_cancel();
  p_exit(num);     /* Goodnight Vienna */
}


/*******************************************************************************
***
*** Function         : msg_queue
*** Preconditions    : An asynchronous message receive request is queued
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID msg_queue()
{
  p_mreceive(&msg_status, &msg_this);
}


/*******************************************************************************
***
*** Function         : msg_init()
*** Preconditions    : 
*** Postconditions   : The message queue is initialised, and msg_init() is TRUE.
***                    Or, some error occurred, and it's FALSE.
***
*******************************************************************************/

GLDEF_C UINT msg_init()
{
  /* Initialise the message queue */
  switch (p_minit(E_MAX_PROCESSES - 4, sizeof(DBMSG))) {
    case E_GEN_NOMEMORY: 
      console_puts("No memory for messages"); 
      return FALSE;
    case E_GEN_NOSEM:
      console_puts("No semaphores for messages");
      return FALSE;
  }

  /* Queue our first asynchronous read */
  msg_queue();
  return TRUE;
}


/*******************************************************************************
***
*** Function         : msg_process()
*** Preconditions    : A message has been received, and msg_this is pointing to
***                    it.
*** Postconditions   : It is dealt with.
***
*******************************************************************************/

GLDEF_C VOID msg_process()
{
static UBYTE msgbuf[DM_MSGMAX];
UWORD msglen;

  if (msg_this->dm_record != NULL) {
    /* Read the message into the msgbuf from the client data segment */
    msglen = msg_this->dm_len > DM_MSGMAX ? 
             DM_MSGMAX : msg_this->dm_len;
    p_pcpyfr(msg_this->dm_pid, msg_this->dm_record, msgbuf, msglen);
  }

  /* msg_this->dm_type is a DTN_??? table identifier */
  if (msg_this->dm_type < DTN_MAXTABLES &&
      msg_this->dm_op < DOP_MAXOPERATIONS) {
    console_printf("%s: %s REC %05u\r\n", 
                   (UINT) dbnames[msg_this->dm_type], opnames[msg_this->dm_op],
                   msg_this->dm_recno);
    /* Dispatch the message, and reply with the return of this */
    p_mfree(msg_this, (*(dispatch[msg_this->dm_type][msg_this->dm_op]))());
    /* Dig that bletcherous C.... Page 118 of K&R if you need it :-) */
  }
  else {
    /* Sorry, but I don't know what you're talking about!! */
    console_printf("bad table %d or bad op %d\r\n", 
                   msg_this->dm_type, msg_this->dm_op);
    p_mfree(msg_this, -1);
  }
}



/*******************************************************************************
***
*** Function         : main
*** Preconditions    : Eh-oh!
*** Postconditions   : Teletubbies go to sleep.
***
*******************************************************************************/

GLDEF_C VOID main(VOID)
{
  /* We're running at priority 0x82 if started by the stack kernel, but
     let's just make sure, in case the user starts us... */
  p_setpri(p_getpid(), 0x82);

  p_unmarka();

  console_setprefix("DBMS"); /* Doesn't context switch */

  /* See comment in the kernel at this point, as to why we initialise 
     messaging straight away */
  if (!msg_init()) {
    console_puts("Couldn't init messaging");
    dbms_exit(0);
  }

  /* Which type of machine are we running on? */
  machine_check();
  if (machine_type == MACH_UNKNOWN) {
    console_puts("Unknown machine!");
    dbms_exit(0);
  }

  console_printf("PsiStack DBMS v%d.%02d%c %s\r\n", version/100, version%100, 
          releaselevel, szNickname);
  console_puts("Built on " __DATE__);


  if (!dbms_open()) {
    console_puts("Couldn't open all tables");
    dbms_exit(0);
  }

  console_puts("All OK... off we go..");

  /* Drop to the same priority as the stack kernel */
  p_setpri(p_getpid(), 0x81);

  /* Main Loop */
  
  while (active) {
    p_iowait();

    /* Messages from clients? */
    if (msg_status != E_FILE_PENDING) {
      msg_process();
      msg_queue();
    }
  }
  dbms_exit(0);
}

