/*******************************************************************************
***
*** Filename         : dialogs.c
*** Purpose          : Dialogs for the front-end to PsiStack.
*** Author           : Matt J. Gumbley, Patrick H. Robbe
*** Created          : 05/04/97
*** Last updated     : 02/12/97
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#include "..\kernel\stack.h" /* The stack kernel and system includes */
#include "dialogs.h"

LOCAL_D TEXT *szYes = "Yes";
LOCAL_D TEXT *szNo = "No";
LOCAL_D TEXT *szIgnore = "Ignore";
LOCAL_D TEXT *szIF = "I/F";
LOCAL_D TEXT *szNone = "None";
LOCAL_D TEXT *szGateways = "Gateways";
LOCAL_D TEXT *szDNSs = "DNSs";
LOCAL_D TEXT *szConfirm = "Confirm";
LOCAL_D TEXT *szInterfaceNotDown = "I/F must be down";
LOCAL_D TEXT *szNeedReload = "Reload to use new settings";
LOCAL_D TEXT ifaceTitle[64];

GLREF_D TEXT *DatUsedPathNamePtr;

/* Forward declarations */
LOCAL_C VOID CancelTimer(VOID);
LOCAL_C VOID SwitchMode(INT);



/*******************************************************************************
***
*** Function         : InterfaceDialog
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

LOCAL_C INT InterfaceDialog(INT iface, TEXT *rest)
{
  p_atos(&ifaceTitle[0],"%s %d %s",szIF,iface,rest);
  return uOpenDialog(ifaceTitle);
}


/*******************************************************************************
***
*** Function         : PortDialog
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID PortDialog(INT iface)
{
register struct netif *n = &nif[iface];
static char szTTYA[]="TTY:A";
static char szTTYB[]="TTY:B";
static char szTTYI[]="TTY:I";
UWORD device, origdevice;
UWORD baud;
UWORD databits;
UWORD stopbits;
UWORD parity;
UWORD handshaking;
UWORD devicedetect;
UWORD carrierdetect;
int valid;

  if (nif[iface].ni_state != NIS_DOWN) {
    wInfoMsg(szInterfaceNotDown);
    return;
  }

  /* Set up the defaults for the dialog, as taken from the network interface
     structure. */

  if (p_scmp(n->ni_serDeviceName,szTTYB)==0)
    device=2;
  else if (p_scmp(n->ni_serDeviceName,szTTYI)==0)
    device=3;
  else 
    device=1;

  origdevice = device;

  switch (n->ni_speed) {
    case P_BAUD_300:
      baud = 1; break;
    case P_BAUD_600:
      baud = 2; break;
    case P_BAUD_1200:
      baud = 3; break;
    case P_BAUD_2400:
      baud = 4; break;
    case P_BAUD_4800:
      baud = 5; break;
    default:
    case P_BAUD_9600:
      baud = 6; break;
    case P_BAUD_19200:
      baud = 7; break;
    case P_BAUD_38400:
      baud = 8; break;
    case P_BAUD_56000:
      baud = 9; break;
    case P_BAUD_115000:
      baud = 10; break;
  }

  switch (n->ni_databits) {
    case P_DATA_7:
      databits = 1; break;
    default:
    case P_DATA_8:
      databits = 2; break;
  }

  switch (n->ni_stopbits) {
    default:
    case 0:
      stopbits = 1; break;
    case P_TWOSTOP:
      stopbits = 2; break;
  }

  switch (n->ni_parity) {
    case IGNORE:
      parity = 1; break;
    default:
    case NOPAR:
      parity = 2; break;
    case EVENPAR:
      parity = 3; break;
    case ODDPAR:
      parity = 4; break;
  }

  switch (n->ni_handshake) {
    case IGNORE:
      handshaking = 1; break;
    default:
    case RTSCTS: 
      handshaking = 2; break;
    case XONXOFF: 
      handshaking = 3; break;
  }

  /* 1 is yes, detect it, 2 is no, ignore it */
  devicedetect = n->ni_devicedetect;
  carrierdetect = n->ni_carrierdetect;

  /* Create the dialog. Loop forever, until the user puts valid settings in. */
  do {
    valid = TRUE;

    if (InterfaceDialog(iface,"port settings") ||
        uAddChoiceList("Device", &device, 
          "RS232 (TTY:A)",
          "RS232 (TTY:B)",
          "IrDA (TTY:I)" ,NULL) ||
        uAddChoiceList("Baud rate", &baud,
          szBaudRates[BR_300],
          szBaudRates[BR_600],
          szBaudRates[BR_1200],
          szBaudRates[BR_2400],
          szBaudRates[BR_4800],
          szBaudRates[BR_9600],
          szBaudRates[BR_19200],
          szBaudRates[BR_38400],
          szBaudRates[BR_57600],
          szBaudRates[BR_115200], NULL) ||
        uAddChoiceList("Data bits", &databits, "7", "8", NULL) ||
        uAddChoiceList("Stop bits", &stopbits, "1", "2", NULL) ||
        uAddChoiceList("Parity", &parity, 
          szIgnore, "None", "Even", "Odd", NULL) ||
        uAddChoiceList("Handshaking", &handshaking, 
          szIgnore, "RTS/CTS", "XON/XOFF", NULL) ||
        uAddChoiceList("Ignore DSR/DTR", &devicedetect, 
          szNo, szYes, NULL) ||
        uAddChoiceList("Ignore DCD", &carrierdetect, 
          szNo, szYes, NULL) ||
        uRunDialog()<=0)
      return;

    /* Are these settings valid? */
    if (device != 3 && baud == 10) { /* Serial port at 115200?! */
      wInfoMsg("RS232 > 115200 baud");
      valid = FALSE;
    }
    /* Need check for a 3a, since this can't have IrDA, or serial ports
       faster than 19200 (CHECK THIS) */
    if (machine_type == MACH_SIENA && baud > 7 && device ==3) {
      wInfoMsg("IrDA > 19200 baud");
      valid = FALSE;
    }
  } while (!valid);

  /* Get the user's settings, and put them back into the network interface.
     Adjust the serial device's settings. */

  switch (device) {
    case 1: p_scpy(n->ni_serDeviceName, szTTYA); break;
    case 2: p_scpy(n->ni_serDeviceName, szTTYB); break;
    case 3: p_scpy(n->ni_serDeviceName, szTTYI); break;
  }

  switch (baud) {
    case 1: n->ni_speed = P_BAUD_300; break;
    case 2: n->ni_speed = P_BAUD_600; break;
    case 3: n->ni_speed = P_BAUD_1200; break;
    case 4: n->ni_speed = P_BAUD_2400; break;
    case 5: n->ni_speed = P_BAUD_4800; break;
    case 6: n->ni_speed = P_BAUD_9600; break;
    case 7: n->ni_speed = P_BAUD_19200; break;
    case 8: n->ni_speed = P_BAUD_38400; break;
    case 9: n->ni_speed = P_BAUD_56000; break;
    case 10:n->ni_speed = P_BAUD_115000; break;
  }

  switch (databits) {
    case 1 : n->ni_databits = P_DATA_7; break;
    case 2 : n->ni_databits = P_DATA_8; break;
  }

  switch (stopbits) {
    case 1: n->ni_stopbits = 0; break;
    case 2: n->ni_stopbits = P_TWOSTOP; break;
  }

  switch (parity) {
    case 1: n->ni_parity = IGNORE; break;
    case 2: n->ni_parity = NOPAR; break;
    case 3: n->ni_parity = EVENPAR; break;
    case 4: n->ni_parity = ODDPAR; break;
  }

  switch (handshaking) {
    case 1: n->ni_handshake = IGNORE; break;
    case 2: n->ni_handshake = RTSCTS; break;
    case 3: n->ni_handshake = XONXOFF; break;
  }

  n->ni_devicedetect = devicedetect;
  n->ni_carrierdetect = carrierdetect;

  /* Adjust the serial port immediately, but only if we're not DOWN */
  if (n->ni_state != NIS_DOWN) {
    /* If they've changed the serial port, then close the old port, before
       opening the new one. */
    if (origdevice != device) {
      wInfoMsg("Changing device");
      asy_close(n);
    }
    asy_open(n);
  }

  config_save();
}


/*******************************************************************************
***
*** Function         : StateDialog
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID StateDialog(INT iface)
{
register struct netif *n = &nif[iface];
static char szCancel[]="Cancel";
static char szUp[]="Up";
static char szScript[]="Script";
static char szTerminal[]="Term";
static char szDown[]="Down";
static char szChange[]="Changed to ";
TEXT msgbuf[40];
TEXT titlebuf[40];

  p_scpy(titlebuf, "(");
  switch (n->ni_state) {
    case NIS_UP:     p_scat(titlebuf, szUp); break;
    case NIS_SCRIPT: p_scat(titlebuf, szScript); break;
    case NIS_TERM:   p_scat(titlebuf, szTerminal); break;
    case NIS_DOWN:   p_scat(titlebuf, szDown); break;
    default:         p_scat(titlebuf,"*BUG*"); break;
  }
  p_scat(titlebuf,")");
  if (InterfaceDialog(iface,titlebuf))
    return;

  /* Certain state changes are only allowed in certain states. For instance,
     you can't change from up to terminal at all, and script and terminal are
     only reachable from down. See the diagram in the technical guide, in the
     section on interface states */

  switch (n->ni_state) {

    case NIS_UP:
      if (uAddButtonList(szCancel,W_KEY_ESCAPE,
                         szDown,'d', NULL))
        return;
      break;

    case NIS_SCRIPT:
      if (uAddButtonList(szCancel,W_KEY_ESCAPE,
                         szDown,'d', NULL))
        return;
      break;
      
    case NIS_TERM:
      if (uAddButtonList(szCancel,W_KEY_ESCAPE,
                         szDown,'d',
                         szUp,'u', NULL))
        return;
      break;

    case NIS_DOWN:
      /* Two cases: null script, or proper script */
      if (!n->ni_sc->si_scriptused) {
        if (uAddButtonList(szCancel,W_KEY_ESCAPE,
                           szUp,'u',
                           szTerminal,'t', NULL))
          return;
      }
      else {
        if (uAddButtonList(szCancel,W_KEY_ESCAPE,
                           szScript,'s',
                           szTerminal,'t', NULL))
          return;
      }
      break;

  }
  /* Change state to the newly requested one and display a message */
  p_scpy(msgbuf, szChange);
  switch (uRunDialog()) {
    case W_KEY_ESCAPE:
      return;
    case 'd':
      nif_close(n);
      p_scat(msgbuf, szDown);
      wInfoMsg(msgbuf);
      break;
    case 's':
      if (nif_open(n,NIS_UP)) {
        p_scat(msgbuf, szScript);
        wInfoMsg(msgbuf);
      }
      break;
    case 't':
      if (nif_open(n,NIS_TERM)) {
        p_scat(msgbuf, szTerminal);
        wInfoMsg(msgbuf);
      }
      break;
    case 'u':
      if (nif_open(n,NIS_UP)) {
        p_scat(msgbuf, szUp);
        wInfoMsg(msgbuf);
      }
      break;
  }
}


/*******************************************************************************
***
*** Function         : ScriptDialog
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID ScriptDialog(INT iface)
{
register struct netif *n = &nif[iface];
H_DI_FSEL fsel;
TEXT fname[P_FNAMESIZE+2];
UWORD scriptused,debuglevel;

  if (n->ni_state != NIS_DOWN) {
    wInfoMsg(szInterfaceNotDown);
    return;
  }
  scriptused = n->ni_sc->si_scriptused ? 1 : 2;
  debuglevel = (UWORD) n->ni_sc->si_debuglevel;
  p_scpy(&fname[1],n->ni_sc->si_scriptname);
  fname[0]=(BYTE)p_slen(n->ni_sc->si_scriptname);
  fname[1+fname[0]]=0;

  fsel.fname=(&fname[0]);
  fsel.flags=H_FILE_PICK_SELECTOR|H_FILE_RESTRICT_LIST|H_FILE_SET_DEFEXT|
             H_FILE_ACCEPT_NULL;

  if (InterfaceDialog(iface,"script change"))
    return;
  if (uAddChoiceList("Use script?", &scriptused, szYes, szNo, NULL))
    return;
  if (uAddChoiceList("Debug level", &debuglevel, "None", "Trace", "Trace+Input", NULL))
    return;
  uSetDialogUline(2, TRUE);
  if (uAddDialogItem(H_DIALOG_FSEL, "Script:", &fsel))
    return;
  if (uRunDialog()<=0)
    return;
  fname[1+fname[0]]=0;

  p_scpy(n->ni_sc->si_scriptname,&fname[1]);
  n->ni_sc->si_scriptused = (scriptused==1);
  n->ni_sc->si_debuglevel = (byte) debuglevel & 0xff;

  config_save();
}


/*******************************************************************************
***
*** Function         : ISPDialog
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID ISPDialog(INT iface)
{
register struct netif *n = &nif[iface];
H_DI_SEDIT di_name;
TEXT ispname[ISPSTRINGMAX+1];

H_DI_SEDIT di_phone1;
TEXT phone1[ISPSTRINGMAX+1];

H_DI_SEDIT di_phone2;
TEXT phone2[ISPSTRINGMAX+1];

H_DI_SEDIT di_phone3;
TEXT phone3[ISPSTRINGMAX+1];

H_DI_SEDIT di_username;
TEXT username[ISPSTRINGMAX+1];

H_DI_XINPUT di_password;
TEXT password[ISPSTRINGMAX+1];

UWORD di_datalinkproto;

  if (n->ni_state != NIS_DOWN) {
    wInfoMsg(szInterfaceNotDown);
    return;
  }

  /* Copy data in */
  di_name.len=ISPSTRINGMAX;
  di_name.width=ISPSTRINGMAX;
  di_name.str=&ispname[0];
  uZTStoBCS(di_name.str, n->ni_ISP->isp_name);

  di_phone1.len=ISPSTRINGMAX;
  di_phone1.width=ISPSTRINGMAX;
  di_phone1.str=&phone1[0];
  uZTStoBCS(di_phone1.str, n->ni_ISP->isp_phone1);

  di_phone2.len=ISPSTRINGMAX;
  di_phone2.width=ISPSTRINGMAX;
  di_phone2.str=&phone2[0];
  uZTStoBCS(di_phone2.str, n->ni_ISP->isp_phone2);

  di_phone3.len=ISPSTRINGMAX;
  di_phone3.width=ISPSTRINGMAX;
  di_phone3.str=&phone3[0];
  uZTStoBCS(di_phone3.str, n->ni_ISP->isp_phone3);

  di_username.len=ISPSTRINGMAX;
  di_username.width=ISPSTRINGMAX;
  di_username.str=&username[0];
  uZTStoBCS(di_username.str, n->ni_ISP->isp_username);

  di_password.str=&password[0];
  uZTStoBCS(di_password.str, n->ni_ISP->isp_password);

  di_datalinkproto = n->ni_dlp - 1;
  /* To map NIDLP_SLIP to 0x01 since 0x01 = NIDLP_LOCAL */

  do { 

    /* Do the dialog */
    if (InterfaceDialog(iface,"ISP account"))
      return;

    if (uAddDialogItem(H_DIALOG_SEDIT, "ISP name", &di_name))
      return;

    uSetDialogUline(1, TRUE);
    if (uAddDialogItem(H_DIALOG_SEDIT, "\005 number 1", &di_phone1))
      return;

    if (uAddDialogItem(H_DIALOG_SEDIT, "\005 number 2", &di_phone2))
      return;

    if (uAddDialogItem(H_DIALOG_SEDIT, "\005 number 3", &di_phone3))
      return;

    uSetDialogUline(4, TRUE);
    if (uAddDialogItem(H_DIALOG_SEDIT, "User name", &di_username))
      return;

    if (uAddDialogItem(H_DIALOG_XINPUT, "Password", &di_password))
      return;

    uSetDialogUline(6, TRUE);
    if (uAddChoiceList("Data link", &di_datalinkproto, 
        "SLIP", "CSLIP", "PPP", NULL))
      return;

    if (uRunDialog()<=0)
      return;

    if (di_datalinkproto == 2)
      wInfoMsg("CSLIP is not yet supported");
  }
  while (di_datalinkproto == 2);

  /* Copy back */
  ispname[1+ispname[0]]=0;
  p_scpy(n->ni_ISP->isp_name,&ispname[1]);

  phone1[1+phone1[0]]=0;
  p_scpy(n->ni_ISP->isp_phone1,&phone1[1]);

  phone2[1+phone2[0]]=0;
  p_scpy(n->ni_ISP->isp_phone2,&phone2[1]);

  phone3[1+phone3[0]]=0;
  p_scpy(n->ni_ISP->isp_phone3,&phone3[1]);

  username[1+username[0]]=0;
  p_scpy(n->ni_ISP->isp_username,&username[1]);

  password[1+password[0]]=0;
  p_scpy(n->ni_ISP->isp_password,&password[1]);

  n->ni_dlp = di_datalinkproto + 1;

  config_save();
}


/*******************************************************************************
***
*** Function         : PasswordDialog
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID PasswordDialog(INT iface)
{
register struct netif *n = &nif[iface];
TEXT usertitle[50];

H_DI_XINPUT di_password;
TEXT password[ISPSTRINGMAX+1];

  if (n->ni_state != NIS_DOWN) {
    wInfoMsg(szInterfaceNotDown);
    return;
  }

  /* Copy data in */
  di_password.str=&password[0];
  uZTStoBCS(di_password.str, n->ni_ISP->isp_password);

  /* Do the dialog */
  p_atos(usertitle, "Password for \"%s\"", n->ni_ISP->isp_username);
  if (InterfaceDialog(iface,usertitle))
    return;

  if (uAddDialogItem(H_DIALOG_XINPUT, "Password", &di_password))
    return;

  if (uRunDialog()<=0)
    return;

  /* Copy back */
  password[1+password[0]]=0;
  p_scpy(n->ni_ISP->isp_password,&password[1]);

  config_save();
}



/*******************************************************************************
***
*** Function         : MultiAddrsDialog
*** Preconditions    : numaddrs is the number of addresses, and must be <=4
*** Postconditions   : A dialog has been presented to edit these IP addrs
***
*******************************************************************************/

GLDEF_C VOID MultiAddressesDialog(INT iface, TEXT *heading, TEXT *singular,
                                  INT numaddrs, IPaddr *addrtable)
{
int i, error;
TEXT errmsg[20];
struct addrdetail {
  H_DI_EDIT di_IPaddress;
  TEXT IPaddress[16];
  TEXT Description[16];
  IPaddr ip_IPaddress;
} copyaddr[4];

  /* Copy data in */
  for (i=0; i<numaddrs; i++) {
    copyaddr[i].di_IPaddress.len=15;
    copyaddr[i].di_IPaddress.str=&copyaddr[i].IPaddress[0];
    p_bcpy(copyaddr[i].ip_IPaddress, addrtable[i], IP_ALEN);
    p_atos(&copyaddr[i].Description[0], "%s %d", singular, i+1);
    uZTStoBCS(copyaddr[i].di_IPaddress.str, szConvertIPaddr(addrtable[i]));
  }

  /* Run dialog until valid */
  for (;;) {
    if (InterfaceDialog(iface,heading))
      return;
    for (i=0; i<numaddrs; i++) {
      if (uAddDialogItem(H_DIALOG_EDIT, copyaddr[i].Description, 
                                        &copyaddr[i].di_IPaddress))
        return;
    }
    if (uRunDialog() <= 0)
      return;

    /* Validate */
    error = FALSE;
    for (i=0; i<numaddrs; i++) {
      copyaddr[i].IPaddress[1+copyaddr[i].IPaddress[0]]=0;

      if (!IPaddrConvertsz(copyaddr[i].IPaddress+1, copyaddr[i].ip_IPaddress)) {
        p_atos(errmsg, "Address %d is invalid", i+1);
        wInfoMsg(errmsg);
        error = TRUE;
        break;
      }
    }

    if (!error)
      break;
  }

  /* All OK. Copy to main nif structure */
  for (i=0; i<numaddrs; i++) 
    p_bcpy(addrtable[i], copyaddr[i].ip_IPaddress, IP_ALEN);
  config_save();
}


/*******************************************************************************
***
*** Function         : IPAddressesDialog
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID IPAddressesDialog(INT iface)
{
register struct netif *n = &nif[iface];
H_DI_EDIT di_IPaddress;
TEXT IPaddress[16];
IPaddr ip_IPaddress;

H_DI_EDIT di_SubnetMask;
TEXT SubnetMask[16];
IPaddr ip_SubnetMask;

UWORD defaultgateway;
#define DEF_NO 1
#define DEF_YES 2

  if (n->ni_state != NIS_DOWN) {
    wInfoMsg(szInterfaceNotDown);
    return;
  }

  /* Copy data in */
  di_IPaddress.len=15;
  di_IPaddress.str=&IPaddress[0];
  p_bcpy(ip_IPaddress, n->ni_ip, IP_ALEN);
  uZTStoBCS(di_IPaddress.str, szConvertIPaddr(n->ni_ip));

  di_SubnetMask.len=15;
  di_SubnetMask.str=&SubnetMask[0];
  p_bcpy(ip_SubnetMask, n->ni_mask, IP_ALEN);
  uZTStoBCS(di_SubnetMask.str, szConvertIPaddr(n->ni_mask));

  defaultgateway = (iface == nifdefault) ? DEF_YES : DEF_NO;

  /* Run dialog until valid */
  for (;;) {
    if (InterfaceDialog(iface,"IP addresses") ||
        uAddDialogItem(H_DIALOG_EDIT, "Interface", &di_IPaddress) ||
        uAddDialogItem(H_DIALOG_EDIT, "Subnet mask", &di_SubnetMask) ||
        uAddChoiceList("Default gateway", &defaultgateway, 
          szNo, szYes, NULL) ||
        uAddButtonList(szDNSs, 'd', szGateways, 'g', 
                       szConfirm, W_KEY_RETURN, NULL))
      return;
    switch (uRunDialog()) {
      case 'd':
        MultiAddressesDialog(iface, szDNSs, "Server addr",
                             NI_DNSS, &n->ni_dnss[0]);
        continue;
      case 'g':
        MultiAddressesDialog(iface, szGateways, "Gateway addr",
                             NI_GATEWAYS, &n->ni_gateways[0]);
        continue;
      case W_KEY_RETURN:
        break;
      default: /* negative */
        return;
    }

    /* Validate */
    IPaddress[1+IPaddress[0]]=0;
    SubnetMask[1+SubnetMask[0]]=0;

    if (!IPaddrConvertsz(IPaddress+1, ip_IPaddress)) {
      wInfoMsg("Bad interface address");
      continue;
    }

    if (!IPaddrConvertsz(SubnetMask+1, ip_SubnetMask)) {
      wInfoMsg("Bad subnet mask");
      continue;
    }

    /* All OK. Copy to main nif structure */
    p_bcpy(n->ni_ip, ip_IPaddress, IP_ALEN);
    p_bcpy(n->ni_mask, ip_SubnetMask, IP_ALEN);
    if (defaultgateway == DEF_YES)
      nifdefault = iface;
    else
      nifdefault = (iface == NI_IFACE1 ? NI_IFACE2 : NI_IFACE1);
    config_save();
    return;
  }
}


/*******************************************************************************
***
*** Function         : PingDialog
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C INT PingDialog(struct ping_info *p)
{
H_DI_EDIT di_IPaddress;
TEXT IPaddress[16];
IPaddr ip_IPaddress;
H_DI_NUMBER di_size;
LONG size;
H_DI_NUMBER di_numpings;
LONG numpings;

  /* Copy data in */
  di_IPaddress.len=15;
  di_IPaddress.str=&IPaddress[0];
  p_bcpy(ip_IPaddress, p->p_ip, IP_ALEN);
  uZTStoBCS(di_IPaddress.str, szConvertIPaddr(p->p_ip));

  di_size.value=&size;
  di_size.low=1;
  di_size.high=65535L - sizeof(struct ip) - sizeof(struct icmp);
  size = p->p_size;

  di_numpings.value=&numpings;
  di_numpings.low=1;
  di_numpings.high=100;
  numpings = p->p_numpings;

  /* Run dialog until valid */
  for (;;) {
    if (uOpenDialog("Ping parameters") ||
        uAddDialogItem(H_DIALOG_EDIT, "IP address", &di_IPaddress) ||
        uAddDialogItem(H_DIALOG_NUMBER, "Payload size", &di_size) ||
        uAddDialogItem(H_DIALOG_NUMBER, "Number of pings", &di_numpings) ||
        uRunDialog()<=0)
      return FALSE;

    /* Validate */
    IPaddress[1+IPaddress[0]]=0;
    if (!IPaddrConvertsz(IPaddress+1, ip_IPaddress)) 
      wInfoMsg("Bad IP address");
    else
      break;
  }

  /* All OK. Copy to main pinginfo structure */
  p_bcpy(p->p_ip, ip_IPaddress, IP_ALEN);
  p->p_size = size;
  p->p_numpings = numpings;

  config_save();

  /* And generate the first ICMP echo request frame. 
     ping_process will be called as a result of this. */
  attn_signal(ATTN_PING);
  return TRUE;
}


/*******************************************************************************
***
*** Function         : LogDialog
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID LogDialog()
{
UWORD level, device, ao, termctrl;
int i;

  /* Copy data in */
  level = log_level;
  device = log_device;
  ao = log_openmode;
  termctrl = term_ctrlchars;

  if (uOpenDialog("Logging facilities") ||
      uAddChoiceList("Log level", &level, 
                     "Error messages", "Flow trace", 
                     "Network dump", "Transport dump", NULL) ||
      uAddChoiceList("Output device", &device, 
                     "Console", "Logfile", "Both", NULL) ||
      uAddChoiceList("Log file", &ao, 
                     "Append", "Overwrite", NULL) ||
      uAddChoiceList("Term ctrl chars", &termctrl, "Raw", "Expanded", NULL))
    return;
  uSetDialogUline(3, TRUE);
  if (uRunDialog()<=0)
    return;

  log_level = level;
  log_device = device;
  log_openmode = ao;
  term_ctrlchars = termctrl;
  for (i=0; i<NI_MAXINTERFACES; i++)
    nif_sethandler(&nif[i]);
  log_open();
  config_save();
}


/*******************************************************************************
***
*** Function         : StatsDialog
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID StatsDialog()
{
UWORD endstats;
#define DEF_NO 1
#define DEF_YES 2

  /* Copy data in */
  endstats = stack_endstats ? DEF_YES : DEF_NO;

  /* Run dialog until valid */
  if (uOpenDialog("Stats Production") ||
      uAddChoiceList("Stats on close?", &endstats, szNo, szYes, NULL) ||
      uAddButtonList("Generate now", 'n', szConfirm, W_KEY_RETURN, NULL))
      return;
  switch (uRunDialog()) {
    case 'n':
      stack_stats();
      break;
    case W_KEY_RETURN:
      break;
    default: /* negative */
      return;
  }

  stack_endstats = (endstats == DEF_YES);
  config_save();
}


/*******************************************************************************
***
*** Function         : UDPDialog
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID UDPDialog(struct udp_config *u)
{
H_DI_NUMBER di_UPPS;
LONG UPPS;
H_DI_NUMBER di_UPPLEN;
LONG UPPLEN;

  /* Copy data in */
  di_UPPS.value=&UPPS;
  di_UPPS.low=MINUPPS;
  di_UPPS.high=MAXUPPS;
  UPPS = u->udpc_UPPS;

  di_UPPLEN.value=&UPPLEN;
  di_UPPLEN.low=MINUPPLEN;
  di_UPPLEN.high=MAXUPPLEN;
  UPPLEN = u->udpc_UPPLEN;

  if (uOpenDialog("UDP settings") ||
      uAddDialogItem(H_DIALOG_NUMBER, "# UDP ports", &di_UPPS) ||
      uAddDialogItem(H_DIALOG_NUMBER, "UDP port queue size", &di_UPPLEN) ||
      uRunDialog()<=0)
    return;

  /* All OK. Copy to main udp_config structure */
  u->udpc_UPPS = UPPS;
  u->udpc_UPPLEN = UPPLEN;

  config_save();
  wInfoMsg(szNeedReload);
}


/*******************************************************************************
***
*** Function         : TCPDialog
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID TCPDialog(struct tcp_config *t)
{
UWORD bsdurg;
#define DEF_NO 1
#define DEF_YES 2

  /* Copy data in */
  bsdurg = t->tcp_bsdurg ? DEF_YES : DEF_NO;

  /* Run dialog until valid */
  if (uOpenDialog("TCP settings") ||
      uAddChoiceList("BSD urgent semantics?", &bsdurg, szNo, szYes, NULL) ||
      uRunDialog()<=0)
    return;

  /* All OK. Copy to main tcp_config structure */
  t->tcp_bsdurg = bsdurg;

  config_save();
  wInfoMsg(szNeedReload);
}


/*******************************************************************************
***
*** Function         : Confirm
*** Preconditions    : 
*** Postconditions   : Present a "No/Yes" dialog
***
*******************************************************************************/

GLDEF_C INT Confirm(TEXT *pb)
{
  if (uOpenDialog(pb))    /* title text supplied by caller */
    return(-1);
  if (uAddButtonList("No",-'n',"Yes",'y',NULL))
    return(-1);
  return(uRunDialog()!='y');   /* return TRUE iff user answered Yes */
}


