/*******************************************************************************
***
*** Filename         : dbms.h
*** Purpose          : PsiStack Database API Link Library Definitions
*** Author           : Matt J. Gumbley
*** Created          : 06/02/98
*** Last updated     : 17/02/98
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#ifndef DBMS_H
#define DBMS_H


/* Database table numbers */
#define DTN_ISP 0  /* v Internet Service Providers */
#define DTN_PHN 1  /* v ISP phone numbers */
#define DTN_SRV 2  /*   /etc/services */
#define DTN_PRT 3  /*   /etc/protocols */
#define DTN_ISV 4  /* v ISP services */
#define DTN_MPH 5  /*   Method phone numbers */
#define DTN_HCT 6  /*   Host connect table */
#define DTN_MET 7  /*   Method table */
#define DTN_DEV 8  /* v Devices table */
#define DTN_SYS 9  /*   System properties */
#define DTN_HST 10 /*   /etc/hosts */

#define DTN_MAXTABLES 11

/* Database operation codes */
#define DOP_STAT      0  /* Get information about the database system */
#define DOP_UNLOAD    1  /* Shut the database system down */
#define DOP_SEEK      2  /* Seek to a given record in the database */
#define DOP_READ      3  /* Read the current record */
#define DOP_WRITE     4  /* Overwite the current record */
#define DOP_DELETE    5  /* Delete the current record */
#define DOP_KREAD     6  /* Read record by key */
#define DOP_KWRITE    7  /* Write record by key */
#define DOP_KDELETE   8  /* Delete record by key */
#define DOP_LOCK      9  /* Lock this table against writes */
#define DOP_UNLOCK    10 /* Unlock the table */
/* May have to think again about locks... */

#define DOP_MAXOPERATIONS 11


/* Lengths of the various string fields */
#define DB_NAME 8
#define DB_DESC 20
#define DB_UINT 5
#define DB_CHAR 1

/* Structures for the records of the various files */
struct db_isp {
  BYTE isp_name[DB_NAME+1]; /* Plus null-terminator */
  BYTE isp_desc[DB_DESC+1];
  BYTE isp_def_protocol[DB_NAME+1];
  BYTE isp_def_username[DB_DESC+1]; /* Should be enough? */
  BYTE isp_def_password[DB_DESC+1]; /* Should be enough? */
  BYTE isp_def_script[DB_NAME+1]; 
};

struct db_phn {
  BYTE phn_isp_name[DB_NAME+1];
  BYTE phn_location[DB_DESC+1];
  BYTE phn_number[DB_DESC+1]; /* Should be long enough? */
};

struct db_srv {
  BYTE srv_port[DB_UINT+1];
  BYTE srv_transport[DB_NAME+1]; /* see prt_name */
  BYTE srv_name[DB_DESC+1];
};

struct db_prt {
  BYTE prt_num[DB_UINT+1];
  BYTE prt_name[DB_NAME+1]; /* Hope this is enough for a protocol name:
                               this is typically tcp, udp, telnet, etc. */
};

struct db_isv {
  BYTE isv_isp_name[DB_NAME+1];
  BYTE isv_srv_port[DB_UINT+1];
  BYTE isv_srv_transport[DB_NAME+1]; /* see prt_name */
  BYTE isv_addr[DB_DESC+1]; /* 123.123.123.123 is 15 chars - could be a FQDN? */
};

struct db_mph {
  BYTE mph_met_num[DB_UINT+1];
  BYTE mph_phn_location[DB_DESC+1];
  BYTE mph_order[DB_UINT+1];
};

struct db_hct {
  BYTE hct_num[DB_UINT+1];
  BYTE hct_met_num[DB_UINT+1];
  BYTE hct_listtype[DB_CHAR+1];
  BYTE hct_order[DB_UINT+1];
};

struct db_met {
  BYTE met_num[DB_UINT+1];
  BYTE met_desc[DB_DESC+1];
  BYTE met_isp_name[DB_NAME+1];
  BYTE met_dev_name[DB_NAME+1];
  BYTE met_isp_inh_protocol[DB_NAME+1];
  BYTE met_isp_inh_username[DB_DESC+1];
  BYTE met_isp_inh_password[DB_DESC+1];
  BYTE met_isp_inh_script[DB_NAME+1];
};

struct db_dev {
  BYTE dev_name[DB_NAME+1];
  BYTE dev_desc[DB_DESC+1];
  BYTE dev_port[DB_NAME+1]; /* Should be enough */
  BYTE dev_baudrate[DB_NAME+1];
  BYTE dev_databits[DB_CHAR+1];
  BYTE dev_stopbits[DB_CHAR+1];
  BYTE dev_handshaking[DB_CHAR+1];
  BYTE dev_parity[DB_CHAR+1];
  BYTE dev_detect_dcd[DB_CHAR+1];
  BYTE dev_detect_dsrdtr[DB_CHAR+1];
};

struct db_sys {
  BYTE sys_name[DB_NAME+1];
  BYTE sys_value[DB_DESC+1];
};

struct db_hst {
  BYTE hst_address[DB_DESC+1]; /* Always an IP address */
  BYTE hst_hostname[DB_DESC+1]; /* How long can a hostname be? */
};

/* The status report structure */
struct db_stat {
  UINT stat_tuples[DTN_MAXTABLES];
  UINT stat_locks[DTN_MAXTABLES];
  UINT stat_recordnum[DTN_MAXTABLES];
  UINT stat_requests;
};


GLREF_C HANDLE dbms_loaded(VOID);
GLREF_C HANDLE dbms_load(VOID);

/* All database operations are done like this: */
GLREF_C UINT dbms_send(BYTE table, BYTE op, UINT recno, VOID *rec, UWORD len);

/* Although you never call that... you use these instead... */
#define db_stat(stat)       dbms_send(0,       DOP_STAT,    -1,    stat, sizeof(struct db_stat)))
#define db_unload()         dbms_send(0,       DOP_UNLOAD,  -1,    NULL, 0)

#define db_isp_seek(recno)  dbms_send(DTN_ISP, DOP_SEEK,    recno, NULL, 0)
#define db_isp_read(buf)    dbms_send(DTN_ISP, DOP_READ,    -1,    buf, sizeof(struct db_isp))
#define db_isp_write(buf)   dbms_send(DTN_ISP, DOP_WRITE,   -1,    buf, sizeof(struct db_isp))
#define db_isp_delete()     dbms_send(DTN_ISP, DOP_DELETE,  -1,    NULL, 0)
#define db_isp_kread(buf)   dbms_send(DTN_ISP, DOP_KREAD,   -1,    buf, sizeof(struct db_isp))
#define db_isp_kwrite(buf)  dbms_send(DTN_ISP, DOP_KWRITE,  -1,    buf, sizeof(struct db_isp))
#define db_isp_kdelete(buf) dbms_send(DTN_ISP, DOP_KDELETE, -1,    buf, sizeof(struct db_isp))
#define db_isp_lock()       dbms_send(DTN_ISP, DOP_LOCK,    -1,    NULL, 0)
#define db_isp_unlock()     dbms_send(DTN_ISP, DOP_UNLOCK,  -1,    NULL, 0)
 
#define db_phn_seek(recno)  dbms_send(DTN_PHN, DOP_SEEK,    recno, NULL, 0)
#define db_phn_read(buf)    dbms_send(DTN_PHN, DOP_READ,    -1,    buf, sizeof(struct db_phn))
#define db_phn_write(buf)   dbms_send(DTN_PHN, DOP_WRITE,   -1,    buf, sizeof(struct db_phn))
#define db_phn_delete()     dbms_send(DTN_PHN, DOP_DELETE,  -1,    NULL, 0)
#define db_phn_kread(buf)   dbms_send(DTN_PHN, DOP_KREAD,   -1,    buf, sizeof(struct db_phn))
#define db_phn_kwrite(buf)  dbms_send(DTN_PHN, DOP_KWRITE,  -1,    buf, sizeof(struct db_phn))
#define db_phn_kdelete(buf) dbms_send(DTN_PHN, DOP_KDELETE, -1,    buf, sizeof(struct db_phn))
#define db_phn_lock()       dbms_send(DTN_PHN, DOP_LOCK,    -1,    NULL, 0)
#define db_phn_unlock()     dbms_send(DTN_PHN, DOP_UNLOCK,  -1,    NULL, 0)
 
#define db_srv_seek(recno)  dbms_send(DTN_SRV, DOP_SEEK,    recno, NULL, 0)
#define db_srv_read(buf)    dbms_send(DTN_SRV, DOP_READ,    -1,    buf, sizeof(struct db_srv))
#define db_srv_write(buf)   dbms_send(DTN_SRV, DOP_WRITE,   -1,    buf, sizeof(struct db_srv))
#define db_srv_delete()     dbms_send(DTN_SRV, DOP_DELETE,  -1,    NULL, 0)
#define db_srv_kread(buf)   dbms_send(DTN_SRV, DOP_KREAD,   -1,    buf, sizeof(struct db_srv))
#define db_srv_kwrite(buf)  dbms_send(DTN_SRV, DOP_KWRITE,  -1,    buf, sizeof(struct db_srv))
#define db_srv_kdelete(buf) dbms_send(DTN_SRV, DOP_KDELETE, -1,    buf, sizeof(struct db_srv))
#define db_srv_lock()       dbms_send(DTN_SRV, DOP_LOCK,    -1,    NULL, 0)
#define db_srv_unlock()     dbms_send(DTN_SRV, DOP_UNLOCK,  -1,    NULL, 0)
 
#define db_prt_seek(recno)  dbms_send(DTN_PRT, DOP_SEEK,    recno, NULL, 0)
#define db_prt_read(buf)    dbms_send(DTN_PRT, DOP_READ,    -1,    buf, sizeof(struct db_prt))
#define db_prt_write(buf)   dbms_send(DTN_PRT, DOP_WRITE,   -1,    buf, sizeof(struct db_prt))
#define db_prt_delete()     dbms_send(DTN_PRT, DOP_DELETE,  -1,    NULL, 0)
#define db_prt_kread(buf)   dbms_send(DTN_PRT, DOP_KREAD,   -1,    buf, sizeof(struct db_prt))
#define db_prt_kwrite(buf)  dbms_send(DTN_PRT, DOP_KWRITE,  -1,    buf, sizeof(struct db_prt))
#define db_prt_kdelete(buf) dbms_send(DTN_PRT, DOP_KDELETE, -1,    buf, sizeof(struct db_prt))
#define db_prt_lock()       dbms_send(DTN_PRT, DOP_LOCK,    -1,    NULL, 0)
#define db_prt_unlock()     dbms_send(DTN_PRT, DOP_UNLOCK,  -1,    NULL, 0)
 
#define db_isv_seek(recno)  dbms_send(DTN_ISV, DOP_SEEK,    recno, NULL, 0)
#define db_isv_read(buf)    dbms_send(DTN_ISV, DOP_READ,    -1,    buf, sizeof(struct db_isv))
#define db_isv_write(buf)   dbms_send(DTN_ISV, DOP_WRITE,   -1,    buf, sizeof(struct db_isv))
#define db_isv_delete()     dbms_send(DTN_ISV, DOP_DELETE,  -1,    NULL, 0)
#define db_isv_kread(buf)   dbms_send(DTN_ISV, DOP_KREAD,   -1,    buf, sizeof(struct db_isv))
#define db_isv_kwrite(buf)  dbms_send(DTN_ISV, DOP_KWRITE,  -1,    buf, sizeof(struct db_isv))
#define db_isv_kdelete(buf) dbms_send(DTN_ISV, DOP_KDELETE, -1,    buf, sizeof(struct db_isv))
#define db_isv_lock()       dbms_send(DTN_ISV, DOP_LOCK,    -1,    NULL, 0)
#define db_isv_unlock()     dbms_send(DTN_ISV, DOP_UNLOCK,  -1,    NULL, 0)
 
#define db_mph_seek(recno)  dbms_send(DTN_MPH, DOP_SEEK,    recno, NULL, 0)
#define db_mph_read(buf)    dbms_send(DTN_MPH, DOP_READ,    -1,    buf, sizeof(struct db_mph))
#define db_mph_write(buf)   dbms_send(DTN_MPH, DOP_WRITE,   -1,    buf, sizeof(struct db_mph))
#define db_mph_delete()     dbms_send(DTN_MPH, DOP_DELETE,  -1,    NULL, 0)
#define db_mph_kread(buf)   dbms_send(DTN_MPH, DOP_KREAD,   -1,    buf, sizeof(struct db_mph))
#define db_mph_kwrite(buf)  dbms_send(DTN_MPH, DOP_KWRITE,  -1,    buf, sizeof(struct db_mph))
#define db_mph_kdelete(buf) dbms_send(DTN_MPH, DOP_KDELETE, -1,    buf, sizeof(struct db_mph))
#define db_mph_lock()       dbms_send(DTN_MPH, DOP_LOCK,    -1,    NULL, 0)
#define db_mph_unlock()     dbms_send(DTN_MPH, DOP_UNLOCK,  -1,    NULL, 0)
 
#define db_hct_seek(recno)  dbms_send(DTN_HCT, DOP_SEEK,    recno, NULL, 0)
#define db_hct_read(buf)    dbms_send(DTN_HCT, DOP_READ,    -1,    buf, sizeof(struct db_hct))
#define db_hct_write(buf)   dbms_send(DTN_HCT, DOP_WRITE,   -1,    buf, sizeof(struct db_hct))
#define db_hct_delete()     dbms_send(DTN_HCT, DOP_DELETE,  -1,    NULL, 0)
#define db_hct_kread(buf)   dbms_send(DTN_HCT, DOP_KREAD,   -1,    buf, sizeof(struct db_hct))
#define db_hct_kwrite(buf)  dbms_send(DTN_HCT, DOP_KWRITE,  -1,    buf, sizeof(struct db_hct))
#define db_hct_kdelete(buf) dbms_send(DTN_HCT, DOP_KDELETE, -1,    buf, sizeof(struct db_hct))
#define db_hct_lock()       dbms_send(DTN_HCT, DOP_LOCK,    -1,    NULL, 0)
#define db_hct_unlock()     dbms_send(DTN_HCT, DOP_UNLOCK,  -1,    NULL, 0)
 
#define db_met_seek(recno)  dbms_send(DTN_MET, DOP_SEEK,    recno, NULL, 0)
#define db_met_read(buf)    dbms_send(DTN_MET, DOP_READ,    -1,    buf, sizeof(struct db_met))
#define db_met_write(buf)   dbms_send(DTN_MET, DOP_WRITE,   -1,    buf, sizeof(struct db_met))
#define db_met_delete()     dbms_send(DTN_MET, DOP_DELETE,  -1,    NULL, 0)
#define db_met_kread(buf)   dbms_send(DTN_MET, DOP_KREAD,   -1,    buf, sizeof(struct db_met))
#define db_met_kwrite(buf)  dbms_send(DTN_MET, DOP_KWRITE,  -1,    buf, sizeof(struct db_met))
#define db_met_kdelete(buf) dbms_send(DTN_MET, DOP_KDELETE, -1,    buf, sizeof(struct db_met))
#define db_met_lock()       dbms_send(DTN_MET, DOP_LOCK,    -1,    NULL, 0)
#define db_met_unlock()     dbms_send(DTN_MET, DOP_UNLOCK,  -1,    NULL, 0)
 
#define db_dev_seek(recno)  dbms_send(DTN_DEV, DOP_SEEK,    recno, NULL, 0)
#define db_dev_read(buf)    dbms_send(DTN_DEV, DOP_READ,    -1,    buf, sizeof(struct db_dev))
#define db_dev_write(buf)   dbms_send(DTN_DEV, DOP_WRITE,   -1,    buf, sizeof(struct db_dev))
#define db_dev_delete()     dbms_send(DTN_DEV, DOP_DELETE,  -1,    NULL, 0)
#define db_dev_kread(buf)   dbms_send(DTN_DEV, DOP_KREAD,   -1,    buf, sizeof(struct db_dev))
#define db_dev_kwrite(buf)  dbms_send(DTN_DEV, DOP_KWRITE,  -1,    buf, sizeof(struct db_dev))
#define db_dev_kdelete(buf) dbms_send(DTN_DEV, DOP_KDELETE, -1,    buf, sizeof(struct db_dev))
#define db_dev_lock()       dbms_send(DTN_DEV, DOP_LOCK,    -1,    NULL, 0)
#define db_dev_unlock()     dbms_send(DTN_DEV, DOP_UNLOCK,  -1,    NULL, 0)
 
#define db_sys_seek(recno)  dbms_send(DTN_SYS, DOP_SEEK,    recno, NULL, 0)
#define db_sys_read(buf)    dbms_send(DTN_SYS, DOP_READ,    -1,    buf, sizeof(struct db_sys))
#define db_sys_write(buf)   dbms_send(DTN_SYS, DOP_WRITE,   -1,    buf, sizeof(struct db_sys))
#define db_sys_delete()     dbms_send(DTN_SYS, DOP_DELETE,  -1,    NULL, 0)
#define db_sys_kread(buf)   dbms_send(DTN_SYS, DOP_KREAD,   -1,    buf, sizeof(struct db_sys))
#define db_sys_kwrite(buf)  dbms_send(DTN_SYS, DOP_KWRITE,  -1,    buf, sizeof(struct db_sys))
#define db_sys_kdelete(buf) dbms_send(DTN_SYS, DOP_KDELETE, -1,    buf, sizeof(struct db_sys))
#define db_sys_lock()       dbms_send(DTN_SYS, DOP_LOCK,    -1,    NULL, 0)
#define db_sys_unlock()     dbms_send(DTN_SYS, DOP_UNLOCK,  -1,    NULL, 0)

#define db_hst_seek(recno)  dbms_send(DTN_HST, DOP_SEEK,    recno, NULL, 0)
#define db_hst_read(buf)    dbms_send(DTN_HST, DOP_READ,    -1,    buf, sizeof(struct db_hst))
#define db_hst_write(buf)   dbms_send(DTN_HST, DOP_WRITE,   -1,    buf, sizeof(struct db_hst))
#define db_hst_delete()     dbms_send(DTN_HST, DOP_DELETE,  -1,    NULL, 0)
#define db_hst_kread(buf)   dbms_send(DTN_HST, DOP_KREAD,   -1,    buf, sizeof(struct db_hst))
#define db_hst_kwrite(buf)  dbms_send(DTN_HST, DOP_KWRITE,  -1,    buf, sizeof(struct db_hst))
#define db_hst_kdelete(buf) dbms_send(DTN_HST, DOP_KDELETE, -1,    buf, sizeof(struct db_hst))
#define db_hst_lock()       dbms_send(DTN_HST, DOP_LOCK,    -1,    NULL, 0)
#define db_hst_unlock()     dbms_send(DTN_HST, DOP_UNLOCK,  -1,    NULL, 0)

#endif /* DBMS_H */
