/*******************************************************************************
***
*** Filename         : asy.h
*** Purpose          : Data Definitions for the ASY physical layer in PsiStack
*** Author           : Matt J. Gumbley
*** Created          : 16/01/97
*** Last updated     : 01/03/98
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#ifndef SYS_ASY_H
#define SYS_ASY_H

/* Preprocessor definitions */

#define IGNORE 0

/* These are convenient definitions, since Psion's are a bit complex */
#define NOPAR 1
#define EVENPAR 2
#define ODDPAR 3

#define XONXOFF 1
#define RTSCTS 2

#define ASY_SERBUFSIZE    300                        /* Size of serial buffer */

struct asy {
  TEXT asy_serDeviceName[10];            /* Name of the device: TTY:A */
  VOID *asy_serH;
  WORD asy_serStatus;
  WORD asy_serLength;
  UBYTE asy_serBuffer[ASY_SERBUFSIZE];
  /* This is an optimisation for the serial read routine in psistack.c
     Whenever the state of the interface changes, or the term_ctrlchars
     variable changes, nif_sethandler() is called, and this sets the
     serial handler to point to either:
       discard_handler() in psistack.c
       script_handler() in script.c
       slip_handler() or ppp_inhandler() in slip.c / ppp.c
       term_raw_handler() or term_expanded_handler() in psistack.c
  */
  VOID (*asy_serHandler)(VOID); 
  INT asy_speed;                       /* Bits per second, P_BAUD_xxx */
  INT asy_parity;                /* IGNORE, NOPAR, EVENPAR, or ODDPAR */
  INT asy_databits;                           /* P_DATA_7 or P_DATA_8 */
  INT asy_stopbits;                                 /* STOP1 or STOP2 */
  BYTE asy_handshake;                    /* XONXOFF, RTSCTS or IGNORE */
  BYTE asy_devicedetect;         /* DSR/DTR: 1 is detect, 2 is ignore */
  BYTE asy_carrierdetect;            /* DCD: 1 is detect, 2 is ignore */
};


#endif /* SYS_ASY_H */

