/*******************************************************************************
***
*** Filename         : buffer.h
*** Purpose          : Linked memory buffer ADT definitions and prototypes
*** Author           : Matt J. Gumbley
*** Created          : 02/02/97
*** Last updated     : 01/03/98
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#ifndef SYS_BUFFER_H
#define SYS_BUFFER_H

/* The individual buffer descriptors, and entries in the buffer pool, look like
   this:
*/

#define BUF_FIRST 0x80          /* Set if this is the first buffer in a chain */
                                   /* Is this really necessary or useful? */
#define BUF_INUSE 0x40   /* Set when a buffer is allocated, cleared otherwise */

struct buf {
  int allocsize;              /* How many bytes are allocated for this buffer */
  int usedsize;         /* How many bytes are currently stored in this buffer */
  struct buf *curr;   /* Buffer in chain containing the current buildup point */
  long order;   /* For ordered, chained buffers. Typically, a fragment offset.
                    queue_enqueue_ordered(_,_,i) uses this - i is stored here
                  This field used to be int, but since TCP uses longs for its
                             sequence numbers, this has had to be increased.  */
  byte flags;                                 /* Some of the BUF_ codes above */
  byte *data;                      /* The allocated data area for this buffer */
  struct buf *next;                  /* The next buffer in the chain, or NULL */
};


#endif /* SYS_BUFFER_H */
