/*******************************************************************************
***
*** Filename         : data.h
*** Purpose          : Data model details structure
*** Author           : Matt J. Gumbley
*** Created          : 09/04/97
*** Last updated     : 07/03/98
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#ifndef SYS_DATA_H
#define SYS_DATA_H

#define ISPSTRINGMAX 20

struct ISPcontext {
  char isp_name[ISPSTRINGMAX];                            /* Name of this ISP */
  char isp_phone1[ISPSTRINGMAX];                  /* Their first phone number */
  char isp_phone2[ISPSTRINGMAX];                 /* Their second phone number */
  char isp_phone3[ISPSTRINGMAX];                  /* Their third phone number */
  char isp_username[ISPSTRINGMAX];                   /* The user's login name */
  char isp_password[ISPSTRINGMAX]; /* Their password or null string to prompt */
};

/* M I S C E L L A N E O U S   K E R N E L   F L A G S
 *
 * Logging enable/disable flags
 * Expand control charcters into <xx> in TERM mode
 * Generate stats on stack close
 *
 * Changed from using a log level to using log flags. This is more flexible.
 * If we need the level functionality, simply set all flags up to a given bit
 * number. However, might just want to drop one type of logging, which we
 * couldn't do with the level-based system. 
 *
 * There's no "Error Messages" flag, since these are *always* output.
 *
 * We're also storing the "do we expand control characters received in term 
 * mode" and "generate stats on stack close" flags here, it's just easier 
 * that way. 
 *
 */

#define LOG_FLAG_MSG       0x0001
#define LOG_FLAG_FLOWTRACE 0x0002
#define LOG_FLAG_LINKDUMP  0x0004
#define LOG_FLAG_IPDUMP    0x0008
#define LOG_FLAG_ICMPDUMP  0x0010
#define LOG_FLAG_UPPERDUMP 0x0020  /* UDP, TCP, etc. */
#define LOG_FLAG_TIMESTAMP 0x0040  /* Whether stack does console_timestamp() */

#define MISC_FLAG_ENDSTATS 0x4000
#define MISC_FLAG_TERMCTRLCHARS 0x8000

struct flags {
  UWORD log_flag;
  UWORD misc_flag;
};

/* Also, we don't handle logging to a file in the stack any more - that's a job
 * for the standalone console program. */

#endif /* SYS_DATA_H */

