/*******************************************************************************
***
*** Filename         : icmp.h
*** Purpose          : Definitions for Internet Control Message Protocol 
*** Author           : Matt J. Gumbley
*** Created          : 21/01/97
*** Last updated     : 01/03/98
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#ifndef SYS_ICMP_H
#define SYS_ICMP_H

/* Message types */
#define	ICT_ECHORP        0                                     /* Echo Reply */
#define	ICT_DESTUR        3                        /* Destination Unreachable */
#define	ICT_SRCQ          4                                  /* Source Quench */
#define	ICT_REDIRECT      5                                       /* Redirect */
#define	ICT_ECHORQ        8                                   /* Echo Request */
#define	ICT_TIMEX         11                         /* Time-to-live Exceeded */
#define	ICT_PARAMP        12                             /* Parameter Problem */
#define	ICT_TIMERQ        13                             /* Timestamp Request */
#define	ICT_TIMERP        14                               /* Timestamp Reply */
#define	ICT_INFORQ        15                           /* Information Request */
#define	ICT_INFORP        16                             /* Information Reply */
#define ICT_MASKRQ        17                          /* Address mask Request */
#define ICT_MASKRP        18                            /* Address mask Reply */

/* icmp_code types */
/* Destination Unreachable codes */
#define	ICC_NETUR         0       /* Destination unreachable, net unreachable */
#define	ICC_HOSTUR        1      /* Destination unreachable, host unreachable */
#define	ICC_PROTUR        2  /* Destination unreachable, protocol unreachable */
#define	ICC_PORTUR        3      /* Destination unreachable, port unreachable */
#define	ICC_FNADF         4    /* Dest. unr., fragmentation needed and DF set */
#define	ICC_SRCRF         5   /* Destination unreachable, source route failed */

/* Time Exceeded codes */
#define	ICC_TIMEX         0                          /* Time-to-live exceeded */
#define	ICC_FTIMEX        1              /* Fragment reassembly time exceeded */

/* Redirect message codes */
#define	ICC_NETRD         0                       /* Redirect for the network */
#define	ICC_HOSTRD        1                          /* Redirect for the host */
#define	IC_TOSNRD         2          /* Redirect for Type of Service, network */
#define	IC_TOSHRD         3             /* Redirect for Type of Service, host */

/* Format of an ICMP header */
#define	ICMP_HLEN         8               /* Length of ICMP header on the net */
#define ICMP_PADLEN       3                            /* Pad length in bytes */
	
struct icmp {
  byte icmp_type;                             /* Type of message: ICT_* above */
  byte icmp_code;                                        /* Code: ICC_* above */
  word16 icmp_cksum;                          /* Checksum of ICMP header+data */
  union {
    struct {
      word16 icmp1_id;                         /* For echo type, a message ID */
      word16 icmp1_seq;                   /* For echo type, a sequence number */
    } icmp1;
    IPaddr icmp2_gw;                               /* For redirect, a gateway */
    struct {
      byte icmp3_ptr;                              /* Pointer, for ICT_PARAMP */
      byte icmp3_pad[ICMP_PADLEN];
    } icmp3;
    word32 icmp4_mbz;              /* Must be zero - padding to 32-bit length */
  } icmpu;
  byte icmp_data[1];                                  /* Variable data length */
};

/* Format 1 */
#define icmp_id           icmpu.icmp1.icmp1_id
#define icmp_seq          icmpu.icmp1.icmp1_seq

/* Format 2 */
#define icmp_gw           icmpu.icmp2_gw

/* Format 3 */
#define icmp_ptr          icmpu.icmp3.icmp3_ptr
#define icmp_pad          icmpu.icmp3.icmp3_pad

/* Format 4 */
#define icmp_mbz          icmpu.icmp4_mbz

struct ping_info {
  IPaddr p_ip;
  LONG p_size;
  LONG p_numpings;
};


#endif /* SYS_ICMP_H */
