/*******************************************************************************
***
*** Filename         : ip.h
*** Purpose          : Definitions of the IP datagram
*** Author           : Matt J. Gumbley
*** Created          : 19/01/97
*** Last updated     : 01/03/98
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#ifndef SYS_IP_H
#define SYS_IP_H

#define IP_ALEN 4                               /* IP address length in bytes */
typedef byte IPaddr[IP_ALEN];                 /* 'Structure' of an IP address */

/* Which class address is this? */
#define IP_CLASSA(x) ((x[0] & 0x80) == 0x00)            /* IP Class A address */
#define IP_CLASSB(x) ((x[0] & 0xc0) == 0x80)            /* IP Class B address */
#define IP_CLASSC(x) ((x[0] & 0xe0) == 0xc0)            /* IP Class C address */
#define IP_CLASSD(x) ((x[0] & 0xf0) == 0xe0)            /* IP Class D address */
#define IP_CLASSE(x) ((x[0] & 0xf8) == 0xf0)            /* IP Class E address */

/* Some assigned protocol numbers */
#define IPT_ICMP    1                       /* protocol type for ICMP packets */
#define IPT_IGMP    2
#define IPT_GGP     3
#define IPT_TCP     6                        /* protocol type for TCP packets */
#define IPT_EGP     8
#define IPT_IGP     9
#define IPT_UDP     17                       /* protocol type for UDP packets */
#define IPT_ISOTP4  29
#define IPT_IGRP    88
#define IPT_OSPFIGP 89

/* IP datagram format */
struct ip {
  byte   ip_verlen;                  /* IP version & header length (in longs) */
  byte   ip_tos;                                           /* Type of service */
  word16 ip_len;                            /* Total packet length (in bytes) */
  word16 ip_id;                                                /* Datagram ID */
  word16 ip_fragoff;                         /* Fragment offset (in 8-byte's) */
  byte   ip_ttl;                             /* Time to live, in gateway hops */
  byte   ip_proto;                           /* IP protocol - see IPT_* above */
  word16 ip_cksum;                                         /* Header checksum */
  IPaddr ip_src;                                      /* IP address of source */
  IPaddr ip_dst;                                 /* IP address of destination */
  byte   ip_data[1];                                  /* Variable length data */
};

#define IP_VERSION 4                                 /* Current version value */

/* Macro to compute a datagram's header length in bytes */
#define IP_HLEN(pip) ((pip->ip_verlen & 0xf)<<2)

#define IPMHLEN    20                  /* Minimum IP header length (in bytes) */
#define IP_MINHLEN 5                   /* Minimum IP header length (in longs) */

#define IP_INIT_TTL  255                        /* Initial time-to-live value */

#define IP_MF      0x2000                             /* 'More Fragments' bit */
#define IP_DF      0x4000                             /* 'Don't Fragment' bit */
#define IP_FRAGOFF 0x1fff                             /* Fragment offset mask */
#define IP_PREC    0xe0                          /* Precedence portion of TOS */

#define IP_MAXSIZE 2048          /* The largest IP datagram we can reassemble */

/* IP Precedence values */
#define IPP_NETCTL 0xe0                                    /* Network control */
#define IPP_INCTL  0xc0                                   /* Internet control */
#define IPP_CRIT   0xa0                                           /* Critical */
#define IPP_FLASHO 0x80                                    /* Flash over-ride */
#define IPP_FLASH  0x60                                              /* Flash */
#define IPP_IMMED  0x40                                          /* Immediate */
#define IPP_PRIO   0x20                                           /* Priority */
#define IPP_NORMAL 0x00                                             /* Normal */

/* IP options */
#define IPO_COPY   0x80                              /* Copy on fragment mask */
#define IPO_CLASS  0x60                                       /* Option class */
#define IPO_NUM    0x17                                      /* Option number */
#define IPO_EOOP   0x00                                     /* End of options */
#define IPO_NOP    0x01                                       /* No operation */
#define IPO_SEC    0x82                  /* DoD security/compartmentalization */
#define IPO_LSRCRT 0x83                               /* Loose source routing */
#define IPO_SSRCRT 0x89                              /* Strict source routing */
#define IPO_RECRT  0x07                                       /* Record route */
#define IPO_STRID  0x88                                          /* Stream ID */
#define IPO_TIME   0x44                                 /* Internet timestamp */

#endif /* SYS_IP_H */

