/*******************************************************************************
***
*** Filename         : nif.h
*** Purpose          : Network Interface abstraction definitions
*** Author           : Matt J. Gumbley, Patrick H. Robbe
*** Created          : 19/01/97
*** Last updated     : 02/07/98
***
********************************************************************************
***
*** Modification Record: 
***
*******************************************************************************/

#ifndef SYS_NIF_H
#define SYS_NIF_H


/* Interface states */
#define NIS_UP       0x1
#define NIS_DOWN     0x2
#ifdef WANT_SCRIPT
#define NIS_SCRIPT   0x3
#endif 
#ifdef WANT_TERM
#define NIS_TERM     0x4
#endif
#ifdef WANT_PPP
#define NIS_PPP_UP   0x5 /* Intermediate state after physical connection,
                            during PPP negotiation, but before we go NIS_UP */
#define NIS_PPP_DOWN 0x6 /* Similar during PPP link tear-down before NIS_DOWN */
#endif

#define NIS_MAXSTATES 6                         /* How many states are there? */

/* Data link protocol types */
#define NIDLP_LOCAL  0x1
#ifdef WANT_SLIP
#define NIDLP_SLIP   0x2
#endif
#ifdef WANT_CSLIP
#define NIDLP_CSLIP  0x3
#endif
#ifdef WANT_PPP
#define NIDLP_PPP    0x4
#endif
#define NIDLP_KISS   0x5 /* eventually... */

#define NI_INQSZ 30                             /* Interface input queue size */
#define NETNLEN 30                                  /* Length of network name */

#define NI_BUILDUPSZ 1024                    /* Max size of an incoming frame */

/* Definition of network interface structure (one per interface, so there are
   two on the Psion-TCP/IP software; one for the serial device, 
   one for local loopback.
*/
#define NI_MAXINTERFACES 2
#define NI_LOCAL         0                        /* Index of local interface */
#define NI_NETWORK       1                      /* Index of network interface */

#define NI_GATEWAYS      4       /* Number of gateway addresses per interface */
#define NI_DNSS          2           /* Number of DNS addresses per interface */


struct netif {
  BYTE ni_state;             /* Interface state: One of the NIS_ states above */
  BYTE ni_dlp;           /* Data link protocol: One of the NIDLP_ types above */

  IPaddr ni_ip;                              /* IP address for this interface */
  IPaddr ni_net;                                        /* Network IP address */
  IPaddr ni_subnet;                                  /* Subnetwork IP address */
  IPaddr ni_mask;                             /* IP subnet mask for interface */
  IPaddr ni_brc;                                      /* IP broadcast address */
  IPaddr ni_nbrc;                                 /* IP net broadcast address */
  IPaddr ni_gateways[NI_GATEWAYS];                    /* IP gateway addresses */
  IPaddr ni_dnss[NI_DNSS];                         /* IP DNS server addresses */

  INT ni_mtu;                                  /* Max. transfer unit in bytes */

  struct queue *ni_ipinq;          /* Queue: data from the device going to IP */
  struct queue *ni_devoutq;              /* Queue: data from IP to the device */

  /* Interface stats */

  /* Input (NET -> IP) stats */
  INT ni_ioctets;                                     /* # of octets received */
  INT ni_iucast;                                     /* # of unicast received */
  INT ni_inucast;                                /* # of non-unicast received */
  INT ni_idiscard;                            /* # dropped; output queue full */
  INT ni_ierrors;                                    /* # input packet errors */
  INT ni_iunkproto;                  /* # input packets for unknown protocols */
  INT ni_iaddress;                   /* # input packets for class D or E addr */

  /* Output (NET <- IP) stats */
  INT ni_ooctets;                                         /* # of octets sent */
  INT ni_oucast;                                         /* # of unicast sent */
  INT ni_onucast;                                    /* # of non-unicast sent */
  INT ni_odiscard;                                             /* # discarded */
  INT ni_oerrors;                                   /* # output packet errors */
  INT ni_oqlen;                                        /* output queue length */
  INT ni_maxreasm;                             /* Max datagram can reassemble */
};


#endif /* SYS_NIF_H */
