/*******************************************************************************
***
*** Filename         : socket.h
*** Purpose          : PsiStack Sockets EPOC style API Link Library Definitions
*** Author           : Matt J. Gumbley
*** Created          : 16/04/98
*** Last updated     : 17/04/98
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#ifndef SYS_SOCKET_H
#define SYS_SOCKET_H

struct sockaddr {
  unsigned char  sa_len;                                      /* Total length */
  unsigned short sa_family;                        /* Address family, AF_xxx  */
  char           sa_data[14];                /* 14 bytes of protocol address  */
};

struct sockaddr_in {
  unsigned char  sin_len;                                     /* Total length */
  unsigned short sin_family;                       /* Address family, AF_xxx  */
  unsigned short sin_port;                            /* Protocol port number */
  unsigned long  sin_addr;                                      /* IP address */
  char           sin_zero[8];
};

struct linger {
  int            l_onoff;                                    /* Linger active */
  int            l_linger;                          /* How long to linger for */
};


/* Socket types. */
#define SOCK_STREAM  1            /* Stream (TCP, connection-oriented) socket */
#define SOCK_DGRAM   2               /* Datagram (UDP, connectionless) socket */
#define SOCK_RAW     3                                       /* Raw IP socket */
#define SOCK_RAWINTERFACE 4    /* Raw data over an interface, for the dialler */

/* Supported address families. */
#define AF_INET      2                                /* Internet IP Protocol */
#define AF_MAX       1                                           /* For now.. */

/* Protocol families, same as address families. */
#define PF_INET      AF_INET
#define PF_MAX       AF_MAX

/* Maximum queue length specifiable by listen.  */
#define SOMAXCONN    5

/* Flags we can use with send and recv. */
#define MSG_OOB        1
#define MSG_PEEK       2
#define MSG_DONTROUTE  4
#define MSG_PROXY      16                    /* Supply or ask second address. */

/* Setsockoptions(2) level. Thanks to BSD these must match IPPROTO_xxx */
#define SOL_IP       0
#define SOL_TCP      6
#define SOL_UDP      17

/* IP options */
#define IP_TOS    1
#define  IPTOS_LOWDELAY     0x10
#define  IPTOS_THROUGHPUT   0x08
#define  IPTOS_RELIABILITY  0x04
#define  IPTOS_MINCOST      0x02
#define IP_TTL    2
#define IP_HDRINCL  3
#define IP_OPTIONS  4

#define IP_MULTICAST_IF     32
#define IP_MULTICAST_TTL    33
#define IP_MULTICAST_LOOP   34
#define IP_ADD_MEMBERSHIP   35
#define IP_DROP_MEMBERSHIP  36

/* These need to appear somewhere around here */
#define IP_DEFAULT_MULTICAST_TTL        1
#define IP_DEFAULT_MULTICAST_LOOP       1
#define IP_MAX_MEMBERSHIPS              20
 
/* TCP options - this way around because someone left a set in the C 
   library includes */
#define TCP_NODELAY  1
#define TCP_MAXSEG   2

/* The various priorities. */
#define SOPRI_INTERACTIVE  0
#define SOPRI_NORMAL       1
#define SOPRI_BACKGROUND   2

/* Standard errno codes, converted from Linux' errno.h */

#define E_NET_PERM            -101      /* Operation not permitted */
#define E_NET_NOENT           -102      /* No such file or directory */
#define E_NET_SRCH            -103      /* No such process */
#define E_NET_INTR            -104      /* Interrupted system call */
#define E_NET_IO              -105      /* I/O error */
#define E_NET_NXIO            -106      /* No such device or address */
#define E_NET_2BIG            -107      /* Arg list too long */
#define E_NET_NOEXEC          -108      /* Exec format error */
#define E_NET_BADF            -109      /* Bad file number */
#define E_NET_CHILD           -110      /* No child processes */
#define E_NET_AGAIN           -111      /* Try again */
#define E_NET_NOMEM           -112      /* Out of memory */
#define E_NET_ACCES           -113      /* Permission denied */
#define E_NET_FAULT           -114      /* Bad address */
#define E_NET_NOTBLK          -115      /* Block device required */
#define E_NET_BUSY            -116      /* Device or resource busy */
#define E_NET_EXIST           -117      /* File exists */
#define E_NET_XDEV            -118      /* Cross-device link */
#define E_NET_NODEV           -119      /* No such device */
#define E_NET_NOTDIR          -120      /* Not a directory */
#define E_NET_ISDIR           -121      /* Is a directory */
#define E_NET_INVAL           -122      /* Invalid argument */
#define E_NET_NFILE           -123      /* File table overflow */
#define E_NET_MFILE           -124      /* Too many open files */
#define E_NET_NOTTY           -125      /* Not a typewriter */
#define E_NET_TXTBSY          -126      /* Text file busy */
#define E_NET_FBIG            -127      /* File too large */
#define E_NET_NOSPC           -128      /* No space left on device */
#define E_NET_SPIPE           -129      /* Illegal seek */
#define E_NET_ROFS            -130      /* Read-only file system */
#define E_NET_MLINK           -131      /* Too many links */
#define E_NET_PIPE            -132      /* Broken pipe */
#define E_NET_DOM             -133      /* Math argument out of domain of func */
#define E_NET_RANGE           -134      /* Math result not representable */
#define E_NET_DEADLK          -135      /* Resource deadlock would occur */
#define E_NET_NAMETOOLONG     -136      /* File name too long */
#define E_NET_NOLCK           -137      /* No record locks available */
#define E_NET_NOSYS           -138      /* Function not implemented */
#define E_NET_NOTEMPTY        -139      /* Directory not empty */
#define E_NET_LOOP            -140      /* Too many symbolic links encountered */
#define E_NET_WOULDBLOCK     E_NET_AGAIN  /* Operation would block */
#define E_NET_NOMSG           -142      /* No message of desired type */
#define E_NET_IDRM            -143      /* Identifier removed */
#define E_NET_CHRNG           -144      /* Channel number out of range */
#define E_NET_L2NSYNC         -145      /* Level 2 not synchronized */
#define E_NET_L3HLT           -146      /* Level 3 halted */
#define E_NET_L3RST           -147      /* Level 3 reset */
#define E_NET_LNRNG           -148      /* Link number out of range */
#define E_NET_UNATCH          -149      /* Protocol driver not attached */
#define E_NET_NOCSI           -150      /* No CSI structure available */
#define E_NET_L2HLT           -151      /* Level 2 halted */
#define E_NET_BADE            -152      /* Invalid exchange */
#define E_NET_BADR            -153      /* Invalid request descriptor */
#define E_NET_XFULL           -154      /* Exchange full */
#define E_NET_NOANO           -155      /* No anode */
#define E_NET_BADRQC          -156      /* Invalid request code */
#define E_NET_BADSLT          -157      /* Invalid slot */
#define E_NET_DEADLOCK        -158      /* File locking deadlock error */
#define E_NET_BFONT           -159      /* Bad font file format */
#define E_NET_NOSTR           -160      /* Device not a stream */
#define E_NET_NODATA          -161      /* No data available */
#define E_NET_TIME            -162      /* Timer expired */
#define E_NET_NOSR            -163      /* Out of streams resources */
#define E_NET_NONET           -164      /* Machine is not on the network */
#define E_NET_NOPKG           -165      /* Package not installed */
#define E_NET_REMOTE          -166      /* Object is remote */
#define E_NET_NOLINK          -167      /* Link has been severed */
#define E_NET_ADV             -168      /* Advertise error */
#define E_NET_SRMNT           -169      /* Srmount error */
#define E_NET_COMM            -170      /* Communication error on send */
#define E_NET_PROTO           -171      /* Protocol error */
#define E_NET_MULTIHOP        -172      /* Multihop attempted */
#define E_NET_DOTDOT          -173      /* RFS specific error */
#define E_NET_BADMSG          -174      /* Not a data message */
#define E_NET_OVERFLOW        -175      /* Value too large for defined data type */
#define E_NET_NOTUNIQ         -176      /* Name not unique on network */
#define E_NET_BADFD           -177      /* File descriptor in bad state */
#define E_NET_REMCHG          -178      /* Remote address changed */

/* ELIBACC, ELIBBAD, ELIBSCN, ELIBMAX are not in the SVR4 ABI.  It's
   just that the error numbers that they use in native linux collide
   with error numbers that are defined in the SVR4 ABI. */

#define E_NET_LIBACC          -179      /* Can not access a needed shared library */
#define E_NET_LIBBAD          -180      /* Accessing a corrupted shared library */
#define E_NET_LIBSCN          -181      /* .lib section in a.out corrupted */
#define E_NET_LIBMAX          -182      /* Attempting to link in too many shared libraries */
#define E_NET_RESTART         -185      /* Interrupted system call should be restarted */
#define E_NET_STRPIPE         -186      /* Streams pipe error */
#define E_NET_USERS           -187      /* Too many users */

/* Error numbers 89-94 are used by the SVR4 ABI, so these errors are
   relocated in SVR4 ABI Linux to 132-137. */

#define E_NET_DESTADDRREQ     -189      /* Destination address required */
#define E_NET_MSGSIZE         -190      /* Message too long */
#define E_NET_PROTOTYPE       -191      /* Protocol wrong type for socket */
#define E_NET_NOPROTOOPT      -192      /* Protocol not available */
#define E_NET_PROTONOSUPPORT  -193      /* Protocol not supported */
#define E_NET_SOCKTNOSUPPORT  -194      /* Socket type not supported */

#define E_NET_LIBEXEC         -183      /* Cannot exec a shared library directly */
#define E_NET_ILSEQ           -184      /* Illegal byte sequence */
#define E_NET_NOTSOCK         -188      /* Socket operation on non-socket */

#define E_NET_OPNOTSUPP       -195      /* Operation not supported on transport endpoint */
#define E_NET_PFNOSUPPORT     -196      /* Protocol family not supported */
#define E_NET_AFNOSUPPORT     -197      /* Address family not supported by protocol */
#define E_NET_ADDRINUSE       -198      /* Address already in use */
#define E_NET_ADDRNOTAVAIL    -199      /* Cannot assign requested address */
#define E_NET_NETDOWN         -200     /* Network is down */
#define E_NET_NETUNREACH     -201     /* Network is unreachable */
#define E_NET_NETRESET       -202     /* Network dropped connection because of reset */
#define E_NET_CONNABORTED    -203     /* Software caused connection abort */
#define E_NET_CONNRESET      -204     /* Connection reset by peer */
#define E_NET_NOBUFS         -205     /* No buffer space available */
#define E_NET_ISCONN         -206     /* Transport endpoint is already connected */
#define E_NET_NOTCONN        -207     /* Transport endpoint is not connected */
#define E_NET_SHUTDOWN       -208     /* Cannot send after transport endpoint shutdown */
#define E_NET_TOOMANYREFS    -209     /* Too many references: cannot splice */
#define E_NET_TIMEDOUT       -210     /* Connection timed out */
#define E_NET_CONNREFUSED    -211     /* Connection refused */
#define E_NET_HOSTDOWN       -212     /* Host is down */
#define E_NET_HOSTUNREACH    -213     /* No route to host */
#define E_NET_ALREADY        -214     /* Operation already in progress */
#define E_NET_INPROGRESS     -215     /* Operation now in progress */
#define E_NET_STALE          -216     /* Stale NFS file handle */
#define E_NET_UCLEAN         -217     /* Structure needs cleaning */
#define E_NET_NOTNAM         -218     /* Not a XENIX named type file */
#define E_NET_NAVAIL         -219     /* No XENIX semaphores available */
#define E_NET_ISNAM          -220     /* Is a named type file */
#define E_NET_REMOTEIO       -221     /* Remote I/O error */
#define E_NET_DQUOT          -222     /* Quota exceeded */

/* Should never be seen by user programs */
#define E_NET_RESTARTSYS     -512
#define E_NET_RESTARTNOINTR  -513
#define E_NET_RESTARTNOHAND  -514     /* restart if no handler.. */
#define E_NET_NOIOCTLCMD     -515     /* No ioctl command */

/* And if you get this, then the code hasn't been written yet */
#define E_NET_UNIMPLEMENTED  -600

/* Standard well-known ports.  */
#define IPPORT_ECHO 7               /* Echo service.  */
#define IPPORT_DISCARD 9            /* Discard transmissions service.  */
#define IPPORT_SYSTAT 11            /* System status service.  */
#define IPPORT_DAYTIME 13   /* Time of day service.  */
#define IPPORT_NETSTAT 15   /* Network status service.  */
#define IPPORT_FTP 21               /* File Transfer Protocol.  */
#define IPPORT_TELNET 23            /* Telnet protocol.  */
#define IPPORT_SMTP 25              /* Simple Mail Transfer Protocol.  */
#define IPPORT_TIMESERVER 37        /* Timeserver service.  */
#define IPPORT_NAMESERVER 42        /* Domain Name Service.  */
#define IPPORT_WHOIS 43             /* Internet Whois service.  */
#define IPPORT_MTP 57

#define IPPORT_TFTP 69              /* Trivial File Transfer Protocol.  */
#define IPPORT_RJE 77
#define IPPORT_FINGER 79            /* Finger service.  */
#define IPPORT_TTYLINK 87
#define IPPORT_SUPDUP 95            /* SUPDUP protocol.  */

#define IPPORT_EXECSERVER 512       /* execd service.  */
#define IPPORT_LOGINSERVER 513      /* rlogind service.  */
#define IPPORT_CMDSERVER 514
#define IPPORT_EFSSERVER 520

 /* UDP ports.  */
#define IPPORT_BIFFUDP 512
#define IPPORT_WHOSERVER 513
#define IPPORT_ROUTESERVER 520

 /* Ports less than this value are reserved for privileged processes.  */
#define IPPORT_RESERVED 1024

 /* Ports greater this value are reserved for (non-privileged) servers.  */
#define IPPORT_USERRESERVED 5000

/* Standard well-defined IP protocols.  */
#define IPPROTO_IP 0            /* Dummy protocol for TCP               */
#define IPPROTO_ICMP 1          /* Internet Control Message Protocol    */
#define IPPROTO_IGMP 2          /* Internet Group Management Protocol   */
#define IPPROTO_IPIP 4          /* IPIP tunnels (older KA9Q tunnels use 94) */
#define IPPROTO_TCP 6           /* Transmission Control Protocol        */
#define IPPROTO_EGP 8           /* Exterior Gateway Protocol            */
#define IPPROTO_PUP 12          /* PUP protocol                         */
#define IPPROTO_UDP 17          /* User Datagram Protocol               */
#define IPPROTO_IDP 22          /* XNS IDP protocol                     */

#define IPPROTO_RAW 255         /* Raw IP packets                       */

/*
 * Definitions of the bits in an Internet address integer.
 * On subnets, host and network parts are found according
 * to the subnet mask, not these masks.
 */
#define IN_CLASSA(a)            ((((long int) (a)) & 0x80000000) == 0)
#define IN_CLASSA_NET           0xff000000
#define IN_CLASSA_NSHIFT        24
#define IN_CLASSA_HOST          (0xffffffff & ~IN_CLASSA_NET)
#define IN_CLASSA_MAX           128

#define IN_CLASSB(a)            ((((long int) (a)) & 0xc0000000) == 0x80000000)
#define IN_CLASSB_NET           0xffff0000
#define IN_CLASSB_NSHIFT        16
#define IN_CLASSB_HOST          (0xffffffff & ~IN_CLASSB_NET)
#define IN_CLASSB_MAX           65536

#define IN_CLASSC(a)            ((((long int) (a)) & 0xe0000000) == 0xc0000000)
#define IN_CLASSC_NET           0xffffff00
#define IN_CLASSC_NSHIFT        8
#define IN_CLASSC_HOST          (0xffffffff & ~IN_CLASSC_NET)

#define IN_CLASSD(a)            ((((long int) (a)) & 0xf0000000) == 0xe0000000)
#define IN_MULTICAST(a)         IN_CLASSD(a)
#define IN_MULTICAST_NET        0xF0000000

#define IN_EXPERIMENTAL(a)      ((((long int) (a)) & 0xe0000000) == 0xe0000000)
#define IN_BADCLASS(a)          ((((long int) (a)) & 0xf0000000) == 0xf0000000)

/* Address to accept any incoming messages. */
#define INADDR_ANY              ((unsigned long int) 0x00000000)

/* Address to send to all hosts. */
#define INADDR_BROADCAST        ((unsigned long int) 0xffffffff)

/* Address indicating an error return. */
#define INADDR_NONE             ((unsigned long int) 0xffffffff)

/* Network number for local host loopback. */
#define IN_LOOPBACKNET          127

/* Address to loopback in software to local host.  */
#define INADDR_LOOPBACK         0x7f000001      /* 127.0.0.1   */
#define IN_LOOPBACK(a)          ((((long int) (a)) & 0xff000000) == 0x7f000000)

/* Defines for Multicast INADDR */
#define INADDR_UNSPEC_GROUP     0xe0000000      /* 224.0.0.0   */
#define INADDR_ALLHOSTS_GROUP   0xe0000001      /* 224.0.0.1   */
#define INADDR_MAX_LOCAL_GROUP  0xe00000ff      /* 224.0.0.255 */

#endif /* SYS_SOCKET_H */

