/*******************************************************************************
***
*** Filename         : socket.h
*** Purpose          : PsiStack Sockets Library definitions and prototypes
*** Author           : Linux authors, modified heavily for PsiStack by 
***                    Matt J. Gumbley
*** Created          : 05/05/97
*** Last updated     : 20/11/97
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#ifndef SYS_SOCKET_H
#define SYS_SOCKET_H

/* Used by select() */
#ifndef howmany
#define howmany(x, y)  (((x)+((y)-1))/(y))
#endif

#ifndef roundup
#define roundup(x, y)  ((((x)+((y)-1))/(y))*(y))
#endif

typedef unsigned long fd_mask;

struct sockaddr {
  unsigned char  sa_len;                                      /* Total length */
  unsigned short sa_family;                        /* Address family, AF_xxx  */
  char           sa_data[14];                /* 14 bytes of protocol address  */
};

struct sockaddr_in {
  unsigned char  sin_len;                                     /* Total length */
  unsigned short sin_family;                       /* Address family, AF_xxx  */
  unsigned short sin_port;                            /* Protocol port number */
  unsigned long  sin_addr;                                      /* IP address */
  char           sin_zero[8];
};

struct linger {
  int            l_onoff;                                    /* Linger active */
  int            l_linger;                          /* How long to linger for */
};


/* Socket types. */
#define SOCK_STREAM  1            /* Stream (TCP, connection-oriented) socket */
#define SOCK_DGRAM   2               /* Datagram (UDP, connectionless) socket */
#define SOCK_RAW     3                                       /* Raw IP socket */
#define SOCK_RAWINTERFACE 4    /* Raw data over an interface, for the dialler */

/* Supported address families. */
#define AF_INET      2                                /* Internet IP Protocol */
#define AF_MAX       1                                           /* For now.. */

/* Protocol families, same as address families. */
#define PF_INET      AF_INET
#define PF_MAX       AF_MAX

/* Maximum queue length specifiable by listen.  */
#define SOMAXCONN    5

/* Flags we can use with send and recv. */
#define MSG_OOB        1
#define MSG_PEEK       2
#define MSG_DONTROUTE  4
#define MSG_PROXY      16                    /* Supply or ask second address. */

/* Setsockoptions(2) level. Thanks to BSD these must match IPPROTO_xxx */
#define SOL_IP       0
#define SOL_TCP      6
#define SOL_UDP      17

/* IP options */
#define IP_TOS    1
#define  IPTOS_LOWDELAY     0x10
#define  IPTOS_THROUGHPUT   0x08
#define  IPTOS_RELIABILITY  0x04
#define  IPTOS_MINCOST      0x02
#define IP_TTL    2
#define IP_HDRINCL  3
#define IP_OPTIONS  4

#define IP_MULTICAST_IF     32
#define IP_MULTICAST_TTL    33
#define IP_MULTICAST_LOOP   34
#define IP_ADD_MEMBERSHIP   35
#define IP_DROP_MEMBERSHIP  36

/* These need to appear somewhere around here */
#define IP_DEFAULT_MULTICAST_TTL        1
#define IP_DEFAULT_MULTICAST_LOOP       1
#define IP_MAX_MEMBERSHIPS              20
 
/* TCP options - this way around because someone left a set in the C 
   library includes */
#define TCP_NODELAY  1
#define TCP_MAXSEG   2

/* The various priorities. */
#define SOPRI_INTERACTIVE  0
#define SOPRI_NORMAL       1
#define SOPRI_BACKGROUND   2

/* Standard errno codes, converted from Linux' errno.h */

#define EPERM            1      /* Operation not permitted */
#define ENOENT           2      /* No such file or directory */
#define ESRCH            3      /* No such process */
#define EINTR            4      /* Interrupted system call */
#define EIO              5      /* I/O error */
#define ENXIO            6      /* No such device or address */
#define E2BIG            7      /* Arg list too long */
#define ENOEXEC          8      /* Exec format error */
#define EBADF            9      /* Bad file number */
#define ECHILD          10      /* No child processes */
#define EAGAIN          11      /* Try again */
#define ENOMEM          12      /* Out of memory */
#define EACCES          13      /* Permission denied */
#define EFAULT          14      /* Bad address */
#define ENOTBLK         15      /* Block device required */
#define EBUSY           16      /* Device or resource busy */
#define EEXIST          17      /* File exists */
#define EXDEV           18      /* Cross-device link */
#define ENODEV          19      /* No such device */
#define ENOTDIR         20      /* Not a directory */
#define EISDIR          21      /* Is a directory */
#define EINVAL          22      /* Invalid argument */
#define ENFILE          23      /* File table overflow */
#define EMFILE          24      /* Too many open files */
#define ENOTTY          25      /* Not a typewriter */
#define ETXTBSY         26      /* Text file busy */
#define EFBIG           27      /* File too large */
#define ENOSPC          28      /* No space left on device */
#define ESPIPE          29      /* Illegal seek */
#define EROFS           30      /* Read-only file system */
#define EMLINK          31      /* Too many links */
#define EPIPE           32      /* Broken pipe */
#define EDOM            33      /* Math argument out of domain of func */
#define ERANGE          34      /* Math result not representable */
#define EDEADLK         35      /* Resource deadlock would occur */
#define ENAMETOOLONG    36      /* File name too long */
#define ENOLCK          37      /* No record locks available */
#define ENOSYS          38      /* Function not implemented */
#define ENOTEMPTY       39      /* Directory not empty */
#define ELOOP           40      /* Too many symbolic links encountered */
#define EWOULDBLOCK     EAGAIN  /* Operation would block */
#define ENOMSG          42      /* No message of desired type */
#define EIDRM           43      /* Identifier removed */
#define ECHRNG          44      /* Channel number out of range */
#define EL2NSYNC        45      /* Level 2 not synchronized */
#define EL3HLT          46      /* Level 3 halted */
#define EL3RST          47      /* Level 3 reset */
#define ELNRNG          48      /* Link number out of range */
#define EUNATCH         49      /* Protocol driver not attached */
#define ENOCSI          50      /* No CSI structure available */
#define EL2HLT          51      /* Level 2 halted */
#define EBADE           52      /* Invalid exchange */
#define EBADR           53      /* Invalid request descriptor */
#define EXFULL          54      /* Exchange full */
#define ENOANO          55      /* No anode */
#define EBADRQC         56      /* Invalid request code */
#define EBADSLT         57      /* Invalid slot */
#define EDEADLOCK       58      /* File locking deadlock error */
#define EBFONT          59      /* Bad font file format */
#define ENOSTR          60      /* Device not a stream */
#define ENODATA         61      /* No data available */
#define ETIME           62      /* Timer expired */
#define ENOSR           63      /* Out of streams resources */
#define ENONET          64      /* Machine is not on the network */
#define ENOPKG          65      /* Package not installed */
#define EREMOTE         66      /* Object is remote */
#define ENOLINK         67      /* Link has been severed */
#define EADV            68      /* Advertise error */
#define ESRMNT          69      /* Srmount error */
#define ECOMM           70      /* Communication error on send */
#define EPROTO          71      /* Protocol error */
#define EMULTIHOP       72      /* Multihop attempted */
#define EDOTDOT         73      /* RFS specific error */
#define EBADMSG         74      /* Not a data message */
#define EOVERFLOW       75      /* Value too large for defined data type */
#define ENOTUNIQ        76      /* Name not unique on network */
#define EBADFD          77      /* File descriptor in bad state */
#define EREMCHG         78      /* Remote address changed */

/* ELIBACC, ELIBBAD, ELIBSCN, ELIBMAX are not in the SVR4 ABI.  It's
   just that the error numbers that they use in native linux collide
   with error numbers that are defined in the SVR4 ABI. */

#define ELIBACC         79      /* Can not access a needed shared library */
#define ELIBBAD         80      /* Accessing a corrupted shared library */
#define ELIBSCN         81      /* .lib section in a.out corrupted */
#define ELIBMAX         82      /* Attempting to link in too many shared libraries */
#define ERESTART        85      /* Interrupted system call should be restarted */
#define ESTRPIPE        86      /* Streams pipe error */
#define EUSERS          87      /* Too many users */

/* Error numbers 89-94 are used by the SVR4 ABI, so these errors are
   relocated in SVR4 ABI Linux to 132-137. */

#define EDESTADDRREQ    89      /* Destination address required */
#define EMSGSIZE        90      /* Message too long */
#define EPROTOTYPE      91      /* Protocol wrong type for socket */
#define ENOPROTOOPT     92      /* Protocol not available */
#define EPROTONOSUPPORT 93      /* Protocol not supported */
#define ESOCKTNOSUPPORT 94      /* Socket type not supported */

#define ELIBEXEC        83      /* Cannot exec a shared library directly */
#define EILSEQ          84      /* Illegal byte sequence */
#define ENOTSOCK        88      /* Socket operation on non-socket */

#define EOPNOTSUPP      95      /* Operation not supported on transport endpoint */
#define EPFNOSUPPORT    96      /* Protocol family not supported */
#define EAFNOSUPPORT    97      /* Address family not supported by protocol */
#define EADDRINUSE      98      /* Address already in use */
#define EADDRNOTAVAIL   99      /* Cannot assign requested address */
#define ENETDOWN        100     /* Network is down */
#define ENETUNREACH     101     /* Network is unreachable */
#define ENETRESET       102     /* Network dropped connection because of reset */
#define ECONNABORTED    103     /* Software caused connection abort */
#define ECONNRESET      104     /* Connection reset by peer */
#define ENOBUFS         105     /* No buffer space available */
#define EISCONN         106     /* Transport endpoint is already connected */
#define ENOTCONN        107     /* Transport endpoint is not connected */
#define ESHUTDOWN       108     /* Cannot send after transport endpoint shutdown */
#define ETOOMANYREFS    109     /* Too many references: cannot splice */
#define ETIMEDOUT       110     /* Connection timed out */
#define ECONNREFUSED    111     /* Connection refused */
#define EHOSTDOWN       112     /* Host is down */
#define EHOSTUNREACH    113     /* No route to host */
#define EALREADY        114     /* Operation already in progress */
#define EINPROGRESS     115     /* Operation now in progress */
#define ESTALE          116     /* Stale NFS file handle */
#define EUCLEAN         117     /* Structure needs cleaning */
#define ENOTNAM         118     /* Not a XENIX named type file */
#define ENAVAIL         119     /* No XENIX semaphores available */
#define EISNAM          120     /* Is a named type file */
#define EREMOTEIO       121     /* Remote I/O error */
#define EDQUOT          122     /* Quota exceeded */

/* Should never be seen by user programs */
#define ERESTARTSYS     512
#define ERESTARTNOINTR  513
#define ERESTARTNOHAND  514     /* restart if no handler.. */
#define ENOIOCTLCMD     515     /* No ioctl command */


/* Function protoypes */

/* Create a new socket of type TYPE in domain DOMAIN, using
   protocol PROTOCOL.  If PROTOCOL is zero, one is chosen
   automatically.  Returns a file descriptor for the new socket,
   or -1 for errors.  */
GLREF_C int socket(int family, int type, int protocol);

/* Create two new sockets, of type TYPE in domain DOMAIN and using
   protocol PROTOCOL, which are connected to each other, and put file
   descriptors for them in FDS[0] and FDS[1].  If PROTOCOL is zero,
   one will be chosen automatically.  Returns 0 on success, -1
   for errors.  */
GLREF_C int socketpair(int family, int type, int protocol, int sockvec[2]);

/* Give the socket FD the local address ADDR(which is LEN bytes
   long).  */
GLREF_C int bind(int sockfd, struct sockaddr *my_addr, int addrlen);

/* Open a connection on socket FD to peer at ADDR(which LEN bytes
   long). For connectionless socket types, just set the default
   address to send to and the only address from which to accept
   transmissions.  Return 0 on success, -1 for errors.  */
GLREF_C int connect(int sockfd, struct sockaddr *serv_addr, int addrlen);

/* Prepare to accept connections on socket FD.
   N connection requests will be queued before further requests are
   refused. Returns 0 on success, -1 for errors.  */
GLREF_C int listen(int sockfd, int n);

/* Await a connection on socket FD.
   When a connection arrives, open a new socket to communicate with it,
   set *ADDR(which is *ADDR_LEN bytes long) to the address of the
   connecting peer and *ADDR_LEN to the address's actual length, and
   return the new socket's descriptor, or -1 for errors.  */
GLREF_C int accept(int sockfd, struct sockaddr *peer, int *paddrlen);

/* Put the current value for socket FD's option OPTNAME at protocol
   level LEVEL into OPTVAL(which is *OPTLEN bytes long), and set
   *OPTLEN to the value's actual length. Returns 0 on success, -1 for
   errors.  */
GLREF_C int getsockopt(int sockfd, int level, int optname,
                       void *optval, int *optlen);

/* Set socket FD's option OPTNAME at protocol level LEVEL
   to *OPTVAL(which is OPTLEN bytes long).
   Returns 0 on success, -1 for errors.  */
GLREF_C int setsockopt(int sockfd, int level, int optname,
                       void *optval, int optlen);

/* Put the local address of FD into *ADDR and its length in *LEN.  */
GLREF_C int getsockname(int sockfd, struct sockaddr *addr, int *paddrlen);

/* Put the address of the peer connected to socket FD into *ADDR
  (which is *LEN bytes long), and its actual length into *LEN.  */
GLREF_C int getpeername(int sockfd, struct sockaddr *peer, int *paddrlen);

/* Send N bytes of BUF to socket FD.  Returns the number sent or -1. */
GLREF_C int send(int sockfd, void *buff, int len, unsigned int flags);

/* Read N bytes into BUF from socket FD.
   Returns the number read or -1 for errors.  */
GLREF_C int recv(int sockfd, void *buff, int len, unsigned int flags);

/* Send N bytes of BUF on socket FD to peer at address ADDR(which is
   ADDR_LEN bytes long).  Returns the number sent, or -1 for errors. */
GLREF_C int sendto(int sockfd, void *buff, int len,
                   unsigned int flags, struct sockaddr *to, int tolen);

/* Read N bytes into BUF through socket FD.
   If ADDR is not NULL, fill in *ADDR_LEN bytes of it with tha address
   of the sender, and store the actual size of the address in
   *ADDR_LEN. Returns the number of bytes read or -1 for errors. */
GLREF_C int recvfrom(int sockfd, void *buff, int len,
                     unsigned int flags, struct sockaddr *from, int *fromlen);

/**************
 Send a message described MESSAGE on socket FD.
   Returns the number of bytes sent, or -1 for errors.  
GLREF_C int sendmsg(int fd, struct msghdr *message, unsigned int flags);

   Receive a message as described by MESSAGE from socket FD.
   Returns the number of bytes read or -1 for errors. 
GLREF_C int recvmsg(int fd, struct msghdr *message, unsigned int flags);
***************/
 
/* Shut down all or part of the connection open on socket FD.
   HOW determines what to shut down:
  0 = No more receptions;
  1 = No more transmissions;
  2 = No more receptions or transmissions.
   Returns 0 on success, -1 for errors.  */
GLREF_C int shutdown(int sockfd, int how);

#endif /* SYS_SOCKET_H */
