/*******************************************************************************
***
*** Filename         : attn.c
*** Purpose          : Main Loop attention grabbing mechanism
*** Author           : Matt J. Gumbley
*** Created          : 15/07/97
*** Last updated     : 02/12/97
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#include "stack.h"

GLDEF_D UWORD attn;

void attn_diag(char *hdr, UWORD thisattn)
{
char buf[256];
  p_scpy(buf, "attn: ");
  p_scat(buf, hdr);

  if (thisattn & ATTN_IP) {
    p_scat(buf,"IP ");
  }
  if (thisattn & ATTN_NIF) {
    p_scat(buf,"NIF ");
  }
  if (thisattn & ATTN_SCRIPT) {
    p_scat(buf,"SCRIPT ");
  }
  if (thisattn & ATTN_PING) {
    p_scat(buf,"PING ");
  }
  if (thisattn & ATTN_TRACE) {
    p_scat(buf,"TRACE ");
  }
  if (thisattn & ATTN_TCPOUTPUT) {
    p_scat(buf,"TCPOUTPUT ");
  }
  if (thisattn & ATTN_FSM) {
    p_scat(buf,"PPP FSM ");
  }
  if (thisattn & ATTN_LCP) {
    p_scat(buf,"PPP LCP ");
  }
  console_puts(buf);
}

/*******************************************************************************
***
*** Function         : attn_process()
*** Preconditions    : There may be events awaiting attention
*** Postconditions   : They are serviced, and the attention flag is reset.
***
*******************************************************************************/

void attn_process()
{
register UWORD thisattn = attn;
  attn = 0;

  if (thisattn & ATTN_IP) {
    ip_process();
  }
  if (thisattn & ATTN_NIF) {
    nif_process();
  }
  if (thisattn & ATTN_SCRIPT) {
    script_process();
  }
  if (thisattn & ATTN_PING) {
    ping_process();
  }
  if (thisattn & ATTN_TRACE) {
    trace_process();
  }
  if (thisattn & ATTN_TCPOUTPUT) {
    tcpo_process();
  }
  if (thisattn & ATTN_FSM) {
    fsm_process();
  }
  if (thisattn & ATTN_LCP) {
    lcp_process();
  }

  /* attn will either be zero (no more attn events have been set by the above 
     _process handlers, or non-zero (they want something doing). If non-zero,
     signal the main loop, so more work will be done next time in here */

  if (attn != 0) {
    p_iosignal();
  }

  /* We couldn't do a loop while (attn != 0)... since this could lead
     to process starvation */
}


/*******************************************************************************
***
*** Function         : attn_signal(newattn)
*** Preconditions    : newattn is some combination of signals
*** Postconditions   : The signals are noted, and if attn was 0 before this,
***                    the asynchronous event loop is woken up.
***
*******************************************************************************/

GLDEF_C VOID attn_signal(UWORD newattn)
{
  if (attn == 0)
    p_iosignal();
  attn |= newattn;
}

