/*******************************************************************************
***
*** Filename         : config.c
*** Purpose          : Simple configuration storage routines for PsiStack
*** Author           : Matt J. Gumbley, Patrick Robbe
*** Created          : 07/05/97
*** Last updated     : 24/08/98
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#include "stack.h" /* The stack kernel and system includes */

LOCAL_D TEXT config_fullname[P_FNAMESIZE] = "LOC::M:\\APP\\PSISTACK\\STACK.CFG";


VOID config_save(VOID)
{
VOID *fd = NULL;
INT j,errcode;

  errcode=p_open(&fd,config_fullname,P_FREPLACE|P_FSTREAM|P_FUPDATE);
  if (errcode != 0) {
    console_printf("Couldn't create \"%s\"\r\n", (UINT)config_fullname+21);
    return;
  }
  p_write(fd,(VOID *)&version,sizeof(int));
  p_write(fd,(VOID *)&releaselevel,sizeof(char));

  p_write(fd,(VOID *)asy.asy_serDeviceName,10);
  p_write(fd,(VOID *)&asy.asy_speed,sizeof(int));
  p_write(fd,(VOID *)&asy.asy_parity,sizeof(int));
  p_write(fd,(VOID *)&asy.asy_databits,sizeof(int));
  p_write(fd,(VOID *)&asy.asy_stopbits,sizeof(int));
  p_write(fd,(VOID *)&asy.asy_handshake,sizeof(byte));
  p_write(fd,(VOID *)&asy.asy_devicedetect,sizeof(byte));
  p_write(fd,(VOID *)&asy.asy_carrierdetect,sizeof(byte));

  p_write(fd,(VOID *)&netif->ni_dlp,sizeof(char));
  p_write(fd,(VOID *)netif->ni_ip,IP_ALEN);
  p_write(fd,(VOID *)netif->ni_mask,IP_ALEN);
  for (j=0; j<NI_GATEWAYS; j++)
    p_write(fd,(VOID *)netif->ni_gateways[j],IP_ALEN);
  for (j=0; j<NI_DNSS; j++)
    p_write(fd,(VOID *)netif->ni_dnss[j],IP_ALEN);
  p_write(fd,(VOID *)&netif->ni_mtu,sizeof(int));

  p_write(fd,(VOID *)&sc.si_scriptused,sizeof(byte));
  p_write(fd,(VOID *)&sc.si_debuglevel,sizeof(byte));
  p_write(fd,(VOID *)sc.si_scriptname,P_FNAMESIZE+2);

  p_write(fd,(VOID *)ispc.isp_name,ISPSTRINGMAX);
  p_write(fd,(VOID *)ispc.isp_phone1,ISPSTRINGMAX);
  p_write(fd,(VOID *)ispc.isp_phone2,ISPSTRINGMAX);
  p_write(fd,(VOID *)ispc.isp_phone3,ISPSTRINGMAX);
  p_write(fd,(VOID *)ispc.isp_username,ISPSTRINGMAX);
  p_write(fd,(VOID *)ispc.isp_password,ISPSTRINGMAX);

  p_write(fd,(VOID *)&ping_info,sizeof(struct ping_info));
  p_write(fd,(VOID *)&udp_config,sizeof(struct udp_config));
  p_write(fd,(VOID *)&tcp_config,sizeof(struct tcp_config));
  p_write(fd,(VOID *)&ppp_config,sizeof(struct ppp_config));

  p_write(fd,(VOID *)&flags.log_flag,sizeof(UWORD));
  p_close(fd);
}


VOID config_load(VOID)
{
VOID *fd = NULL;
int j, fileversion;
char filereleaselevel;

  if (p_open(&fd,config_fullname,P_FOPEN|P_FSTREAM|P_FSHARE) != 0) {
    console_printf("Creating \"%s\"\r\n", (UINT)config_fullname+21);
    config_save();
    return;
  }
  p_read(fd,(VOID *)&fileversion,sizeof(int));
  p_read(fd,(VOID *)&filereleaselevel,sizeof(char));
  if (fileversion != version || filereleaselevel != releaselevel) {
    console_printf("Config file created by v%d.%02d%c\r\ncannot be used, so defaults\r\nwill be used.\r\n", fileversion / 100, fileversion % 100, filereleaselevel);
    p_close(fd);
    config_save();
    return;
  }

  p_read(fd,(VOID *)asy.asy_serDeviceName,10);
  p_read(fd,(VOID *)&asy.asy_speed,sizeof(int));
  p_read(fd,(VOID *)&asy.asy_parity,sizeof(int));
  p_read(fd,(VOID *)&asy.asy_databits,sizeof(int));
  p_read(fd,(VOID *)&asy.asy_stopbits,sizeof(int));
  p_read(fd,(VOID *)&asy.asy_handshake,sizeof(byte));
  p_read(fd,(VOID *)&asy.asy_devicedetect,sizeof(byte));
  p_read(fd,(VOID *)&asy.asy_carrierdetect,sizeof(byte));

  p_read(fd,(VOID *)&netif->ni_dlp,sizeof(char));
  p_read(fd,(VOID *)netif->ni_ip,IP_ALEN);
  p_read(fd,(VOID *)netif->ni_mask,IP_ALEN);
  for (j=0; j<NI_GATEWAYS; j++)
    p_read(fd,(VOID *)netif->ni_gateways[j],IP_ALEN);
  for (j=0; j<NI_DNSS; j++)
    p_read(fd,(VOID *)netif->ni_dnss[j],IP_ALEN);
  p_read(fd,(VOID *)&netif->ni_mtu,sizeof(int));

  p_read(fd,(VOID *)&sc.si_scriptused,sizeof(byte));
  p_read(fd,(VOID *)&sc.si_debuglevel,sizeof(byte));
  p_read(fd,(VOID *)sc.si_scriptname,P_FNAMESIZE+2);

  p_read(fd,(VOID *)ispc.isp_name,ISPSTRINGMAX);
  p_read(fd,(VOID *)ispc.isp_phone1,ISPSTRINGMAX);
  p_read(fd,(VOID *)ispc.isp_phone2,ISPSTRINGMAX);
  p_read(fd,(VOID *)ispc.isp_phone3,ISPSTRINGMAX);
  p_read(fd,(VOID *)ispc.isp_username,ISPSTRINGMAX);
  p_read(fd,(VOID *)ispc.isp_password,ISPSTRINGMAX);

  p_read(fd,(VOID *)&ping_info,sizeof(struct ping_info));
  /* certain parts of the ping info should always be initialised thus: */
  ping_info.p_numpings=0;
  p_read(fd,(VOID *)&udp_config,sizeof(struct udp_config));
  p_read(fd,(VOID *)&tcp_config,sizeof(struct tcp_config));
  p_read(fd,(VOID *)&ppp_config,sizeof(struct ppp_config));

  p_read(fd,(VOID *)&flags.log_flag,sizeof(UWORD));
  p_close(fd);
}

