/*******************************************************************************
***
*** Filename         : ip.h
*** Purpose          : Definitions for the IP layer in PsiStack
*** Author           : Matt J. Gumbley
*** Created          : 19/01/97
*** Last updated     : 01/03/98
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#ifndef IP_H
#define IP_H

extern IPaddr ip_maskall;                                /* = 255.255.255.255 */
extern IPaddr ip_anyaddr;                                        /* = 0.0.0.0 */
extern IPaddr ip_loopback;                                     /* = 127.0.0.1 */
extern IPaddr ip_loopnet;                                      /* = 127.0.0.0 */
extern IPaddr ip_loopmask;                                     /* = 255.0.0.0 */
extern IPaddr ip_loopbrc;                                /* = 127.255.255.255 */

extern word16 ip_ident;          /* Current IP datagram identifier for output */

#define IP_INQSZ 30                                    /* IP Input queue size */

/* Information to support IP datagram reassembly, which is actually done by
   local_process, using routines in ip.c */

#define IPFQ_SIZE 10                /* Maximum number of fragmentation queues */
#define IPFQ_MAXNF  10            /* Maximum number of fragments per datagram */
#define IPFQ_TTL   60                    /* Fragment time-to-live, in seconds */

/* Settings for ipfq_state in the fragment queue structure */
#define IPFQS_VALID 1                                   /* Contents are valid */
#define IPFQS_BOGUS 2                       /* Drop any fragments which match */
#define IPFQS_FREE  3                   /* This queue is free to be allocated */

/* Structure associated with chained buffers used when reassembling datagrams
   There will be an array of these reassembly structures in IP representing
   datagrams being reassembled.
   See Comer v1, p95...99; v2, p113...124
*/
struct ip_fq {
  char   ipfq_state;                            /* IPFQS_VALID,BOGUS, or FREE */
  IPaddr ipfq_src;      /* Source address of datagram used for identification */
  word16 ipfq_id; /* ID for unique datagrams - only needed for first fragment */
  int    ipfq_ttl;                    /* Countdown to disposal for this queue */
  struct buf *ipfq_q;             /* Buffer chain of fragments for reassembly */
};

extern struct ip_fq ip_fq[];     /* IP fragment queue table, declared in ip.c */
extern VOID ip_options(struct buf *);
extern struct buf *ip_frag_reass(struct buf *);
extern int  which_nif(IPaddr, int);
extern VOID ip_process(VOID);
extern VOID ip_dump(struct buf *);
extern VOID ip_net2h(struct ip *);
extern VOID ip_send(IPaddr, struct buf *, int);
extern VOID net_num(IPaddr, IPaddr);
extern VOID net_mask(IPaddr, IPaddr);
extern VOID brc_num(IPaddr, IPaddr);
extern INT  net_match(IPaddr, IPaddr, IPaddr, int);
extern INT  isbrc(IPaddr);
extern VOID ip_stats(VOID);

#endif /* IP_H */
