/*******************************************************************************
***
*** Filename         : lcp.h
*** Purpose          : Definitions for the PPP LCP in PsiStack
*** Author           : Patrick Robbe
*** Created          : 10/10/97
*** Last updated     : 21/03/98
***
********************************************************************************
***
*** Modification Record: 
***
*******************************************************************************/

#ifndef LCP_H
#define LCP_H

/* The different LCP Options Type Field values */
#define Maximum_Receive_Unit                  1
#define Async_Control_Character_Map           2
#define Authentication_Protocol               3
#define Quality_Protocol                      4
#define Magic_Number                          5
#define Protocol_Field_Compression            6
#define Address_and_Control_Field_Compression 7

extern void lcp_process(void);
extern void ppp_send_configure_request(void);
extern void ppp_send_configure_ack(void);
extern void ppp_send_configure_nack(void);
extern void ppp_send_terminate_request(void);
extern void ppp_send_terminate_ack(void);
extern void ppp_send_code_reject(void);
extern void ppp_send_protocol_reject(void);
extern void ppp_send_echo_reply(void);
extern long lcp_new_magic_nb(void);

struct lcp_storage {
  int lcp_mru;                    /* Maximum-Receive-Unit (default 1500) */
  int lcp_auth_prot;              /* Authentication Protocol (PAP or CHAP) */
  int lcp_qual_prot;              /* Quality Monitoring Protocol (LQR) */
  unsigned long lcp_magic_number; /* Magic Number */
  byte lcp_pfc;                   /* Protocol Field Compression */
  byte lcp_acfc;                  /* Address and Control Field Compression */

  struct queue *lcp_inframes;     /* Incoming LCP frames from ppp_inhandler */
  struct queue *lcp_outframes;    /* Outgoing LCP frames to ppp_handler */

  struct buf *lcp_txbuildup;      /* Tx frame buildup buffer for LCP */
  struct buf *lcp_rxbuildup;      /* Rx frame buildup buffer for LCP */
  int lcp_rcnt;                   /* Next posision in frame being built up */

  byte lcp_cur_tx_id;             /* Identifier for last emitted LCP packets */
  byte lcp_cur_rx_id;             /* Identifier for last received LCP packet */
};

extern struct lcp_storage lcs;

#endif /* LCP_H */
