/*******************************************************************************
***
*** Filename         : slip.h
*** Purpose          : Definitions for the SLIP data-link layer in Psion-TCP/IP
*** Author           : Matt J. Gumbley
*** Created          : 20/01/97
*** Last updated     : 21/01/98
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#ifndef SLIP_H
#define SLIP_H

/* SLIP framing characters */
#define SLIP_END    0xc0                                         /* Frame end */
#define SLIP_ESC    0xdb                                      /* Frame escape */
#define SLIP_ESCEND 0xdc                              /* Transposed frame end */
#define SLIP_ESCESC 0xdd                           /* Transposed frame escape */

/* The Maximum Transfer Unit for SLIP networks */
#define SLIP_MTU 296 /* What Linux uses... although RFC 1055 says 1006? */

/* Storage requirements for SLIP */
struct slip_storage {
  byte *ss_txbuildup;                     /* Tx frame buildup buffer for SLIP */
  struct buf *ss_rxbuildup;               /* Rx frame buildup buffer for SLIP */
  int ss_rcnt;                       /* Next posision in frame being built up */
  int ss_escaped;                                     /* Are we in an escape? */
};

extern void slip_stats(void);
extern void slip_inprocess(void);
extern int  slip_outprocess(void);

#endif /* SLIP_H */

