/*******************************************************************************
***
*** Filename         : term.c
*** Purpose          : 'Terminal' mode for PsiStack
*** Author           : Matt J. Gumbley
*** Created          : 11/07/97
*** Last updated     : 14/03/98
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#include "stack.h"

/* GLREF_D VOID *winHandle; * in psistack.c */
/* FIXME: this is coupled to the setup program - it'll have to be rewritten 
          to use the console */

/*******************************************************************************
***
*** Function         : term_raw_handler()
*** Preconditions    : Serial data may be queued
*** Postconditions   : The data is output raw to the screen
***
*******************************************************************************/

void term_raw_handler(void)
{
  /* was p_write(winHandle, asy.asy_serBuffer, asy.asy_serLength); */
  msg_event(EVT_UI_TERMDATA, asy.asy_serBuffer, asy.asy_serLength);
}


/*******************************************************************************
***
*** Function         : term_expanded_handler()
*** Preconditions    : Serial data may be queued
*** Postconditions   : The data is output in expanded form to the screen
***
*******************************************************************************/

void term_expanded_handler(void)
{
byte expandbuf[ASY_SERBUFSIZE * 4]; /* Assume worst case, expand everything */
register byte *s, *d;
register int i;

/* was
static char hexbuf[]="<xx>";
register int chindex;
register byte *p;

  * This could be optimised somewhat.. *
  for (chindex=0,p=asy.asy_serBuffer;chindex<asy.asy_serLength;chindex++,p++) {
    if (!p_isprint(*p)) {
      hexbuf[1]=hexdigs[(*p&0xf0) >> 4];
      hexbuf[2]=hexdigs[*p&0x0f];
      p_write(winHandle, hexbuf, 4);
    }
    else
      p_putch(*p);
  }
*/

  for (i=0,s=asy.asy_serBuffer,d=expandbuf;i<asy.asy_serLength;i++,s++) {
    if (!p_isprint(*s)) {
      *d++='<';
      *d++=hexdigs[(*s&0xf0) >> 4];
      *d++=hexdigs[*s&0x0f];
      *d++='>';
      i+=3;
    }
    else
      *d++=*s;
  }
  msg_event(EVT_UI_TERMDATA, expandbuf, i);
}


/*******************************************************************************
***
*** Function         : term_sendkey(key)
*** Preconditions    : key is a keypress
*** Postconditions   : term_sendkey is TRUE, and the key is sent to the port
***                    term_sendkey is FALSE, and the interface is not NIS_TERM
***                    or the write failed. 
***
*******************************************************************************/

GLDEF_C INT term_sendkey(byte key)
{
INT errcode;
  if (netif->ni_state == NIS_TERM) {
    if ((errcode=asy_writechar(key)) != 1) {
      console_printf("term_sendkey: line failure %d\r\n", errcode);
      nif_close(netif);
      return FALSE;
    }
    else
      return TRUE;
  }
  else
    return FALSE;
}

