/*******************************************************************************
***
*** Filename         : timer.h
*** Purpose          : Timer routines for PsiStack
*** Author           : Matt J. Gumbley
*** Created          : 26/07/97
*** Last updated     : 26/07/97
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#ifndef TIMER_H
#define TIMER_H

/* Format of the tm_msg field:
  +---------------------------------------------------------------+
  |8  |4  |2  |1  |8  |4  |2  |1  |8  |4  |2  |1  |8  |4  |2  |1  |
  +---------------------------------------------------------------+
  | Data, typically a TCB number << 4             | TQ_xxx code   |
  +---------------------------------------------------------------+
*/

/* Can't remember if there's a reason why I don't use zero, but there are
   plenty of codes to go around */

#define TQ_TCP_SEND            0x1
#define TQ_TCP_PERSIST         0x2
#define TQ_TCP_RETRANSMIT      0x3
#define TQ_TCP_DELETE          0x4
#define TQ_SCRIPT_WAKEUP       0x5
#define TQ_PPP_RESTART_TIMEOUT 0x6 /* Patrick, you can redefine these to do */

#define TQ_PPP_xxx2            0x7 /* whatever you need in PPP */
#define TQ_PPP_xxx3            0x8
#define TQ_PPP_xxx4            0x9
#define TQ_PPP_xxx5            0xA
#define TQ_PPP_xxx6            0xB /* And 0xC..0xF are available too */
                                   /* If you change these, change the 
                                      tq_eventnames array in timer.c */
#define TQ_TYPE_MASK           0xF

/* If you add or modify these, don't forget to change the names array in 
   timer.c, timer_qdump */

#define EVENT(x)               (x & TQ_TYPE_MASK)
#define DATA(x)                (x >> 4)
#define MKEVENT(type, data)    ((data<<4) | (type & TQ_TYPE_MASK))

/* Variables for the timer stuff */
GLREF_D TEXT *tq_eventnames[];
GLREF_D WORD  timer_status;
GLREF_D ULONG timer_ticks;
GLREF_D ULONG timer_interval;

GLREF_D struct logstat timer_stat[];

#define timer_stat_oqueuefull timer_stat[0].ls_value
#define timer_stat_setoom timer_stat[1].ls_value

GLREF_C INT   timer_open(VOID);
GLREF_C VOID  timer_queue(VOID);
GLREF_C VOID  timer_cancel(VOID);
GLREF_C VOID  timer_process(VOID);
GLREF_C INT   timer_clear(INT msg);
GLREF_C INT   timer_left(INT msg);
GLREF_C INT   timer_set(INT msg, INT time);
GLREF_C VOID  timer_qdump(VOID);
GLREF_C VOID  timer_stats(VOID);
GLREF_C INT   timer_killtimers(struct tcb *);

#endif  /* TIMER_H */

