/*******************************************************************************
***
*** Filename         : util.c
*** Purpose          : Utility functions for PsiStack 
***                    (there are more in the link library)
*** Author           : Matt J. Gumbley
*** Created          : 20/01/97
*** Last updated     : 01/03/98
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#include "stack.h"


/*******************************************************************************
***
*** Function         : hexdump
*** Preconditions    : buf points to an area of memory and len is the length of
***                    this area
*** Postconditions   : An offset | hexadecimal | printable ASCII dump of the
***                    designated memory area is output
***
*******************************************************************************/

void hexdump(unsigned char *buf, long buflen)
{ 
char LINE[80];
long offset=0L;
long left=buflen;
int i,upto16,x;
unsigned char b;
  while (left>0) {
    for (i=0; i<78; i++)
      LINE[i]=' ';
    LINE[9]=LINE[59]='|';
    LINE[77]='\0';
    p_atos(LINE,"%08lX",offset); 
    LINE[8]=' ';
    upto16 = (left>16) ? 16 : (int)left;
    for (x=0; x<upto16; x++) {
      b = buf[offset+x];
      LINE[11+(3*x)]=hexdig((b&0xf0)>>4);
      LINE[12+(3*x)]=hexdig(b&0x0f);
      LINE[61+x]=p_isprint((char)b) ? ((char)b) : '.';
    }
    console_puts(LINE);
    offset+=16L;
    left-=16L;
  }
}



/*******************************************************************************
***
*** Function         : fatal(s)
*** Preconditions    : Some fatal error has occurred, s is a string describing
***                    the error.
*** Postconditions   : s is written to the output stream, and the system halts.
***
*******************************************************************************/

void fatal(char *s)
{
  console_print("Fatal:");
  console_puts(s);
  p_sleep(50);
  stack_close(1);
}


