/*******************************************************************************
***
*** Filename         : netdb.c
*** Purpose          : Network Database File Management 
***                    PsiStack application-side sockets library
*** Author           : Matt J. Gumbley
*** Created          : 13/10/97
*** Last updated     : 13/10/97
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#include "..\utils\sysincl.h"
#include "..\..\kernel\stackmsg.h"

GLREF_C UINT stack_sendmsg(UINT type, STKMSG *msg); /* Private implementation */

#include "..\..\include\psistack\stack.h"
#include "..\..\include\sys\socket.h"
#include "..\..\include\netdb.h"


int h_errno, h_epocerrno;

/* Static data area for the database functions. The addresses of these
   structures get passed to the PsiStack process. It scans the database files,
   and in some cases, polls a DNS server for data. 

   These structures are defined in the library - each process that is linked 
   with the library gets its own copy of the structures, but data stored here 
   by PsiStack must be copied between calls to the library, otherwise, the 
   result from the first call will be overwritten. 

   The asynchronous versions of the functions allow you to store results 
   in your own structures.
*/
static struct hostent ndb_hostent;

void endhostent(void)
{
}

void endnetent(void)
{
}

void endprotoent(void)
{
}

void endservent(void)
{
}

void endrpcent(void)
{
}

/* Get details from the hosts file, and if that fails, get them from DNS */
struct hostent *gethostbyaddr(char *addr, int len, int type)
{
STKMSG msg;
UINT ret;

  /* type param will always be AF_INET, len is 4 */
  if (type != AF_INET || len != 4) {
    h_errno = NETDB_INTERNAL;
    return NULL;
  }

  msg.sm_uwords[0] = (UWORD) addr;
  msg.sm_uwords[1] = 4;
  msg.sm_uwords[2] = (UWORD) &ndb_hostent;
  msg.sm_uwords[3] = sizeof(struct hostent);
  ret = stack_sendmsg(SMT_NDB_GETHOSTBYADDR, &msg);
  switch (ret) {
    case NETDB_SUCCESS:
      return &ndb_hostent;
    case NETDB_INTERNAL:
    case HOST_NOT_FOUND:
    case TRY_AGAIN:
    case NO_RECOVERY:
    case NO_DATA:
      h_errno = ret;
      h_epocerrno = 0;
      return NULL;
    default:
      h_errno = NO_RECOVERY;
      h_epocerrno = ret;
      return NULL;
  }
}

struct hostent *gethostbyname(char *name)
{
STKMSG msg;
UINT ret;
  msg.sm_uwords[0] = (UWORD) name;
  msg.sm_uwords[1] = p_slen(name);
  msg.sm_uwords[2] = (UWORD) &ndb_hostent;
  msg.sm_uwords[3] = sizeof(struct hostent);
  ret = stack_sendmsg(SMT_NDB_GETHOSTBYNAME, &msg);
  switch (ret) {
    case NETDB_SUCCESS:
      return &ndb_hostent;
    case NETDB_INTERNAL:
    case HOST_NOT_FOUND:
    case TRY_AGAIN:
    case NO_RECOVERY:
    case NO_DATA:
      h_errno = ret;
      h_epocerrno = 0;
      return NULL;
    default:
      h_errno = NO_RECOVERY;
      h_epocerrno = ret;
      return NULL;
  }
}

struct hostent *gethostent(void)
{
  return NULL;
}

struct netent *getnetbyaddr(long pa, int pb) /* nope */
{
  return NULL;
}

struct netent *getnetbyname(char *name)
{
  return NULL;
}

struct netent *getnetent(void)
{
  return NULL;
}

struct protoent *getprotobyname(char *name)
{
  return NULL;
}

struct protoent *getprotobynumber(int pnum)
{
  return NULL;
}

struct protoent *getprotoent(void)
{
  return NULL;
}

struct servent *getservbyname(char *service, char *transport)
{
  return NULL;
}

struct servent *getservbyport(int port, char *transport)
{
  return NULL;
}

struct servent *getservent(void)
{
  return NULL;
}

struct rpcent *getrpcent(void)
{
  return NULL;
}

struct rpcent *getrpcbyname(char *pa)
{
  return NULL;
}

struct rpcent *getrpcbynumber(int pa)
{
  return NULL;
}

void herror(char *pa)
{
}

void sethostent(int useTCP)
{
}

void sethostfile(char *fname)
{
  /* Do nowt - applications can't override our set files. */
}

void setnetent(int pa)
{
}

void setprotoent(int pa)
{
}

void setservent(int pa)
{
}

void setrpcent(int pa)
{
}

