/*******************************************************************************
***
*** Filename         : socket.c
*** Purpose          : PsiStack Sockets Library - BSD style
***                    A wrapper around the EPOC Sockets IPC messages.


well, it will be, when I've finsihed it! I trashed the sm_errno and
sm_epocerrno fields in the stack message format, so all this module need this
change reflecting. I'll wait until the EPOC-style socjets library is done
before working on this one - it's just an error code transaltion layer anyway.


*** Author           : Matt J. Gumbley
*** Created          : 28/07/97
*** Last updated     : 08/04/98
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#include "..\utils\sysincl.h"
#include "..\..\kernel\stackmsg.h"

GLREF_C UINT stack_sendmsg(UINT type, STKMSG *msg); /* Private implementation */

#include "..\..\include\psistack\stack.h"
#include "..\..\include\sys\socket.h" /* BSD Sockets interfaces */

GLDEF_D int errno, epocerrno; /* Each application that links with the sockets 
                                 library gets their own copy of these; errors 
                                 from the kernel are stored here. */


/*******************************************************************************
***
*** Function         : socket_sendmsg(msgtype, msg)
*** Preconditions    : msgtype is as defined in ..\kernel\stackmsg.h
***                    msg is a pointer to a STKMSG structure, holding a
***                    sockets library message ONLY. 
*** Postconditions   : The return value is a sockets.h return code. This is
***                    typically 0 on success, -1 on failure, but it depends
***                    on the type of sockets call being sent to the kernel.
***
***                    Even if an EPOC error has occurred, this is converted 
***                    into an appropriate sockets.h error, since this routine
***                    is used to return values to applications (in the end)
***                    from sockets routines. EPOC error codes are stored in
***                    msg->sm_epocerrno, and sockets.h error codes are in
***                    msg->sm_errno. The return value is as for the sockets
***                    library routines, send, revc, shutdown, etc. The error
***                    variables are library-global, and are overwritten on
***                    each sockets call, so copy them if you need to. They
***                    are guaranteed to be zeroed before the message is sent.
***                    Your app can check for them being non-zero after the
***                    sockets call (unless the sockets result is zero!)
***
*** Warning          : Do NOT use this to send stack messages to the kernel.
***                    Although they'll get sent OK, the return value is
***                    different. 
*** See also         : stack_sendmsg, which is exactly the same, but
***                    returns EPOC error codes.
***
*******************************************************************************/

LOCAL_C INT socket_sendmsg(UINT msgtype, STKMSG *msg)
{
INT retval;

  /* Initialise library-global error variables, and pointers to them in this
     message.
  */
  errno = epocerrno = 0;
  msg->sm_errno = &errno;
  msg->sm_epocerrno = &epocerrno;

  if (stack_ensureloaded() > 0) {

    /* This will block if the stack has no free message slots... */
    retval = p_msendreceivew(hStack, msgtype, (VOID *) msg);
    /* ...and will block until the stack has finished processing it */

    /* The return from that p_msendreceivew will be:

       E_GEN_RECEIVER (-17) meaning that the stack is not loaded, or has
       not yet called p_minit(). Convert this into the sockets code
       ENONET "Machine is not on the network".

       Or the server's p_mfree reply value:

       For sockets library calls, this is the return value to the sockets 
       call, -1 on failure for instance, a positive integer for socket 
       descriptors, 0 for success, etc. 

       For stack control calls, who knows, as I haven't done those yet.
    */
    if (retval == E_GEN_RECEIVER) {
      errno = ENONET;
      epocerrno = retval;
      return -1;
    }
    return retval;
  }

  /* The stack (is not/could not be) loaded. hStack will be set to
     an error code from p_execc or p_pidfind:
     E_GEN_NOMEMORY  - insufficient free system memory
     E_FILE_DEVICE   - device in stack_loadstack does not exist!
     E_FILE_NOTREADY -   "     "       "            "    "  contain a medium!
                       (Your internal drive must be broken?)
     E_FILE_DIR      - directory in stack_loadstack does not exist
                       (\APP - stack probably hasn't been installed properly)
     E_FILE_NXIST    - file in stack_loadstack does not exist
                       (stack hasn't been installed properly)
                     - no matching process
                       (the stack isn't running)
     E_FILE_EXIST    - there is some other program with the same name loaded
                       (sue for copyleft infringement ;-) )
     E_GEN_IMAGE     - stack kernel is not a valid image or corrupt
     E_GEN_NOPROC    - no free process slots
     E_GEN_ARG       - total size of data, stack and heap exceeds 0xFFE0
                       bytes 
                       (need to put the kernel on a diet ;-) )
  */
  errno = ENONET;
  epocerrno = hStack;
  return -1;
}


/*******************************************************************************
***
*** Function         : socket_syscallX(type, params*)
*** Preconditions    : type is a message type, params are a number of parameters
***                    There are X number of parameters for each of these fns.
*** Postconditions   : The type and parameters are placed into a message struct
***                    which is sent to the kernel, and the result is returned.
***
*******************************************************************************/

LOCAL_C int socket_syscall1(UINT type, UWORD w0)
{
STKMSG msg;
  msg.sm_uwords[0] = w0;
  return socket_sendmsg(type, &msg);
}

LOCAL_C int socket_syscall2(UINT type, UWORD w0, UWORD w1)
{
STKMSG msg;
  msg.sm_uwords[0] = w0;
  msg.sm_uwords[1] = w1;
  return socket_sendmsg(type, &msg);
}

LOCAL_C int socket_syscall3(UINT type, UWORD w0, UWORD w1, UWORD w2)
{
STKMSG msg;
  msg.sm_uwords[0] = w0;
  msg.sm_uwords[1] = w1;
  msg.sm_uwords[2] = w2;
  return socket_sendmsg(type, &msg);
}

LOCAL_C int socket_syscall4(UINT type, UWORD w0, UWORD w1, UWORD w2, UWORD w3)
{
STKMSG msg;
  msg.sm_uwords[0] = w0;
  msg.sm_uwords[1] = w1;
  msg.sm_uwords[2] = w2;
  msg.sm_uwords[3] = w3;
  return socket_sendmsg(type, &msg);
}


LOCAL_C int socket_syscall5(UINT type, UWORD w0, UWORD w1, UWORD w2, UWORD w3, 
                            UWORD w4)
{
STKMSG msg;
  msg.sm_uwords[0] = w0;
  msg.sm_uwords[1] = w1;
  msg.sm_uwords[2] = w2;
  msg.sm_uwords[3] = w3;
  msg.sm_uwords[4] = w4;
  return socket_sendmsg(type, &msg);
}

LOCAL_C int socket_syscall6(UINT type, UWORD w0, UWORD w1, UWORD w2, UWORD w3, 
                            UWORD w4, UWORD w5)
{
STKMSG msg;
  msg.sm_uwords[0] = w0;
  msg.sm_uwords[1] = w1;
  msg.sm_uwords[2] = w2;
  msg.sm_uwords[3] = w3;
  msg.sm_uwords[4] = w4;
  msg.sm_uwords[5] = w5;
  return socket_sendmsg(type, &msg);
}


/******************************************************************************/
/*************                                                 ****************/
/************* S O C K E T   L I B R A R Y   F U N C T I O N S ****************/
/*************                                                 ****************/
/******************************************************************************/


/*******************************************************************************
***
*** Function         : socket(domain, type, protocol)
*** Preconditions    : domain is a protocol family identifier, such as PF_INET
***                    type is a socket type identifier; SOCK_DGRAM, SOCK_RAW,
***                    SOCK_STREAM or SOCK_RAWINTERFACE. protocol is usually 0.
*** Postconditions   : Returns a socket descriptor for the new socket, or -1
***                    for errors.
***
*******************************************************************************/

int socket(int domain, int type, int protocol)
{
  return socket_syscall3(SMT_SL_SOCKET, domain, type, protocol);
}


/*******************************************************************************
***
*** Function         : socketpair(domain, type, protocol, sockvec)
*** Preconditions    : As for socket, and sockvec points to two integers.
*** Postconditions   : As for socket. The sockets are connected together.
***
*******************************************************************************/

int socketpair(int domain, int type, int protocol, int sockvec[2])
{
int ret;
  ret = socket_syscall3(SMT_SL_SOCKET, domain, type, protocol);
  if (ret != -1) {
    sockvec[0] = ret;
    ret = socket_syscall3(SMT_SL_SOCKET, domain, type, protocol);
    if (ret == -1) {
      /* Free first socket */
      return -1;
    }
    sockvec[1] = ret;
    /* Connect them together? */
    return 0;
  }
  return -1;
}


/*******************************************************************************
***
*** Function         : bind(sockfd, *saddr, addrlen)
*** Preconditions    : sockfd is a socket descriptor provided by socket or
***                    socketpair. saddr points to a sockaddr structure,
***                    and addrlen is the length of the address.
*** Postconditions   : The socket is given the address. 0 on success -1 on fail.
***
*******************************************************************************/

int bind(int sockfd, struct sockaddr *saddr, int addrlen)
{
  return socket_syscall3(SMT_SL_BIND, (UWORD)sockfd, (UWORD)saddr, 
                         (UWORD)addrlen);
}


/*******************************************************************************
***
*** Function         : connect(sockfd, serv_addr, addrlen)
*** Preconditions    : sockfd is a socket descriptor provided by socket or
***                    socketpair. serv_addr points to a sockaddr structure,
***                    and addrlen is the length of the address.
*** Postconditions   : A connection on socket sockfd to peer at address
***                    serv_addr (which is addrlen bytes long) is opened. For
***                    connectionless socket types, just set the default address
***                    to send to and the only address from which to accept
***                    transmissions. 0 on success -1 on failure.
***
*******************************************************************************/

int connect(int sockfd, struct sockaddr *serv_addr, int addrlen)
{
  return socket_syscall3(SMT_SL_CONNECT, (UWORD)sockfd, (UWORD)serv_addr, 
                         (UWORD)addrlen);
}


/*******************************************************************************
***
*** Function         : listen(sockfd, n)
*** Preconditions    : sockfd is a socket descriptor provided by socket or
***                    socketpair. n is an integer.
*** Postconditions   : The socket is prepared to queue n connection requests
***                    before further requests are refused. 0 on success
***                    -1 on failure.
***
*******************************************************************************/

int listen(int sockfd, int n)
{
  return socket_syscall2(SMT_SL_LISTEN, (UWORD)sockfd, (UWORD)n);
}


/*******************************************************************************
***
*** Function         : accept(sockfd, peeraddr, paddrlen)
*** Preconditions    : sockfd is a socket descriptor provided by socket or
***                    socketpair. peeraddr is the address of a sockaddr
***                    structure, and paddrlen is the address of an integer.
*** Postconditions   : A connection is waited for on the socket. When it
***                    arrives, a new socket is opened to communicate with it,
***                    *peeraddr (which is *paddrlen bytes long) is set to the
***                    address of the connecting peer, and *paddrlen is set
***                    to the address's actual length. The socket's new
***                    descriptor is returned, or -1 on failure.
***
*******************************************************************************/

int accept(int sockfd, struct sockaddr *peeraddr, int *paddrlen)
{
  return socket_syscall3(SMT_SL_ACCEPT, (UWORD)sockfd, (UWORD)peeraddr, 
                         (UWORD)paddrlen);
}


/*******************************************************************************
***
*** Function         : getsockopt(sockfd, level, optname, *optval, *optlen)
*** Preconditions    : sockfd is a socket descriptor provided by socket or
***                    socketpair. *optval is a buffer, *optlen bytes long.
*** Postconditions   : Put the current value for socket sockfd's option 
***                    optname at protocol level into optval and set 
***                    *optlen to the value's actual length. Returns 0 on 
***                    success, -1 for errors.
***
*******************************************************************************/

int getsockopt(int sockfd, int level, int optname, void *optval, int *optlen)
{
  return socket_syscall5(SMT_SL_GETSOCKOPT, (UWORD)sockfd, (UWORD)level, 
                         (UWORD)optname, (UWORD)optval, (UWORD)optlen);
}


/*******************************************************************************
***
*** Function         : setsockopt(sockfd, level, optname, *optval, optlen)
*** Preconditions    : sockfd is a socket descriptor provided by socket or
***                    socketpair. 
*** Postconditions   : Set socket sockfd's option optname at protocol level
***                    to *optval (which is optlen bytes long) Returns 0 on
***                    success, -1 on failure.
***
*******************************************************************************/

int setsockopt(int sockfd, int level, int optname, void *optval, int optlen)
{
  return socket_syscall5(SMT_SL_SETSOCKOPT, (UWORD)sockfd, (UWORD)level, 
                         (UWORD)optname, (UWORD)optval, (UWORD)optlen);
}


/*******************************************************************************
***
*** Function         : getsockname(sockfd, *addr, *paddrlen)
*** Preconditions    : sockfd is a socket descriptor provided by socket or
***                    socketpair. 
*** Postconditions   : Puts the local address of sockfd into *addr and its
***                    length in *paddrlen.
***
*******************************************************************************/

int getsockname(int sockfd, struct sockaddr *addr, int *paddrlen)
{
  return socket_syscall3(SMT_SL_GETSOCKNAME, (UWORD)sockfd, (UWORD)addr, 
                         (UWORD)paddrlen);
}


/*******************************************************************************
***
*** Function         : getpeername(sockfd, *peer, *paddrlen)
*** Preconditions    : sockfd is a socket descriptor provided by socket or
***                    socketpair. 
*** Postconditions   : Puts the address of the peer connected to socket sockfd 
***                    into *peer and its length in *paddrlen.
***
*******************************************************************************/

int getpeername(int sockfd, struct sockaddr *peer, int *paddrlen)
{
  return socket_syscall3(SMT_SL_GETPEERNAME, (UWORD)sockfd, (UWORD)peer, 
                         (UWORD)paddrlen);
}


/*******************************************************************************
***
*** Function         : send(sockfd, *buff, len, flags)
*** Preconditions    : sockfd is a socket descriptor provided by socket or
***                    socketpair. *buff is a pointer to a buffer, len is the
***                    length of the buffer contents, flags are flags :-)
*** Postconditions   : len bytes of the buffer are sent to the socket.
***                    Returns the number of bytes sent, or -1 on error.
***
*******************************************************************************/

int send(int sockfd, void *buff, int len, unsigned int flags)
{
  return socket_syscall4(SMT_SL_SEND, (UWORD)sockfd, (UWORD)buff, (UWORD)len, 
                         (UWORD)flags);
}


/*******************************************************************************
***
*** Function         : recv(sockfd, *buff, len, flags)
*** Preconditions    : sockfd is a socket descriptor provided by socket or
***                    socketpair. *buff is a pointer to a buffer, len is the
***                    length of the buffer, flags are flags :-)
*** Postconditions   : up to len bytes of the buffer are read from the socket.
***                    Returns the number of bytes read, or -1 on error.
***
*******************************************************************************/

int recv(int sockfd, void *buff, int len, unsigned int flags)
{
  return socket_syscall6(SMT_SL_RECVFROM, (UWORD)sockfd, (UWORD)buff, 
                         (UWORD)len, (UWORD)flags, (UWORD)NULL, (UWORD)NULL);
}


/*******************************************************************************
***
*** Function         : sendto(sockfd, *buff, len, flags, *to, tolen)
*** Preconditions    : sockfd is a socket descriptor provided by socket or
***                    socketpair. *buff is a pointer to a buffer, len is the
***                    length of the buffer, flags are flags, to is a pointer to
***                    a sockaddr structure and tolen is the length of that
***                    address.
*** Postconditions   : len bytes of *buff are sent to socket sockfd to the peer
***                    at address *to (of length tolen). Returns the number
***                    sent, or -1 on failure.
***
*******************************************************************************/

int sendto(int sockfd, void *buff, int len, 
           unsigned int flags, struct sockaddr *to, int tolen)
{
  return socket_syscall6(SMT_SL_SENDTO, (UWORD)sockfd, (UWORD)buff, 
                         (UWORD)len, (UWORD)flags, (UWORD)to, (UWORD)tolen);
}


/*******************************************************************************
***
*** Function         : recvfrom(sockfd, *buff, len, flags, *from, *fromlen)
*** Preconditions    : sockfd is a socket descriptor provided by socket or
***                    socketpair. *buff is a pointer to a buffer, len is the
***                    length of the buffer, flags are flags, from is a pointer
***                    to a sockaddr structure and tolen is the length of that
***                    address.
*** Postconditions   : up to len bytes of the buffer are read from the socket.
***                    If from is non-NULL, fill in *fromlen bytes of it with
***                    the address of the sender, and store the actual size
***                    of the address in *fromlen.
***                    Returns the number of bytes read, or -1 on error.
***
*******************************************************************************/

int recvfrom(int sockfd, void *buff, int len,
             unsigned int flags, struct sockaddr *from, int *fromlen)
{
  return socket_syscall6(SMT_SL_RECVFROM, (UWORD)sockfd, (UWORD)buff, 
                         (UWORD)len, (UWORD)flags, (UWORD)from, (UWORD)fromlen);
}


/**************
 Send a message described MESSAGE on socket FD.
   Returns the number of bytes sent, or -1 for errors.  
int sendmsg (int fd, struct msghdr *message,
      unsigned int flags);

   Receive a message as described by MESSAGE from socket FD.
   Returns the number of bytes read or -1 for errors. 
int recvmsg (int fd, struct msghdr *message,
      unsigned int flags);
***************/
 

/*******************************************************************************
***
*** Function         : shutdown(sockfd, how)
*** Preconditions    : sockfd is a socket descriptor provided by socket or
***                    socketpair. how specifies what to shut down:
***                    0 = no more receptions
***                    1 = no more transmissions
***                    2 = neither
*** Postconditions   : All or part of the connection open on sockfd is shut down
***                    Returns 0 on success, -1 on failure
***
*******************************************************************************/

int shutdown(int sockfd, int how)
{
  return socket_syscall2(SMT_SL_SHUTDOWN, (UWORD)sockfd, (UWORD)how);
}


/*******************************************************************************
***
*** Function         : close(sockfd)
*** Preconditions    : sockfd is a socket descriptor provided by socket or
***                    socketpair. 
*** Postconditions   : The socket is closed. Returns 0 on success, -1 on error.
***
*******************************************************************************/

int close(int sockfd)
{
  return socket_syscall1(SMT_SL_CLOSE, (UWORD)sockfd);
}

