/*******************************************************************************
***
*** Filename         : echod.c
*** Purpose          : A simple TCP ECHO server for PsiStack.
*** Author           : Matt J. Gumbley
*** Created          : 25/09/97
*** Last updated     : 04/07/98
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#include "p_std.h"
#include "p_file.h"
#include "p_sys.h"
#include "\sibosdk\include\psistack\sys\socket.h" /* #defines */
#include "\sibosdk\include\psistack\socket.h"     /* API prototypes */

#define BUFSIZE 4096

GLREF_D VOID *winHandle;
static char szCRLF[]="\r\n";

VOID log_write(TEXT *buf, int len)
{
  p_write(winHandle, buf, len);
  p_iow(winHandle, P_FFLUSH);
}


VOID log_writes(TEXT *s)
{
int len = p_slen(s);
  log_write(s,len);
}


VOID log_writesn(TEXT *s)
{
int len = p_slen(s);
  log_write(s, len);
  log_write(szCRLF, 2);
}


VOID CDECL log_printf(TEXT *fmt,UINT arg,...)
{
UINT len;
UBYTE buf[256]; 
  len = p_atob((TEXT *)&buf[0], fmt, &arg);
  log_write((TEXT *)buf, len);
}


void CDECL errexit(TEXT *fmt,UINT arg,...)
{
UINT len;
UBYTE buf[256]; 
  len = p_atob((TEXT *)&buf[0], fmt, &arg);
  log_write((TEXT *)buf, len);
  p_sleep(50); /* increase this? */
  p_exit(0);
}


int passiveopen()
{
struct sockaddr_in sin; /* an Internet endpoint address */
int s; /* socket descriptor */
int err; 

  p_bfil(&sin, sizeof(sin), 0);
  sin.sin_family = AF_INET;
  sin.sin_addr = INADDR_ANY;  /* this is 0 */
  sin.sin_port = IPPORT_ECHO; /* this is 7 */

  /* Allocate a socket */
  s = socket_socket(PF_INET, SOCK_STREAM, 0);
  if (s < 0)
    errexit("can't create socket: %d", s);

  /* Bind the socket */
  err = socket_bind(s, (struct sockaddr *)&sin, sizeof(sin));
  if (err < 0)
    errexit("can't bind to port: %d", err);

  /* Initialise the listen queue, with one connection allowed */
  err = socket_listen(s, 1);
  if (err < 0)
    errexit("can't listen on port: %d", err);

  /* Cool.. all OK */
  return s;
}



void echo(int fd)
{
char buf[BUFSIZE];
int cc;
  log_printf("Slave socket ID: %d\r\n", fd);
  while ((cc = socket_recv(fd, buf, sizeof(buf), 0)) != 0) {
    if (cc < 0)
      errexit("echo read: %d", cc);
    if (socket_send(fd, buf, cc, 0) < 0)
      errexit("echo write: %d", cc);
  }
}


void main()
{
struct sockaddr_in fsin; /* the address of a client */
int alen; /* length of client's address */
int msock; /* master server socket */
int ssock; /* slave server socket */
int quit;

  /* Open a socket, and specify that we'll be waiting for incoming 
     connections on it */
  p_puts("ECHO Server\r\nTelnet to me on port 7\r\nPerforming passive open...");
  msock = passiveopen();
  quit = FALSE;
  log_printf("Master socket ID: %d\r\n", msock);

  while (!quit) {
    alen = sizeof(fsin);
    /* Wait for an incoming connection */
    log_writesn("Waiting for a connection...");
    ssock = socket_accept(msock, (struct sockaddr *) &fsin, &alen);
    log_writesn("Connection accept unblocked...");
    if (ssock < 0) 
      errexit("accept error: %d", ssock);
    /* we have a connection, on a new socket */
    echo(ssock);
    socket_close(ssock);
  } 
  socket_close(msock);
}

