/*******************************************************************************
***
*** Filename         : dialogs.c
*** Purpose          : Dialogs for the front-end to PsiStack.
*** Author           : Matt J. Gumbley, Patrick H. Robbe
*** Created          : 05/04/97
*** Last updated     : 24/08/98
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#include "..\library\utils\sysincl.h"
#include "..\library\utils\machine.h"
#include "..\library\utils\version.h"
#include "..\include\psistack\sys\stack.h"    /* Kernel structure definitions */
#include "..\include\psistack\stack.h"                          /* Kernel API */
#include "..\include\psistack\events.h"                         /* Kernel API */
#include "..\library\utils\utils.h"
#include "dialogs.h"

LOCAL_D TEXT *szYes = "Yes";
LOCAL_D TEXT *szNo = "No";
LOCAL_D TEXT *szIgnore = "Ignore";
LOCAL_D TEXT *szNone = "None";
LOCAL_D TEXT *szGateways = "Gateways";
LOCAL_D TEXT *szDNSs = "DNSs";
LOCAL_D TEXT *szConfirm = "Confirm";
LOCAL_D TEXT *szConnectionNotDown = "Connection must be down";
LOCAL_D TEXT *szConnectionNotUp = "Connection must be up";
LOCAL_D TEXT *szNeedReload = "Reload to use new settings";
LOCAL_D TEXT *szStackCommFailure = "Stack communication failed";
LOCAL_D TEXT *szPPPNotSelected = "You must select PPP Protocol";

GLREF_D TEXT *DatUsedPathNamePtr;
GLDEF_D UWORD usesmalldialogs; /* Copied to _SmallFontDialog in myuOpenDialog */
GLREF_D UWORD _UseFullScreen, _SmallFontDialog;

/* Forward declarations */
LOCAL_C VOID CancelTimer(VOID);
LOCAL_C VOID SwitchMode(INT);



GLDEF_C INT myuOpenDialog(TEXT *title)
{
  _SmallFontDialog = usesmalldialogs;
  return uOpenDialog(title);
}

/*******************************************************************************
***
*** Function         : PortDialog
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID PortDialog(VOID)
{
struct asy asy;
static char szTTYA[]="TTY:A";
static char szTTYB[]="TTY:B";
static char szTTYI[]="TTY:I";
UWORD device;
/*UWORD origdevice;*/
UWORD baud;
UWORD databits;
UWORD stopbits;
UWORD parity;
UWORD handshaking;
UWORD devicedetect;
UWORD carrierdetect;
int valid;
UINT ni_state;
UINT returnval;

  ni_state = stack_get_netifstate();
  if (ni_state < 0) {
    wInfoMsg(szStackCommFailure);
    return;
  }
  if (ni_state != NIS_DOWN) {
    wInfoMsg(szConnectionNotDown);
    return;
  }

  returnval=stack_get_asy(&asy);
  if (returnval != TRUE) {
    wInfoMsg(szStackCommFailure);
    return;
  }

  /* Set up the defaults for the dialog, as taken from the network interface
     structure. */

  if (p_scmp(asy.asy_serDeviceName,szTTYB)==0)
    device=2;
  else if (p_scmp(asy.asy_serDeviceName,szTTYI)==0)
    device=3;
  else 
    device=1;

  /*origdevice = device;*/

  switch (asy.asy_speed) {
    case P_BAUD_300:
      baud = 1; break;
    case P_BAUD_600:
      baud = 2; break;
    case P_BAUD_1200:
      baud = 3; break;
    case P_BAUD_2400:
      baud = 4; break;
    case P_BAUD_4800:
      baud = 5; break;
    default:
    case P_BAUD_9600:
      baud = 6; break;
    case P_BAUD_19200:
      baud = 7; break;
    case P_BAUD_38400:
      baud = 8; break;
    case P_BAUD_56000:
      baud = 9; break;
    case P_BAUD_115000:
      baud = 10; break;
  }

  switch (asy.asy_databits) {
    case P_DATA_7:
      databits = 1; break;
    default:
    case P_DATA_8:
      databits = 2; break;
  }

  switch (asy.asy_stopbits) {
    default:
    case 0:
      stopbits = 1; break;
    case P_TWOSTOP:
      stopbits = 2; break;
  }

  switch (asy.asy_parity) {
    case IGNORE:
      parity = 1; break;
    default:
    case NOPAR:
      parity = 2; break;
    case EVENPAR:
      parity = 3; break;
    case ODDPAR:
      parity = 4; break;
  }

  switch (asy.asy_handshake) {
    case IGNORE:
      handshaking = 1; break;
    default:
    case RTSCTS: 
      handshaking = 2; break;
    case XONXOFF: 
      handshaking = 3; break;
  }

  /* 1 is yes, detect it, 2 is no, ignore it */
  devicedetect = asy.asy_devicedetect;
  carrierdetect = asy.asy_carrierdetect;

  /* Create the dialog. Loop forever, until the user puts valid settings in. */
  do {
    valid = TRUE;

    if (myuOpenDialog("Port settings") ||
        uAddChoiceList("Device", &device, 
          "RS232 (TTY:A)",
          "RS232 (TTY:B)",
          "IrDA (TTY:I)" ,NULL) ||
        uAddChoiceList("Baud rate", &baud,
          szBaudRates[BR_300],
          szBaudRates[BR_600],
          szBaudRates[BR_1200],
          szBaudRates[BR_2400],
          szBaudRates[BR_4800],
          szBaudRates[BR_9600],
          szBaudRates[BR_19200],
          szBaudRates[BR_38400],
          szBaudRates[BR_57600],
          szBaudRates[BR_115200], NULL) ||
        uAddChoiceList("Data bits", &databits, "7", "8", NULL) ||
        uAddChoiceList("Stop bits", &stopbits, "1", "2", NULL) ||
        uAddChoiceList("Parity", &parity, 
          szIgnore, "None", "Even", "Odd", NULL) ||
        uAddChoiceList("Handshaking", &handshaking, 
          szIgnore, "RTS/CTS", "XON/XOFF", NULL) ||
        uAddChoiceList("Ignore DSR/DTR", &devicedetect, 
          szNo, szYes, NULL) ||
        uAddChoiceList("Ignore DCD", &carrierdetect, 
          szNo, szYes, NULL) ||
        uRunDialog()<=0)
      return;

    /* Are these settings valid? */
    if (device != 3 && baud == 10) { /* Serial port at 115200?! */
      wInfoMsg("RS232 > 115200 baud");
      valid = FALSE;
    }
    /* Need check for a 3a, since this can't have IrDA, or serial ports
       faster than 19200 (CHECK THIS) */
    if (machine_type == MACH_SIENA && baud > 7 && device ==3) {
      wInfoMsg("IrDA > 19200 baud");
      valid = FALSE;
    }
  } while (!valid);

  /* Get the user's settings, and put them back into the network interface.
     Adjust the serial device's settings. */

  switch (device) {
    case 1: p_scpy(asy.asy_serDeviceName, szTTYA); break;
    case 2: p_scpy(asy.asy_serDeviceName, szTTYB); break;
    case 3: p_scpy(asy.asy_serDeviceName, szTTYI); break;
  }

  switch (baud) {
    case 1: asy.asy_speed = P_BAUD_300; break;
    case 2: asy.asy_speed = P_BAUD_600; break;
    case 3: asy.asy_speed = P_BAUD_1200; break;
    case 4: asy.asy_speed = P_BAUD_2400; break;
    case 5: asy.asy_speed = P_BAUD_4800; break;
    case 6: asy.asy_speed = P_BAUD_9600; break;
    case 7: asy.asy_speed = P_BAUD_19200; break;
    case 8: asy.asy_speed = P_BAUD_38400; break;
    case 9: asy.asy_speed = P_BAUD_56000; break;
    case 10:asy.asy_speed = P_BAUD_115000; break;
  }

  switch (databits) {
    case 1 : asy.asy_databits = P_DATA_7; break;
    case 2 : asy.asy_databits = P_DATA_8; break;
  }

  switch (stopbits) {
    case 1: asy.asy_stopbits = 0; break;
    case 2: asy.asy_stopbits = P_TWOSTOP; break;
  }

  switch (parity) {
    case 1: asy.asy_parity = IGNORE; break;
    case 2: asy.asy_parity = NOPAR; break;
    case 3: asy.asy_parity = EVENPAR; break;
    case 4: asy.asy_parity = ODDPAR; break;
  }

  switch (handshaking) {
    case 1: asy.asy_handshake = IGNORE; break;
    case 2: asy.asy_handshake = RTSCTS; break;
    case 3: asy.asy_handshake = XONXOFF; break;
  }

  asy.asy_devicedetect = devicedetect;
  asy.asy_carrierdetect = carrierdetect;

  /* Adjust the serial port immediately */
  if (stack_set_asy(&asy) != TRUE)
    wInfoMsg(szStackCommFailure);

  /* I have no idea why I wrote this bit of code :-(

   * If they've changed the serial port, then close the old port, before
     opening the new one. * 
  if (origdevice != device) {
    wInfoMsg("Changing device");
    if (stack_close_asy() != TRUE)
      wInfoMsg(szStackCommFailure);
  }
  if (stack_open_asy() != TRUE)
    wInfoMsg(szStackCommFailure);
  */
}


/*******************************************************************************
***
*** Function         : StateDialog
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID StateDialog(VOID)
{
static char szCancel[]="Cancel";
static char szUp[]="Up";
static char szScript[]="Script";
static char szTerminal[]="Term";
static char szDown[]="Down";
static char szPPPnegotiateUP[]="Opening";
static char szPPPnegotiateDOWN[]="Closing";
static char szChange[]="Changed to ";
TEXT titlebuf[40];
UINT ni_state;
struct scriptcontext sc;

  ni_state = stack_get_netifstate();
  if (ni_state < 0) {
    wInfoMsg(szStackCommFailure);
    return;
  }
  if (stack_get_sc(&sc) != TRUE) {
    wInfoMsg(szStackCommFailure);
    return;
  }

  p_scpy(titlebuf, "Change from ");
  switch (ni_state) {
    case NIS_UP:       p_scat(titlebuf, szUp); break;
    case NIS_SCRIPT:   p_scat(titlebuf, szScript); break;
    case NIS_TERM:     p_scat(titlebuf, szTerminal); break;
    case NIS_DOWN:     p_scat(titlebuf, szDown); break;
    case NIS_PPP_UP:   p_scat(titlebuf, szPPPnegotiateUP); break;
    case NIS_PPP_DOWN: p_scat(titlebuf, szPPPnegotiateDOWN); break;
    default:           p_scat(titlebuf,"*BUG*"); break;
  }
  p_scat(titlebuf," to...");
  if (myuOpenDialog(titlebuf))
    return;

  /* Certain state changes are only allowed in certain states. For instance,
     you can't change from up to terminal at all, and script and terminal are
     only reachable from down. See the diagram in the technical guide, in the
     section on interface states */

  switch (ni_state) {

    case NIS_UP:
    case NIS_PPP_UP:
    case NIS_PPP_DOWN:
      if (uAddButtonList(szCancel,W_KEY_ESCAPE,
                         szDown,'d', NULL))
        return;
      break;

    case NIS_SCRIPT:
      if (uAddButtonList(szCancel,W_KEY_ESCAPE,
                         szDown,'d', NULL))
        return;
      break;
      
    case NIS_TERM:
      if (uAddButtonList(szCancel,W_KEY_ESCAPE,
                         szDown,'d',
                         szUp,'u', NULL))
        return;
      break;

    case NIS_DOWN:
      /* Two cases: null script, or proper script */
      if (!sc.si_scriptused) {
        if (uAddButtonList(szCancel,W_KEY_ESCAPE,
                           szUp,'u',
                           szTerminal,'t', NULL))
          return;
      }
      else {
        if (uAddButtonList(szCancel,W_KEY_ESCAPE,
                           szScript,'s',
                           szTerminal,'t', NULL))
          return;
      }
      break;

  }
  /* Change state to the newly requested one. The stack will send an event when 
     it has changed  state, so no need to display an infomsg here */
  switch (uRunDialog()) {
    case W_KEY_ESCAPE:
      return;
    case 'd':
      if (stack_close_netif() <= 0) 
        wInfoMsg(szStackCommFailure);
      break;
    case 's':
      if (stack_open_netif(NIS_UP) <= 0) 
        wInfoMsg(szStackCommFailure);
      break;
    case 't':
      if (stack_open_netif(NIS_TERM) <= 0) 
        wInfoMsg(szStackCommFailure);
      break;
    case 'u':
      if (stack_open_netif(NIS_UP) <= 0)
        wInfoMsg(szStackCommFailure);
      break;
  }
}


/*******************************************************************************
***
*** Function         : ScriptDialog
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID ScriptDialog(VOID)
{
H_DI_FSEL fsel;
TEXT fname[P_FNAMESIZE+2];
UWORD scriptused,debuglevel;
struct scriptcontext sc;
UINT ni_state;

  ni_state = stack_get_netifstate();
  if (ni_state < 0) {
    wInfoMsg(szStackCommFailure);
    return;
  }

  if (ni_state != NIS_DOWN) {
    wInfoMsg(szConnectionNotDown);
    return;
  }

  if (stack_get_sc(&sc) != TRUE) {
    wInfoMsg(szStackCommFailure);
    return;
  }

  scriptused = sc.si_scriptused ? 1 : 2;
  debuglevel = (UWORD) sc.si_debuglevel;
  p_scpy(&fname[1],sc.si_scriptname);
  fname[0]=(BYTE)p_slen(sc.si_scriptname);
  fname[1+fname[0]]=0;

  fsel.fname=(&fname[0]);
  fsel.flags=H_FILE_PICK_SELECTOR|H_FILE_RESTRICT_LIST|H_FILE_SET_DEFEXT|
             H_FILE_ACCEPT_NULL;

  if (myuOpenDialog("Script change"))
    return;
  if (uAddChoiceList("Use script?", &scriptused, szYes, szNo, NULL))
    return;
  if (uAddChoiceList("Debug level", &debuglevel, "None", "Trace", "Trace+Input", NULL))
    return;
  uSetDialogUline(2, TRUE);
  if (uAddDialogItem(H_DIALOG_FSEL, "Script:", &fsel))
    return;
  if (uRunDialog()<=0)
    return;
  fname[1+fname[0]]=0;

  p_scpy(sc.si_scriptname,&fname[1]);
  sc.si_scriptused = (scriptused==1);
  sc.si_debuglevel = (byte) debuglevel & 0xff;

  if (stack_set_sc(&sc) != TRUE) 
    wInfoMsg(szStackCommFailure);
}


/*******************************************************************************
***
*** Function         : ISPDialog
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID ISPDialog(VOID)
{
H_DI_SEDIT di_name;
TEXT ispname[ISPSTRINGMAX+1];

H_DI_SEDIT di_phone1;
TEXT phone1[ISPSTRINGMAX+1];

H_DI_SEDIT di_phone2;
TEXT phone2[ISPSTRINGMAX+1];

H_DI_SEDIT di_phone3;
TEXT phone3[ISPSTRINGMAX+1];

H_DI_SEDIT di_username;
TEXT username[ISPSTRINGMAX+1];

H_DI_XINPUT di_password;
TEXT password[ISPSTRINGMAX+1];

UWORD di_datalinkproto;
struct ISPcontext ispc;
struct netif netif;

  if (stack_get_netif(&netif) != TRUE) {
    wInfoMsg(szStackCommFailure);
    return;
  }

  if (netif.ni_state != NIS_DOWN) {
    wInfoMsg(szConnectionNotDown);
    return;
  }

  if (stack_get_ispc(&ispc) != TRUE) {
    wInfoMsg(szStackCommFailure);
    return;
  }

  /* Copy data in */
  di_name.len=ISPSTRINGMAX;
  di_name.width=ISPSTRINGMAX;
  di_name.str=&ispname[0];
  uZTStoBCS(di_name.str, ispc.isp_name);

  di_phone1.len=ISPSTRINGMAX;
  di_phone1.width=ISPSTRINGMAX;
  di_phone1.str=&phone1[0];
  uZTStoBCS(di_phone1.str, ispc.isp_phone1);

  di_phone2.len=ISPSTRINGMAX;
  di_phone2.width=ISPSTRINGMAX;
  di_phone2.str=&phone2[0];
  uZTStoBCS(di_phone2.str, ispc.isp_phone2);

  di_phone3.len=ISPSTRINGMAX;
  di_phone3.width=ISPSTRINGMAX;
  di_phone3.str=&phone3[0];
  uZTStoBCS(di_phone3.str, ispc.isp_phone3);

  di_username.len=ISPSTRINGMAX;
  di_username.width=ISPSTRINGMAX;
  di_username.str=&username[0];
  uZTStoBCS(di_username.str, ispc.isp_username);

  di_password.str=&password[0];
  uZTStoBCS(di_password.str, ispc.isp_password);

  di_datalinkproto = netif.ni_dlp - 1;
  /* To map NIDLP_SLIP to 0x01 since 0x01 = NIDLP_LOCAL */

  do { 

    /* Do the dialog */
    if (myuOpenDialog("ISP account"))
      return;

    if (uAddDialogItem(H_DIALOG_SEDIT, "ISP name", &di_name))
      return;

    uSetDialogUline(1, TRUE);
    if (uAddDialogItem(H_DIALOG_SEDIT, "\005 number 1", &di_phone1))
      return;

    if (uAddDialogItem(H_DIALOG_SEDIT, "\005 number 2", &di_phone2))
      return;

    if (uAddDialogItem(H_DIALOG_SEDIT, "\005 number 3", &di_phone3))
      return;

    uSetDialogUline(4, TRUE);
    if (uAddDialogItem(H_DIALOG_SEDIT, "User name", &di_username))
      return;

    if (uAddDialogItem(H_DIALOG_XINPUT, "Password", &di_password))
      return;

    uSetDialogUline(6, TRUE);
    if (uAddChoiceList("Data link", &di_datalinkproto, 
        "SLIP", "CSLIP", "PPP", "KISS", NULL))
      return;

    if (uRunDialog()<=0)
      return;

    if (di_datalinkproto == 2)
      wInfoMsg("CSLIP is not yet supported");
    if (di_datalinkproto == 4)
      wInfoMsg("KISS is not yet supported");
  }
  while (di_datalinkproto == 2 || di_datalinkproto == 4);

  /* Copy back */
  ispname[1+ispname[0]]=0;
  p_scpy(ispc.isp_name,&ispname[1]);

  phone1[1+phone1[0]]=0;
  p_scpy(ispc.isp_phone1,&phone1[1]);

  phone2[1+phone2[0]]=0;
  p_scpy(ispc.isp_phone2,&phone2[1]);

  phone3[1+phone3[0]]=0;
  p_scpy(ispc.isp_phone3,&phone3[1]);

  username[1+username[0]]=0;
  p_scpy(ispc.isp_username,&username[1]);

  password[1+password[0]]=0;
  p_scpy(ispc.isp_password,&password[1]);

  netif.ni_dlp = di_datalinkproto + 1;

  if (stack_set_netif(&netif) != TRUE)
    wInfoMsg(szStackCommFailure);

  if (stack_set_ispc(&ispc) != TRUE)
    wInfoMsg(szStackCommFailure);
}


/*******************************************************************************
***
*** Function         : PasswordDialog
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID PasswordDialog(VOID)
{
TEXT usertitle[50];

H_DI_XINPUT di_password;
TEXT password[ISPSTRINGMAX+1];
struct ISPcontext ispc;
UINT ni_state;

  ni_state = stack_get_netifstate();
  if (ni_state < 0) {
    wInfoMsg(szStackCommFailure);
    return;
  }

  if (ni_state != NIS_DOWN) {
    wInfoMsg(szConnectionNotDown);
    return;
  }
 
  if (stack_get_ispc(&ispc) != TRUE) {
    wInfoMsg(szStackCommFailure);
    return;
  }

  /* Copy data in */
  di_password.str=&password[0];
  uZTStoBCS(di_password.str, ispc.isp_password);

  /* Do the dialog */
  p_atos(usertitle, "Password for \"%s\"", ispc.isp_username);
  if (myuOpenDialog(usertitle))
    return;

  if (uAddDialogItem(H_DIALOG_XINPUT, "Password", &di_password))
    return;

  if (uRunDialog()<=0)
    return;

  /* Copy back */
  password[1+password[0]]=0;
  p_scpy(ispc.isp_password,&password[1]);
  
  if (stack_set_ispc(&ispc) != TRUE)
    wInfoMsg(szStackCommFailure);
}



/*******************************************************************************
***
*** Function         : MultiAddrsDialog
*** Preconditions    : numaddrs is the number of addresses, and must be <=4
*** Postconditions   : A dialog has been presented to edit these IP addrs
***
*******************************************************************************/

GLDEF_C VOID MultiAddressesDialog(TEXT *heading, TEXT *singular,
                                  INT numaddrs, IPaddr *addrtable)
{
int i, error;
TEXT errmsg[20];
struct addrdetail {
  H_DI_EDIT di_IPaddress;
  TEXT IPaddress[16];
  TEXT Description[16];
  IPaddr ip_IPaddress;
} copyaddr[4];

  /* Copy data in */
  for (i=0; i<numaddrs; i++) {
    copyaddr[i].di_IPaddress.len=15;
    copyaddr[i].di_IPaddress.str=&copyaddr[i].IPaddress[0];
    p_bcpy(copyaddr[i].ip_IPaddress, addrtable[i], IP_ALEN);
    p_atos(&copyaddr[i].Description[0], "%s %d", singular, i+1);
    uZTStoBCS(copyaddr[i].di_IPaddress.str, szConvertIPaddr(addrtable[i]));
  }

  /* Run dialog until valid */
  for (;;) {
    if (myuOpenDialog(heading))
      return;
    for (i=0; i<numaddrs; i++) {
      if (uAddDialogItem(H_DIALOG_EDIT, copyaddr[i].Description, 
                                        &copyaddr[i].di_IPaddress))
        return;
    }
    if (uRunDialog() <= 0)
      return;

    /* Validate */
    error = FALSE;
    for (i=0; i<numaddrs; i++) {
      copyaddr[i].IPaddress[1+copyaddr[i].IPaddress[0]]=0;

      if (!IPaddrConvertsz(copyaddr[i].IPaddress+1, copyaddr[i].ip_IPaddress)) {
        p_atos(errmsg, "Address %d is invalid", i+1);
        wInfoMsg(errmsg);
        error = TRUE;
        break;
      }
    }

    if (!error)
      break;
  }

  /* All OK. Copy to main nif structure */
  for (i=0; i<numaddrs; i++) 
    p_bcpy(addrtable[i], copyaddr[i].ip_IPaddress, IP_ALEN);
}


/*******************************************************************************
***
*** Function         : IPAddressesDialog
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID IPAddressesDialog(VOID)
{
H_DI_EDIT di_IPaddress;
TEXT IPaddress[16];
IPaddr ip_IPaddress;

H_DI_EDIT di_SubnetMask;
TEXT SubnetMask[16];
IPaddr ip_SubnetMask;

struct netif netif;

  if (stack_get_netif(&netif) != TRUE) {
    wInfoMsg(szStackCommFailure);
    return;
  }

  if (netif.ni_state != NIS_DOWN) {
    wInfoMsg(szConnectionNotDown);
    return;
  }

  /* Copy data in */
  di_IPaddress.len=15;
  di_IPaddress.str=&IPaddress[0];
  p_bcpy(ip_IPaddress, netif.ni_ip, IP_ALEN);
  uZTStoBCS(di_IPaddress.str, szConvertIPaddr(netif.ni_ip));

  di_SubnetMask.len=15;
  di_SubnetMask.str=&SubnetMask[0];
  p_bcpy(ip_SubnetMask, netif.ni_mask, IP_ALEN);
  uZTStoBCS(di_SubnetMask.str, szConvertIPaddr(netif.ni_mask));

  /* Run dialog until valid */
  for (;;) {
    if (myuOpenDialog("IP addresses") ||
        uAddDialogItem(H_DIALOG_EDIT, "Interface", &di_IPaddress) ||
        uAddDialogItem(H_DIALOG_EDIT, "Subnet mask", &di_SubnetMask) ||
        uAddButtonList(szDNSs, 'd', szGateways, 'g', 
                       szConfirm, W_KEY_RETURN, NULL))
      return;
    switch (uRunDialog()) {
      case 'd':
        MultiAddressesDialog(szDNSs, "Server addr",
                             NI_DNSS, &netif.ni_dnss[0]);
        continue;
      case 'g':
        MultiAddressesDialog(szGateways, "Gateway addr",
                             NI_GATEWAYS, &netif.ni_gateways[0]);
        continue;
      case W_KEY_RETURN:
        break;
      default: /* negative */
        return;
    }

    /* Validate */
    IPaddress[1+IPaddress[0]]=0;
    SubnetMask[1+SubnetMask[0]]=0;

    if (!IPaddrConvertsz(IPaddress+1, ip_IPaddress)) {
      wInfoMsg("Bad interface address");
      continue;
    }

    if (!IPaddrConvertsz(SubnetMask+1, ip_SubnetMask)) {
      wInfoMsg("Bad subnet mask");
      continue;
    }

    /* All OK. Copy to main nif structure */
    p_bcpy(netif.ni_ip, ip_IPaddress, IP_ALEN);
    p_bcpy(netif.ni_mask, ip_SubnetMask, IP_ALEN);

    if (stack_set_netif(&netif) != TRUE)
      wInfoMsg(szStackCommFailure);
    return;
  }
}


/*******************************************************************************
***
*** Function         : PingDialog
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C INT PingDialog(VOID)
{
H_DI_EDIT di_IPaddress;
TEXT IPaddress[16];
IPaddr ip_IPaddress;
H_DI_NUMBER di_size;
LONG size;
H_DI_NUMBER di_numpings;
LONG numpings;
UINT ni_state;
struct ping_info p;

  ni_state = stack_get_netifstate();
  if (ni_state < 0) {
    wInfoMsg(szStackCommFailure);
    return;
  }

  if (ni_state != NIS_UP) {
    wInfoMsg(szConnectionNotUp);
    return;
  }
 
  if (stack_get_ping(&p) != TRUE) {
    wInfoMsg(szStackCommFailure);
    return;
  }

  /* Copy data in */
  di_IPaddress.len=15;
  di_IPaddress.str=&IPaddress[0];
  p_bcpy(ip_IPaddress, p.p_ip, IP_ALEN);
  uZTStoBCS(di_IPaddress.str, szConvertIPaddr(p.p_ip));

  di_size.value=&size;
  di_size.low=1;
  di_size.high=65535L - sizeof(struct ip) - sizeof(struct icmp);
  size = p.p_size;

  di_numpings.value=&numpings;
  di_numpings.low=1;
  di_numpings.high=100;
  numpings = p.p_numpings;

  /* Run dialog until valid */
  for (;;) {
    if (myuOpenDialog("Ping parameters") ||
        uAddDialogItem(H_DIALOG_EDIT, "IP address", &di_IPaddress) ||
        uAddDialogItem(H_DIALOG_NUMBER, "Payload size", &di_size) ||
        uAddDialogItem(H_DIALOG_NUMBER, "Number of pings", &di_numpings) ||
        uRunDialog()<=0)
      return FALSE;

    /* Validate */
    IPaddress[1+IPaddress[0]]=0;
    if (!IPaddrConvertsz(IPaddress+1, ip_IPaddress)) 
      wInfoMsg("Bad IP address");
    else
      break;
  }

  /* All OK. Copy to main pinginfo structure */
  p_bcpy(p.p_ip, ip_IPaddress, IP_ALEN);
  p.p_size = size;
  p.p_numpings = numpings;

  if (stack_set_ping(&p) != TRUE)
    wInfoMsg(szStackCommFailure);
  else {
    /* Generate the first ICMP echo request frame. 
       ping_process will be called as a result of this. */
    stack_send_ping();
  }
  return TRUE;
}


/*******************************************************************************
***
*** Function         : LogDialog
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID LogDialog(VOID)
{
UWORD logflag, miscflag;
UWORD info, flowtrace, linkdump, ipdump, icmpdump, upperdump, termctrl;
UWORD timestamp;

  /* Copy data in */
  logflag = stack_get_log();
  if (logflag < 0) {
    wInfoMsg(szStackCommFailure);
    return;
  }
  miscflag = stack_get_misc();

  info = (logflag & LOG_FLAG_MSG) ? 1 : 2;
  flowtrace = (logflag & LOG_FLAG_FLOWTRACE) ? 1 : 2;
  linkdump = (logflag & LOG_FLAG_LINKDUMP) ? 1 : 2;
  ipdump = (logflag & LOG_FLAG_IPDUMP) ? 1 : 2;
  icmpdump = (logflag & LOG_FLAG_ICMPDUMP) ? 1 : 2;
  upperdump = (logflag & LOG_FLAG_UPPERDUMP) ? 1 : 2;
  timestamp = (logflag & LOG_FLAG_TIMESTAMP) ? 1 : 2;

  termctrl = (miscflag & MISC_FLAG_TERMCTRLCHARS) ? 1 : 2;

  if (myuOpenDialog("Logging facilities") ||
      uAddChoiceList("IPC messages", &info, szYes, szNo, NULL) ||
      uAddChoiceList("Flow Trace", &flowtrace, szYes, szNo, NULL) ||
      uAddChoiceList("Link Dump", &linkdump, szYes, szNo, NULL) ||
      uAddChoiceList("IP Dump", &ipdump, szYes, szNo, NULL) ||
      uAddChoiceList("ICMP Dump", &icmpdump, szYes, szNo, NULL) ||
      uAddChoiceList("TCP/UDP Dump", &upperdump, szYes, szNo, NULL) ||
      uAddChoiceList("Timestamping", &timestamp, szYes, szNo, NULL) ||
      /* May move this to another dialog - it'll get too depp otherwise */
      uAddChoiceList("Term ctrl chars", &termctrl, "Expanded", "Raw", NULL))
    return;
  uSetDialogUline(7, TRUE);
  if (uRunDialog()<=0)
    return;

  logflag = logflag & ~(LOG_FLAG_MSG | LOG_FLAG_FLOWTRACE | 
                        LOG_FLAG_LINKDUMP | LOG_FLAG_IPDUMP | 
                        LOG_FLAG_ICMPDUMP | LOG_FLAG_UPPERDUMP |
                        LOG_FLAG_TIMESTAMP);
  logflag |= (info == 1) ? LOG_FLAG_MSG : 0;
  logflag |= (flowtrace == 1) ? LOG_FLAG_FLOWTRACE : 0;
  logflag |= (linkdump == 1) ? LOG_FLAG_LINKDUMP : 0;
  logflag |= (ipdump == 1) ? LOG_FLAG_IPDUMP : 0;
  logflag |= (icmpdump == 1) ? LOG_FLAG_ICMPDUMP : 0;
  logflag |= (upperdump == 1) ? LOG_FLAG_UPPERDUMP : 0;
  logflag |= (timestamp == 1) ? LOG_FLAG_TIMESTAMP : 0;

  miscflag = miscflag & ~(MISC_FLAG_TERMCTRLCHARS);
  miscflag |= (termctrl == 1) ? MISC_FLAG_TERMCTRLCHARS : 0;

  if (stack_set_log(logflag) != TRUE)
    wInfoMsg(szStackCommFailure);

  if (stack_set_misc(miscflag) != TRUE)
    wInfoMsg(szStackCommFailure);

  if (stack_set_asyhandler() != TRUE)
    wInfoMsg(szStackCommFailure);
}


/*******************************************************************************
***
*** Function         : StatsDialog
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID StatsDialog(VOID)
{
UWORD miscflag, endstats;
  
  /* Copy data in */
  miscflag = stack_get_misc();
  if (miscflag < 0) {
    wInfoMsg(szStackCommFailure);
    return;
  }

  endstats = (miscflag & MISC_FLAG_ENDSTATS) ? 1 : 2;

  /* Run dialog until valid */
  if (myuOpenDialog("Stats Production") ||
      uAddChoiceList("Stats on close?", &endstats, szNo, szYes, NULL) ||
      uAddButtonList("Generate now", 'n', szConfirm, W_KEY_RETURN, NULL))
      return;
  switch (uRunDialog()) {
    case 'n':
      if (stack_stats() != TRUE)
        wInfoMsg(szStackCommFailure);
      break;
    case W_KEY_RETURN:
      break;
    default: /* negative */
      return;
  }

  miscflag = (miscflag & ~(MISC_FLAG_ENDSTATS));
  miscflag |= (endstats == 1) ? MISC_FLAG_ENDSTATS : 0;

  if (stack_set_misc(miscflag) != TRUE)
    wInfoMsg(szStackCommFailure);
}


/*******************************************************************************
***
*** Function         : UDPDialog
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID UDPDialog(VOID)
{
H_DI_NUMBER di_UPPS;
LONG UPPS;
H_DI_NUMBER di_UPPLEN;
LONG UPPLEN;
UINT ni_state;
struct udp_config u;

  ni_state = stack_get_netifstate();
  if (ni_state < 0) {
    wInfoMsg(szStackCommFailure);
    return;
  }

  if (ni_state != NIS_DOWN) {
    wInfoMsg(szConnectionNotDown);
    return;
  }
 
  if (stack_get_udp_config(&u) != TRUE) {
    wInfoMsg(szStackCommFailure);
    return;
  }

  /* Copy data in */
  di_UPPS.value=&UPPS;
  di_UPPS.low=MINUPPS;
  di_UPPS.high=MAXUPPS;
  UPPS = u.udpc_UPPS;

  di_UPPLEN.value=&UPPLEN;
  di_UPPLEN.low=MINUPPLEN;
  di_UPPLEN.high=MAXUPPLEN;
  UPPLEN = u.udpc_UPPLEN;

  if (myuOpenDialog("UDP settings") ||
      uAddDialogItem(H_DIALOG_NUMBER, "# UDP ports", &di_UPPS) ||
      uAddDialogItem(H_DIALOG_NUMBER, "UDP port queue size", &di_UPPLEN) ||
      uRunDialog()<=0)
    return;

  /* All OK. Copy to main udp_config structure */
  u.udpc_UPPS = UPPS;
  u.udpc_UPPLEN = UPPLEN;

  if (stack_set_udp_config(&u) != TRUE)
    wInfoMsg(szStackCommFailure);
  else
    wInfoMsg(szNeedReload);
}


/*******************************************************************************
***
*** Function         : TCPDialog
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID TCPDialog(VOID)
{
UWORD bsdurg;
#define DEF_NO 1
#define DEF_YES 2
UINT ni_state;
H_DI_NUMBER di_NTCBS;
LONG NTCBS;
struct tcp_config t;

  ni_state = stack_get_netifstate();
  if (ni_state < 0) {
    wInfoMsg(szStackCommFailure);
    return;
  }

  if (ni_state != NIS_DOWN) {
    wInfoMsg(szConnectionNotDown);
    return;
  }
 
  if (stack_get_tcp_config(&t) != TRUE) {
    wInfoMsg(szStackCommFailure);
    return;
  }

  /* Copy data in */
  bsdurg = t.tcpc_bsdurg ? DEF_YES : DEF_NO;

  di_NTCBS.value=&NTCBS;
  di_NTCBS.low=MINNTCBS;
  di_NTCBS.high=MAXNTCBS;
  NTCBS = t.tcpc_NTCBS;

  /* Run dialog until valid */
  if (myuOpenDialog("TCP settings") ||
      uAddChoiceList("BSD urgent semantics?", &bsdurg, szNo, szYes, NULL) ||
      uAddDialogItem(H_DIALOG_NUMBER, "# TCP ports", &di_NTCBS) ||
      uRunDialog()<=0)
    return;

  /* All OK. Copy to main tcp_config structure */
  t.tcpc_bsdurg = bsdurg;
  t.tcpc_NTCBS = NTCBS;

  if (stack_set_tcp_config(&t) != TRUE)
    wInfoMsg(szStackCommFailure);
  else
    wInfoMsg(szNeedReload);
}


/*******************************************************************************
***
*** Function         : PPPDialog
*** Preconditions    : 
*** Postconditions   : 
***
*******************************************************************************/

GLDEF_C VOID PPPDialog(VOID)
{
H_DI_NUMBER di_r_timer;
LONG r_timer;

UWORD IP_Header;

UWORD Auth;

struct ppp_config pppc;
struct netif netif;

  if (stack_get_netif(&netif) != TRUE) {
    wInfoMsg(szStackCommFailure);
    return;
  }

  if (netif.ni_state != NIS_DOWN) {
    wInfoMsg(szConnectionNotDown);
    return;
  }

  if (stack_get_ppp_config(&pppc) != TRUE) {
    wInfoMsg(szStackCommFailure);
    return;
  }

  if (netif.ni_dlp != NIDLP_PPP) {
    wInfoMsg(szPPPNotSelected);
    return;
  }

  /* Copy data in */
  di_r_timer.value=&r_timer;
  di_r_timer.low=5;
  di_r_timer.high=100;
  r_timer=(LONG)pppc.restart_timer;

  IP_Header=pppc.IP_Header_compression;
  Auth=pppc.Authentication;

  do { 

    /* Do the dialog */
    if (myuOpenDialog("PPP Configuration") ||
        uAddDialogItem(H_DIALOG_NUMBER,
            "Restart timer", &di_r_timer) ||
        uAddChoiceList("Compress IP Header", &IP_Header,
            szYes, szNo, NULL) ||
        uAddChoiceList("Authentication", &Auth,
            "None (login)", "PAP", "CHAP", NULL) ||
        uRunDialog()<=0)
      return;

    if (IP_Header == PPP_IPHDR_COMPRESSED)
      wInfoMsg("Compressed IP Header is not yet supported");
    if (Auth == PPP_AUTH_PAP)
      wInfoMsg("PAP is not yet supported");
    if (Auth == PPP_AUTH_CHAP)
      wInfoMsg("CHAP is not yet supported");
  }
  while (IP_Header == PPP_IPHDR_COMPRESSED || 
         Auth == PPP_AUTH_PAP || Auth == PPP_AUTH_CHAP);

  /* Copy back */
  pppc.restart_timer=(int)r_timer&0xFFFF;
  pppc.IP_Header_compression=IP_Header;
  pppc.Authentication=Auth;

  /* Don't think we need this? *
  if (stack_set_netif(&netif) != TRUE)
    wInfoMsg(szStackCommFailure);
  */

  if (stack_set_ppp_config(&pppc) != TRUE)
    wInfoMsg(szStackCommFailure);
}


/*******************************************************************************
***
*** Function         : Confirm
*** Preconditions    : 
*** Postconditions   : Present a "No/Yes" dialog
***
*******************************************************************************/

GLDEF_C INT Confirm(TEXT *pb)
{
  if (myuOpenDialog(pb))    /* title text supplied by caller */
    return(-1);
  if (uAddButtonList("No",-'n',"Yes",'y',NULL))
    return(-1);
  return(uRunDialog()!='y');   /* return TRUE iff user answered Yes */
}


